// ---------------------------------------------------------------------------
//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//  Change History:
//
// Date       Programmer   Defect    Description
// ----       ----------   ------    -----------
// 02-07-11   hthomann     132573.8  Added copyright statement
// 03-10-23   hthomann     177265    Added code to demonstrate the Partition service
// ---------------------------------------------------------------------------

package com.ibm.websphere.samples.workarea.companycontextclient;

import javax.naming.*;
import com.ibm.websphere.workarea.*;
import java.util.*;
import java.io.*;
import com.ibm.websphere.samples.workarea.companycontextejb.*;
import com.ibm.websphere.samples.workarea.companycontextlibrary.*;

/**
 * The J2EEClient drives WorkArea usage between the WebSphere J2EE
 * client container and the WebSphere EJB container. This client has exactly the
 * same appearance as the Servlet; therefore, users have only to
 * figure out how to use one client and can immediately use the other. That's why
 * the decision was made to use the JEditorFrame which can read and display html.
 * JEditorFrame has some limitations, however, which result in some non-traditional
 * code. First, JEditorFrame is buggy. Tags it doesn't understand result in unformatted
 * display with no feedback. Crashing isn't unheard of. More significantly, JEditorFrame
 * is perfectly happy to display buttons and the like, but there's no mechanism for registering
 * listeners for the events. Unlike traditional browsers, JEditorFrame doesn't automatically load
 * pages from url events; instead, it sends an event via the hyperLinkUpdate method; within that
 * method, the implementor can do whatever they want. Therefore, we attach 'urls' that simply
 * indicate internally what url was invoked; we then do the right thing locally, whether that be
 * to update the company or priority or company or to lookup the home, create an session instance,
 * and invoke the bean.
 * 
 * Just like the servlet, when we invoke the bean, we first begin a WorkArea and set into it 
 * the company and priority. We then call the bean without explicit parameters; the bean responds
 * with a string [] that corresponds to the bean trace. We embed that string [] with a client trace
 * into the html view and redraw.
 * @version 1.0
 */
public class J2EEClient {

  /**
   * How we find the bean home
   */
  private java.lang.String ejbJndiName = "java:comp/env/ejb/CompanyContext";
  /**
   * The company with which we will associate the request by populating the WorkArea
   */
  private Company company = Company.Main;
  /**
   * 
   * The priority with which we will associate the request by populating the WorkArea
   */
  private Priority priority = Priority.Silver;

  //make static so we only create this partition once.
  static UserWorkArea samplePartition = null;

  /**
   * Default constructor called in main.
   */
  public J2EEClient() {
  }

  private int inputReader () {
    PrintStream out = System.out;
    StringTokenizer st;
    int choice = 0;
    BufferedReader in = null;

    String inputEncoding = System.getProperty("ws.input.encoding");
    String osName = System.getProperty("os.name");

    if (inputEncoding !=null && osName != null &&
         (osName.equalsIgnoreCase("z/os") || osName.equalsIgnoreCase("os/390"))) {
        try {
            in = new BufferedReader(new InputStreamReader(System.in, inputEncoding));
        }
        catch (IOException ioe) {
            out.println("\nERROR-J2EEClient: Caught an exception when trying to create");
            out.println("  an input stream.  Going to end the sample.  Exception is:");
            out.println("    " + ioe);
            out.println("");
            System.exit(0);
        }
    }
    else {
        in = new BufferedReader(new InputStreamReader(System.in));    
    }

    out.println("\n\n:::::::::: Company: " + company + " - Priority: " + priority + " ::::::::::");

    out.println("\n\nChoose a Company:     or    Choose a Priority:");
    out.println("1 " + Company.Main + "           |   6 " + Priority.Platinum);
    out.println("2 " + Company.London_Development + "   |   7 " + Priority.Gold);
    out.println("3 " + Company.London_Sales + "         |   8 " + Priority.Silver);
    out.println("4 " + Company.NewYork_Development + " |   9 " + Priority.Bronze);
    out.println("5 " + Company.NewYork_Sales + "       |   10 " + Priority.Tin);
    
    out.println("                      or");
    out.println("11 Invoke the Bean");
    out.println("12 Print the Sample Description");
    out.println("13 Exit the Sample");
    
    while (choice < 1 || choice > 13) {
        try {
            out.print("Enter a choice (1-13) > ");
            //choice = in.read();
            st = new StringTokenizer(in.readLine());
            choice = Integer.parseInt(st.nextToken());
            if (choice < 1 || choice > 13) {
                out.println("The value: " + choice + " is not a valid choice!");
            }
        }
        catch (Throwable t){
            out.println("\nERROR-J2EEClient: The values entered was not in the");
            out.println("  choice range of (1-13).  Caught the following exception:");
            out.println("    " + t);
            out.println("");
        }
    }

    if (choice >= 1 && choice <= 5) {
        changeCompany(choice);
    }
    else if (choice >= 6 && choice <= 10) {
        changePriority(choice);
    }
    else if (choice == 11) {
        try{
            out.println("\n\n:::::::::::::::::::::Invoking the Bean:::::::::::::::::::::\n");
            if (!this.invoke()) {
              out.println("\nERROR-J2EEClient");
              out.println("  The sample was not successfully executed");
            }

            out.println("\n:::::::::::::::::::::Invoked the Bean!:::::::::::::::::::::\n");
        }
        catch (Exception e) {
          System.out.println("" + e);
          e.printStackTrace();
          System.exit(1);
        }
    }
    else if (choice == 12) {
        this.printDescripiton();
    }
    else if (choice == 13) {
        System.out.println("::::::::::::::::::::::::Exiting the WorkArea Sample:::::::::::::::::::::::::");
        System.exit(1);
    }

    return choice;
  }

  /**
   * The string will match either a Company or Priority static;
   * run a long and slow if/then/else against it. 
   * 
   * @param urlDesc
   */
  private void changeCompany (int choice) {
    if (choice == 2) {
      company = Company.London_Development;
    } else if (choice == 3) {
      company = Company.London_Sales;
    } else if (choice == 4) {
      company = Company.NewYork_Development;
    } else if (choice == 5) {
      company = Company.NewYork_Sales;
    } else {
      company = Company.Main;
    }    
  }

  private void changePriority (int choice) {
    if (choice == 7) {
        priority = Priority.Gold;
    } else if (choice == 8) {
        priority = Priority.Silver;
    } else if (choice == 9) {
        priority = Priority.Bronze;
    } else if (choice == 10) {
        priority = Priority.Tin;
    } else {
        priority = Priority.Platinum;
    }
  }

  private static void printDescripiton() {
      PrintStream out = System.out;
    
      out.println("\n");
      out.println("::::::::::::::::::::::::::Sample Description:::::::::::::::::::::::::::");
      out.println("  The intent of this sample is to verify the WorkArea service in both the");
      out.println("  J2EE client and the ejb environments.  When one of the five company");
      out.println("  locations and/or priorities are selected, the client will");
      out.println("    - store locally the chosen company reference, or");
      out.println("    - store locally the chosen priority reference");
      out.println("  Any combination of company and priority can be entered.  The heart of the");
      out.println("  sample is invoked when the user selects to invoke the bean.  When the user");
      out.println("  selects the option to invoke the bean the part");
      out.println("  of the sample that uses the WorkArea service is invoked and the client will");
      out.println("    - lookup the UserWorkArea Partition in naming and create the sample Partition.");
      out.println("    - begin a WorkArea on the UserWorkArea Partition and the sample Partition.");
      out.println("    - set the Company and Priority into both Partitions WorkArea.");
      out.println("    - lookup and invoke the Bean via the following exact signature:");
      out.println("        String [] test()");
      out.println("      The Bean will");
      out.println("        - attempt to set a value into the imported WorkArea in the");
      out.println("          UserWorkArea Partition.  This will fail because the UserWorkArea"); 
      out.println("          Partition is defined to not be bidirectional and therefore servers");
      out.println("          cannot write directly into an imported WorkArea; they must begin their");
      out.println("          own WorkArea.");  
      out.println("        - also try to write into the sample Partition if the sample Partition");
      out.println("          exists on the server.  This operation will succeed if the sample Partition");
      out.println("          exists and was configured with the \"Bidirectional\" property set to true.");
      out.println("          It will fail if it is configured with the \"Bidirectional\" property set to");
      out.println("          false for the same reasons the attempt failed on the UserWorkArea Partition.");
      out.println("        - correctly begin a WorkArea on the UserWorkArea Partition.");
      out.println("        - attempt to set a new company into the WorkArea on the UserWorkArea"); 
      out.println("          Partition and the sample Partition should the sample Partition exist.");
      out.println("          However this will cause an exception because the client set the company as");
      out.println("          a fixed/read only value.  This shows the ability to enforce a client-defined");
      out.println("          context via modes.");
      out.println("        - successfully change the priority and call another bean (BackendBean).");
      out.println("          The BackendBean will");
      out.println("            - begin a WorkArea on the UserWorkArea context and set some new context into");
      out.println("              it.  Since the UserWorkArea Partition is not, and cannot be, configured");
      out.println("              to be \"Bidirectional\", this newly added context will not propagate back");
      out.println("              to the caller bean or the client which originated this call.");
      out.println("            - operate on the sample Partition (if it exists) depending on what the");
      out.println("              sample Partition's \"Bidirectional\" property is set to:"); 
      out.println("              - if set to \"true\" the BackendBean will set new context into the");
      out.println("                sample Partition.  This context will propagate back to the caller");
      out.println("                bean and client.");
      out.println("              - if set to \"false\" the BackendBean will begin a new WorkArea on the");
      out.println("                sample Partition and set new context into it.  This context will not");
      out.println("                propagate back to the caller bean and client.");  
      out.println("                - In both cases verify the context that is returned to the client by");
      out.println("                  viewing the key/value pair that are printed on the client.");
      out.println("            - get the company and priority from the WorkArea and process the request.");
      out.println("            - return to the caller bean.");
      out.println("        - complete the nested WorkArea on the UserWorkArea Partition.");
      out.println("        - display a message based upon the identify of the original client.");
      out.println("        - return to the client.");
      out.println("      - finally, the client will print the key/value pairs resident in the UserWorkArea");
      out.println("        Partition and sample Partition.");
      out.println("::::::::::::::::::::::::End of Sample Description:::::::::::::::::::::::::");
  }

  /**
   * 
   * @return 
   */
  private boolean invoke() {

    PrintStream out = System.out;
    boolean success = true;
    boolean toContinue = true;

    // this is our sample partition name
    String samplePartitionName = "SamplePartition";

    UserWorkArea userWorkArea = null;

    WorkAreaPartitionManager partitionManager = null;

    // try and retrieve the WorkArea reference
    InitialContext initialContext = null;
    try {
      initialContext = new InitialContext();
    } catch (Exception e) {
      out.println("ERROR-J2EEClient");
      out.println("  Could not make new InitialContext: ");
      out.println("    " + e);
      toContinue = false;
      success = false;
    }

    if (toContinue) {
      try {
        userWorkArea = (UserWorkArea) initialContext.lookup("java:comp/websphere/UserWorkArea");
      } catch (Exception e) {
        out.println("ERROR-J2EEClient");
        out.println("  Could not retrieve UserWorkArea.");
        out.println("    " + e);
        out.println("    Verify that admin server is running.");
        toContinue = false;
        success = false;
      }
    }

    if (toContinue) {
        //Lets get the WorkAreaPartitionManager implementation in order to 
        //create our sample partition.
        //NOTE: On the server, the sample partition MUST be created on the
        //admin console.  On a J2EE client, partitions are created progromatically.
        try {
          partitionManager = (WorkAreaPartitionManager) initialContext.lookup("java:comp/websphere/WorkAreaPartitionManager");
        } catch (Exception e) {
          out.println("ERROR-J2EEClient");
          out.println("  Could not retrieve the WorkAreaPartitionManager.");
          out.println("    " + e);
          out.println("    Verify that admin server is running.");
          toContinue = false;
          success = false;
        }
    
        if (samplePartition == null) {
            try{
                Properties props = new Properties();
                props.put("Bidirectional","true");
                samplePartition = partitionManager.createWorkAreaPartition(samplePartitionName,props);
            }
            catch(PartitionAlreadyExistsException e){
                out.println("J2EEClient-ERROR: sample tried to create the " + samplePartitionName + " Partition but"); 
                out.println("it already exists.");
                out.println("");
                success = false;
            }
            catch(java.lang.IllegalAccessException e){
                out.println("J2EEClient-ERROR: sample could not create the " + samplePartitionName + " Partition.");
                out.println("");
                success = false;
            }
        }
    }

    if (userWorkArea != null) {

      userWorkArea.begin("J2EEClient");

      try {
        userWorkArea.set("company", company, PropertyModeType.fixed_readonly);
        userWorkArea.set("priority", priority);
      } catch (NoWorkArea e) {
        out.println("ERROR-J2EEClient");
        out.println("  Could not begin WorkArea.");
        out.println("    " + e);
        success = false;
      } catch (NotOriginator e) {
        out.println("ERROR-J2EEClient");
        out.println("  Could not begin WorkArea.");
        out.println("    " + e);
        success = false;
      } catch (PropertyReadOnly e) {
        out.println("ERROR-J2EEClient");
        out.println("  Could not begin WorkArea.");
        out.println("    " + e);
        success = false;
      } catch (Exception e) {
        out.println("ERROR-J2EEClient");
        out.println("  Could not begin WorkArea.");
        out.println("    " + e);
        success = false;
      }
    }

    // if we have the sample partition, begin a new WorkArea and set our company and 
    // priority into it
    if (toContinue) {
      if (samplePartition != null) {
        samplePartition.begin("J2EEClient");

        try {
          samplePartition.set("company", company, PropertyModeType.fixed_readonly);
          samplePartition.set("priority", priority);
        } catch (NoWorkArea e) {
          out.println("ERROR-servlet:");
          out.println("  Could not set into Partition: " + samplePartitionName + ".");
          out.println("    " + e);
          success = false;
        } catch (NotOriginator e) {
          out.println("ERROR-servlet:");
          out.println("  Could not set into Partition: " + samplePartitionName + ".");
          out.println("    " + e);
          success = false;
        } catch (PropertyReadOnly e) {
          out.println("ERROR-servlet:");
          out.println("  Could not set into Partition: " + samplePartitionName + ".");
          out.println("    " + e);
          success = false;
        } catch (Exception e) {
          out.println("ERROR-servlet:");
          out.println("  Could not set into Partition: " + samplePartitionName + ".");
          out.println("    " + e);
          success = false;
        }
      }
    }

    // now lookup our bean
    Object o = null;
    if (toContinue) {
      try {
        o = initialContext.lookup(this.ejbJndiName);
      } catch (Exception e) {
        out.println("ERROR-J2EEClient");
        out.println("  Could not resolve bean home.");
        out.println("    " + e);
        out.println("    Verify that admin server is running.");
        toContinue = false;
        success = false;
      }
    }

    Bean bean = null;
    if (toContinue) {
      try {
        BeanHome home = 
        (BeanHome) javax.rmi.PortableRemoteObject.narrow(o,BeanHome.class); 
        bean = home.create();
      } catch (Exception e) {
        out.println("ERROR-J2EEClient");
        out.println("  Could not create bean.");
        out.println("    " + e);
        toContinue = false;
        success = false;
      }
    }

    // now invoke the bean
    if (toContinue) {
      try {
        String [] rs = bean.test();
        for (int i=0; i < rs.length; i++) {
          out.println(rs[i]);
        }
      } catch (Exception e) {
        out.println("ERROR-J2EEClient");
        out.println("Could not invoke bean.");
        out.println("    " + e);
        toContinue = false;
        success = false;
      }
    }

    // lets list the context after the remote call.  Context on the 
    // UserWorkArea Partition should NOT have propagated back to the client.
    // Context on the sample Partition SHOULD flow back if it is configured
    // to be bidirectional.
    if (toContinue) {
        String [] allkeys;
        String keyValuePairs = "";

        if (userWorkArea != null) {
            allkeys = userWorkArea.retrieveAllKeys();
            try{
                for (int i=0; i < allkeys.length; i++) {
                    keyValuePairs = keyValuePairs + "\"" + allkeys[i] + "\"/\"" + userWorkArea.get(allkeys[i]) + "\" ";
                }
                out.println("");
                out.println("J2EEClient: All key/value pairs on the UserWorkArea Partition:");
                out.println("       " + keyValuePairs);
            }
            catch(Exception e){
                out.println("Bean-ERROR:");
                out.println("  Could not get context from the UserWorkArea Partition.");
                out.println("    " + e);
                success = false;
            }
        }
    
        if (samplePartition != null) {
            allkeys = samplePartition.retrieveAllKeys();
            keyValuePairs = "";
            try{
                for (int i=0; i < allkeys.length; i++) {
                    keyValuePairs = keyValuePairs + "\"" + allkeys[i] + "\"/\"" + samplePartition.get(allkeys[i]) + "\" ";
                }
                out.println("J2EEClient: All key/value pairs on the " + samplePartitionName + " Partition:");
                out.println("       " + keyValuePairs);
            }
            catch(Exception e){
                out.println("Bean-ERROR:");
                out.println("  Could not get context from the " + samplePartitionName + " Partition.");
                out.println("    " + e);
                success = false;
            }
        }
    } 

    // now complete our WorkArea on the UserWorkArea Partition
    if (userWorkArea != null) {
      try {
        userWorkArea.complete();
      } catch (NoWorkArea e) {
        out.println("ERROR-J2EEClient");
        out.println("Could not complete the WorkArea on the UserWorkArea Partition.");
        out.println("    " + e);           
        out.println("    " + "Test complete.");
        success = false;
      } catch (NotOriginator e) {
        out.println("ERROR-J2EEClient");
        out.println("Could not complete the WorkArea on the UserWorkArea Partition.");
        out.println("    " + e);
        out.println("    " + "Test complete.");
        success = false;
      } catch (Exception e) {
        out.println("ERROR-J2EEClient");
        out.println("Could not complete the WorkArea on the UserWorkArea Partition.");
        out.println("    " + e);
        out.println("    " + "Test complete.");
        success = false;
      }
    } 

    // now complete our WorkArea on the sample Partition
    if (samplePartition != null) {
      try {
        samplePartition.complete();
      } catch (NoWorkArea e) {
        out.println("ERROR-J2EEClient");
        out.println("Could not complete the WorkArea on the " + samplePartitionName + " Partition.");
        out.println("    " + e);           
        out.println("    " + "Test complete.");
        success = false;
      } catch (NotOriginator e) {
        out.println("ERROR-J2EEClient");
        out.println("Could not complete the WorkArea on the " + samplePartitionName + " Partition.");
        out.println("    " + e);
        out.println("    " + "Test complete.");
        success = false;
      } catch (Exception e) {
        out.println("ERROR-J2EEClient");
        out.println("Could not complete the WorkArea on the " + samplePartitionName + " Partition.");
        out.println("    " + e);
        out.println("    " + "Test complete.");
        success = false;
      }
    } 

    return success;
  }

  public static void main(String [] args) {
    J2EEClient client = new J2EEClient();
//    printDescripiton();
    System.out.println("\n\n\n\n\n\n\n\n");
    System.out.println("::::::::::::::::::::::::::::WorkArea Sample::::::::::::::::::::::::::::");

    while(true) client.inputReader();
  }

}
