// ---------------------------------------------------------------------------
//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//  Change History:
//
// Date       Programmer   Defect    Description
// ----       ----------   ------    -----------
// 02-07-11   hthomann     132573.8  Added copyright statement
// ---------------------------------------------------------------------------

package com.ibm.websphere.samples.workarea.companycontextejb;

import javax.ejb.EJBException;
import javax.ejb.CreateException;
import javax.ejb.*;
import javax.naming.*;
import com.ibm.websphere.workarea.*;
import java.util.*;
import com.ibm.websphere.samples.workarea.companycontextlibrary.*;

/**
 * The Backend represents in the sample business function known to the Bean
 * but not necessarily to the originating client. In this case, because the Bean is the
 * client to the BackendBean, the SampleBean began a new nested WorkArea and overrode the
 * client's priority. This is the intended use of WorkArea nesting. This bean makes nominal use of the
 * WorkArea service; it reads the two keys directly (priority from the nested WorkArea, company from the
 * original WorkArea begun from the client) without being aware of the composition of the work areas, then
 * reads all keys to illustrate another method from the WorkArea api.
 * @version 1.0
 */
public class BackendBeanImpl implements SessionBean {
  private javax.ejb.SessionContext mySessionCtx = null;

/**
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
 * @exception EJBException 
 */
  public void ejbActivate() throws EJBException {}
/**
  * Framework methods. Do not use the WorkArea service in ejb framework methods!
 * @exception javax.ejb.CreateException 
 * @exception EJBException 
 */
  public void ejbCreate() throws CreateException, EJBException {}
/**
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
 * @exception EJBException 
 */
  public void ejbPassivate() throws EJBException {}
/**
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
 * @exception EJBException 
 */
  public void ejbRemove() throws EJBException {}
/**
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
 * @return javax.ejb.SessionContext
 */
  public javax.ejb.SessionContext getSessionContext() {
    return mySessionCtx;
  }
/**
  * Framework methods. Do not use the WorkArea service in ejb framework methods!
 * @param ctx javax.ejb.SessionContext
 * @exception EJBException 
 */
  public void setSessionContext(javax.ejb.SessionContext ctx) throws EJBException {
    mySessionCtx = ctx;
  }

/**
  * BackendBean's only remote method. Note that the method takes no parameters in order
  * to clearly demonstrate the WorkArea service's function. The method will merely retrieve
  * context from the WorkArea. Note that of the two parameters retrieved from the WorkArea, one
  * is defined in the outer scope and one in the inner; end users of the WorkArea (this bean) do
  * not have to worry about WorkArea context is composed, and the lookup is quite lightweight with
  * no declared exceptions.
  * The retrieveAllKeys method is used just to demonstrate one more method from the WorkArea api;
  * note that the method does recurse the stack of imported WorkAreas.
  * 
  * @return java.lang.String [] 
  *         Messages, essentially trace, are collected from each bean and returned as an array of strings
  *         to be displayed by the client.
 */
  public String [] test() {

    String [] toReturn = null;
    InitialContext jndiNaming = null;
    Vector messages = new Vector();
    boolean success = true;
    boolean toContinue = true;
    UserWorkArea userWorkArea = null;
    UserWorkArea samplePartition = null;
    WorkAreaPartitionManager partitionManager = null;
    Company company = null;
    Priority priority = null;
    String samplePartitionName = "SamplePartition";

    try {
      // try and retrieve the WorkArea reference
      try {
        jndiNaming = new InitialContext();
      } catch (Exception e) {
        messages.add("BackendBean-ERROR:");
        messages.add("  Could not make new InitialContext: ");
        messages.add("    " + e);
        toContinue = false;
        success = false;
      }

      if (toContinue) {
        try {
          userWorkArea = (UserWorkArea) jndiNaming.lookup("java:comp/websphere/UserWorkArea");
        } catch (Exception e) {
          messages.add("BackendBean-ERROR:");
          messages.add("  Could not retrieve UserWorkArea.");
          messages.add("    " + e);
          toContinue = false;
          success = false;
        }
      }

      if (toContinue) {
          //Lets get the WorkAreaPartitionManager in order to 
          //get our sample partition
          try {
            partitionManager = (WorkAreaPartitionManager) jndiNaming.lookup("java:comp/websphere/WorkAreaPartitionManager");
          } catch (Exception e) {
            messages.add("BackenBean-ERROR");
            messages.add("  Could not retrieve the WorkAreaPartitionManager.");
            messages.add("    " + e);
            toContinue = false;
            success = false;
          }
    
          try{
              samplePartition = partitionManager.getWorkAreaPartition(samplePartitionName);
          }
          catch(NoSuchPartitionException e){
              // If the sample partition doesn't exist, lets just go on
              // we already printed a message on the client.
          }

          if (samplePartition != null) {
              try{
                  messages.add("BackendBean: going to begin a WorkArea on the UserWorkArea");
                  messages.add("Partition and then set the key/value pair \"key2\"/\"value2\" into it.");
                  userWorkArea.begin("BackendBean");
                  userWorkArea.set("key2","value2");
              }
              catch(Exception e){
                  messages.add("BackendBean-ERROR:");
                  messages.add("  Problems with the begin and set on the UserWorkArea Partition: ");
                  messages.add("    " + e);
                  success = false;
              }
          }

    
          if (samplePartition != null) {
              try{
                  messages.add("BackendBean: going to set the key/value pair \"key2\"/\"value2\" on the " + samplePartitionName + " Partition.");
                  samplePartition.set("key2","value2");
                  //will only be here if the sample Partition is configured to be Bidirectional
                  messages.add("BackendBean: set the key/value pair \"key2\"/\"value2\" on the " + samplePartitionName + " Partition.");
                  messages.add("Note that this operation is permitted since the " + samplePartitionName + " Partition");
                  messages.add("was configured to be Bidirectional.  Therefore this context will propagate back to the caller.");
              }
              catch (NotOriginator e) {
                // The user will be instructed to create the sample partition with bidirectional=true
                // but just in case they don't, lets catch the NotOriginator exception and
                // handle it nicely
                messages.add("BackendBean: tried to write to an imported WorkArea on the " + samplePartitionName); 
                messages.add("Partition but was not permitted.  The " + samplePartitionName + " Partition");
                messages.add("is not configured for bidirectional propagation of its context");
                messages.add("which makes this operation illegal.");
              }
              catch(Exception e){
                  messages.add("BackendBean-ERROR:");
                  messages.add("  Could not set into the " + samplePartitionName + " Partition.");
                  messages.add("    " + e);
                  success = false;
              }
          }
      }

      if (toContinue) {
        if (userWorkArea != null) {
          company = (Company) userWorkArea.get("company");
          priority = (Priority) userWorkArea.get("priority");
        }
      }

      if (company != null) {
        String [] allkeys = userWorkArea.retrieveAllKeys();
        String keyValuePairs = "";
        try{
            for (int i=0; i < allkeys.length; i++) {
              keyValuePairs = keyValuePairs + "\"" + allkeys[i] + "\"/\"" + userWorkArea.get(allkeys[i]) + "\" ";
            }
        }
        catch(Exception e){
            messages.add("BackendBean-ERROR:");
            messages.add("  Could not get context from the UserWorkArea Partition.");
            messages.add("    " + e);
            success = false;
        }

        messages.add("");
        messages.add("BackendBean invoked by " + company + " ");
        messages.add("with priority of " + priority + "; processing successful.");
        messages.add("");
        messages.add("BackendBean: All key/value pairs on the UserWorkArea Partition:");
        messages.add("       " + keyValuePairs);

        if (samplePartition != null) {
            allkeys = samplePartition.retrieveAllKeys();
            keyValuePairs = "";
            try{
                for (int i=0; i < allkeys.length; i++) {
                   keyValuePairs = keyValuePairs + "\"" + allkeys[i] + "\"/\"" + samplePartition.get(allkeys[i]) + "\" ";
                }
                messages.add("BackendBean: All key/value pairs on the " + samplePartitionName + " Partition:");
                messages.add("       " + keyValuePairs);
            }
            catch(Exception e){
                messages.add("BackendBean-ERROR:");
                messages.add("  Could not get context from the " + samplePartitionName + " Partition.");
                messages.add("    " + e);
                success = false;
            }
        }
      }
      else {
        messages.add("BackendBean-ERROR:");
        messages.add("  WorkArea context non-existent,");
        messages.add("  Operation not permitted without a defined office.");
        success = false;
      }
    }
    catch (Exception e) {
      messages.add("BackendBean-ERROR:");
      messages.add("  Unexpected exception: ");
      messages.add("    " + e);
      success = false;
    }

    if (!success) {
      messages.add("BackendBean-ERROR:");
      messages.add("  The sample was not successfully executed.");
    }


    messages.add("BackendBean: Returning to caller.");
    toReturn = new String[messages.size()];

    for (int ii=0; ii < messages.size(); ii++) {
      toReturn[ii] = (String) messages.elementAt(ii);
    }

    return toReturn;
  }
}
