// ---------------------------------------------------------------------------
//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//  Change History:
//
// Date       Programmer   Defect    Description
// ----       ----------   ------    -----------
// 02-07-11   hthomann     132573.8  Added copyright statement
// ---------------------------------------------------------------------------

package com.ibm.websphere.samples.workarea.companycontextejb;

import javax.ejb.EJBException;
import javax.ejb.CreateException;
import javax.ejb.*;
import javax.naming.*;
import com.ibm.websphere.workarea.*;
import java.util.*;
import java.io.Serializable;
import com.ibm.websphere.samples.workarea.companycontextlibrary.*;

/**
 * The Bean demonstrates simple usage of the WorkArea service.
 * The Bean will do the following on its single remote method :
 *      - attempt to set a value into the imported WorkArea
 *            This will fail. Remote processes cannot write directly into
 *            imported WorkAreas. They must begin another one and delta on top of
 *            the imported WorkArea.
 *      - begin a new WorkArea
 *      - attempt to override the Company set by the client
 *            This will fail. Values that were set as read_only cannot be overridden.
 *      - successfuly override the Priority set by the client (will set to Tin)
 *      - invoke the BackendBean in the context of the Bean
 *      - complete the Company created in the Bean
 *      - look up the imported Company and Priority and compose a return message
 * @version 1.0
 */
public class BeanImpl implements SessionBean {
  /**
   */
  private javax.ejb.SessionContext mySessionCtx = null;

  /**
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
   * 
   * @exception EJBException
   */
  public void ejbActivate() throws EJBException {}
  /**
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
   * 
   * @exception javax.ejb.CreateException
   * @exception EJBException
   * @exception CreateException
   */
  public void ejbCreate() throws CreateException, EJBException {}
  /**
   * 
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
   * 
   * @exception EJBException
   */
  public void ejbPassivate() throws EJBException {}
  /**
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
   * 
   * @exception EJBException
   */
  public void ejbRemove() throws EJBException {}
  /**
   * 
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
   * 
   * @return javax.ejb.SessionContext
   */
  public javax.ejb.SessionContext getSessionContext() {
    return mySessionCtx;
  }
  /**
   * 
   * Framework methods. Do not use the WorkArea service in ejb framework methods!
   * 
   * @param ctx    javax.ejb.SessionContext
   * @exception EJBException
   */
  public void setSessionContext(javax.ejb.SessionContext ctx) throws EJBException {
    mySessionCtx = ctx;
  }

  /**
   * Bean's only remote method. Note that the method takes no parameters in order
   * to clearly demonstrate the WorkArea service's function. The method will invoke a number
   * of methods on the WorkArea context, demonstrating both how and how NOT to use the WorkArea
   * service. 
   * 
   * @return java.lang.String [] 
   *         Messages, essentially trace, are collected from each bean and returned as an array of strings
   *         to be displayed by the client.
   */
  public String [] test() {

    String [] toReturn = null;
    InitialContext jndiNaming = null;
    Vector messages = new Vector();
    boolean success = true;
    boolean toContinue = true;
    boolean isBidirectional = true;
    UserWorkArea userWorkArea = null;
    UserWorkArea samplePartition = null;
    WorkAreaPartitionManager partitionManager = null;
    Company company = null;
    Priority priority = null;
    String ejbJndiName = "java:comp/env/ejb/CompanyContextBackend";
    String samplePartitionName = "SamplePartition";
    BackendBean bean = null;

    try {
      // try and retrieve the WorkArea reference
      try {
        jndiNaming = new InitialContext();
      } catch (Exception e) {
        messages.add("Bean-ERROR:");
        messages.add("  Could not make new InitialContext: ");
        messages.add("    " + e);
        toContinue = false;
        success = false;
      }

      if (toContinue) {
        try {
          userWorkArea = (UserWorkArea) jndiNaming.lookup("java:comp/websphere/UserWorkArea");
        } catch (Exception e) {
          messages.add("Bean-ERROR:");
          messages.add("  Could not retrieve UserWorkArea.");
          messages.add("    " + e);
          toContinue = false;
          success = false;
        }
      }

      if (toContinue) {
          //Lets get the WorkAreaPartitionManager in order to 
          //get our sample partition
          try {
            partitionManager = (WorkAreaPartitionManager) jndiNaming.lookup("java:comp/websphere/WorkAreaPartitionManager");
          } catch (Exception e) {
            messages.add("Bean-ERROR");
            messages.add("  Could not retrieve the WorkAreaPartitionManager.");
            messages.add("    " + e);
            toContinue = false;
            success = false;
          }
    
          try{
              samplePartition = partitionManager.getWorkAreaPartition(samplePartitionName);
          }
          catch(NoSuchPartitionException e){
              // If the sample partition doesn't exist, lets just go on
              // we already printed a message on the client.
          }
      }

      if (toContinue) {
        if (userWorkArea != null) {
          String invoker = userWorkArea.getName();
          messages.add("Bean invoked by " + invoker);

          // now let's intentionally fail by trying to
          // write directly to the imported WorkArea.
          // Since the UserWorkArea is not configured to be bidirectional
          // one can't set context into a WorkArea.  It will be shown
          // below that a person can set context into the sample Partition
          // since it is configured to be bidirectional

          try {
            userWorkArea.set("key1", "value1");
          }
          catch (NotOriginator e) {
            messages.add("Bean: tried to write to an imported WorkArea on the UserWorkArea"); 
            messages.add("Partition but was not permitted.  The UserWorkArea Partition");
            messages.add("is not configured for bidirectional propagation of its context");
            messages.add("which makes this operation illegal.");
            messages.add("");
          }
        }

        // Now let's write directly to the imported WorkArea on the sample partition.
        // If the sample partition is configured with bidirectional=true then this is
        // a legal operation and the added context will flow back to the client.
        if (samplePartition != null) {
            try {
              samplePartition.set("key1", "value1");
              // The sample Partition should be configured with bidirectional propagation
              messages.add("Bean: tried to write to an imported WorkArea on the " + samplePartitionName + " Partition"); 
              messages.add("and was successful, it set the key/value pair of \"key1\"/\"value1\".");
              messages.add("The " + samplePartitionName + " Partition is configured for bidirectional");
              messages.add("propagation of its context which makes this operation legal.  This contex");
              messages.add("will propagate to the BackendBean and will flow back to the client since");
              messages.add("the " + samplePartitionName + " is configured to be Bidirectional.");
              messages.add("");
            }
            catch (NotOriginator e) {
                // The user will be instructed to create the sample partition with bidirectional=true
                // but just in case they don't, lets catch the NotOriginator exception and
                // handle it nicely
                messages.add("Bean tried to write to an imported WorkArea on the " + samplePartitionName); 
                messages.add("Partition but was not permitted.  The " + samplePartitionName + " is");
                messages.add("not configured for bidirectional propagation of its context");
                messages.add("which makes this operation illegal.");
                messages.add("");
                isBidirectional = false;
            }
        }
      }

      if (toContinue) {
          try{
            if (userWorkArea != null) {
              messages.add("Bean: going to begin a WorkArea on the UserWorkArea");
              messages.add("Partition and then set the key/value pair \"key1\"/\"value1\".");
              userWorkArea.begin("Bean");
              userWorkArea.set("key1","value1");
            }
          }
          catch(Exception e){
              messages.add("Bean-ERROR:");
              messages.add("  Problems with the begin and set on the UserWorkArea Partition: ");
              messages.add("    " + e);
              success = false;
          }

          //if the sample Partition isn't configured to be Bidirectional, lets begin
          //a new WorkArea and set context into it.  Note that it is legal to set
          //context into a WorkArea that was begun by another process when the Partition
          //is configured to be Bidirectional.  That context will then flow back to the
          //originating process.  Therefore lets not begine a new WorkArea if the sample
          //Partition is configured to be Bidirectional in order to demonstrate that context
          //propagates in both directions when when configured to be Bidirectional.  If a 
          //partition is NOT configured to be Bidirectional then a user must beging a new 
          //WorkArea before they can set context into that Partition.  This of course scopes
          //the WorkArea to this process (bean) and therefore the context will not flow back 
          //to the originating process (client).
          if (samplePartition != null && !isBidirectional) {
            try{
               messages.add("Bean: going to begin a WorkArea on the " + samplePartitionName);
               messages.add("Partition and then set the key/value pair \"key1\"/\"value1\".");
               samplePartition.begin("Bean");
               samplePartition.set("key1","value1");
            }
            catch(Exception e){
                messages.add("Bean-ERROR:");
                messages.add("  Problems with the begin and set on the " + samplePartitionName + " Partition: ");
                messages.add("    " + e);
                success = false;
            }
          }
      }

      if (toContinue) {
        if (userWorkArea != null) {
          company = (Company) userWorkArea.get("company");
          priority = (Priority) userWorkArea.get("priority");
        }
      }

      // update company
      Company newCompany = null;
      if (company != null) {
        int i = company.toInt();
        i = (++i) % 5;
        newCompany = Company.fromInt(i);
      }

      if (toContinue) {
        if (userWorkArea != null) {
          if (newCompany != null) {
            try {
              // We are fully aware that this will fail since the
              // client set the company as fixed but we are trying
              // to demonstrate the ability to enforce a client-defined
              // context via modes 

              userWorkArea.set("company", newCompany);
            } 
            catch (PropertyReadOnly e) {
                //we expect to be here.  Now, if the sample partition exists, lets
                //try the same set on it which should fail as well.  If it fails
                //lets print a message stating that both partitions failed.
                if (samplePartition != null) {
                    try {
                        samplePartition.set("company",newCompany);
                    }
                    catch (PropertyReadOnly ex){
                        messages.add("");
                        messages.add("Bean: Caught an expected exception when trying to change");
                        messages.add("the company on the server to " + newCompany + " on both the");
                        messages.add("UserWorkArea Partition and the " + samplePartitionName + " Partition.");
                        messages.add("Company was set as fixed/read only on the client.");
                    }
                    catch (Exception ex){
                        messages.add("Bean-ERROR:");
                        messages.add("  Could not set into the " + samplePartitionName + " Partition.");
                        messages.add("    " + ex);
                        success = false;
                    }
                }
                //the sample partition doesn't exist so lets just print the message
                //for the UserWorkArea Partition.
                else{
                    messages.add("");
                    messages.add("Bean: Caught an expected exception when trying to change");
                    messages.add("the company on the server to " + newCompany + " on the");
                    messages.add("UserWorkArea Partition.  Company was set as fixed/read only"); 
                    messages.add("on the client.");
                }
            }
            catch (Exception e) {
                messages.add("Bean-ERROR:");
                messages.add("  Could not set into the UserWorkArea Partition.");
                messages.add("    " + e);
                success = false;
            }
          }
        }
      }

      if (toContinue) {
        if (userWorkArea != null) {
           try{
              messages.add("");
              messages.add("Bean: Going to change the priority of " + priority + " set by");
              messages.add("the client on the UserWorkArea Partition, to a priority of " + Priority.Tin + ".");
              userWorkArea.set("priority", Priority.Tin);
           }
           catch (Exception e) {
               messages.add("Bean-ERROR:");
               messages.add("  Could not set into the UserWorkArea Partition");
               messages.add("    " + e);
               success = false;
           }
        }

        if (samplePartition != null) {
           try{
              messages.add("Bean: Going to change the priority of " + priority + " set by");
              messages.add("the client on the " + samplePartitionName + " Partition, to a priority of " + Priority.Tin + ".");
              samplePartition.set("priority", Priority.Tin);
           }
           catch (Exception e) {
               messages.add("Bean-ERROR:");
               messages.add("  Could not set into the " + samplePartitionName + " Partition");
               messages.add("    " + e);
               success = false;
           }
        }
      }

      if (toContinue) {
        if (userWorkArea != null) {
           try{
              messages.add("");
              messages.add("Bean: Going to call BackendBean.");
              messages.add("");

              Object o = jndiNaming.lookup(ejbJndiName);
              BackendBeanHome home = 
                  (BackendBeanHome) javax.rmi.PortableRemoteObject.narrow(o,BackendBeanHome.class); 
              bean = home.create();

              String [] rs = bean.test();
              for (int i=0; i < rs.length; i++) {
                messages.add(rs[i]);
              }
           }
           catch (Exception e) {
           messages.add("Bean-ERROR:");
           messages.add("  Could not call BackendBean.");
           messages.add("    " + e);
           success = false;
         }
        }
      }
    }
    catch (Exception e) {
      messages.add("Bean-ERROR:");
      messages.add("  Unexpected exception: ");
      messages.add("    " + e);
      success = false;
    }

    if (toContinue) {
        if (company != null) {
            messages.add("");
          if (company.equals(Company.Main)) {
            messages.add("Bean: Operation permitted for the main office.");
            messages.addElement("Bean invoked by the Main office with");
            messages.addElement("priority of " + priority + "; processing successful.");
          }
          else if (company.equals(Company.London_Sales )) {
            messages.add("Bean: Operation not permitted for the London sales office.");
            messages.addElement("Bean invoked by the London Sales office with");
            messages.addElement("priority of " + priority + "; processing successful.");
          }
          else if (company.equals(Company.NewYork_Sales )) {
            messages.add("Bean: Operation not permitted for the New York sales office.");
            messages.addElement("Bean invoked by the New York sales office with");
            messages.addElement("priority of " + priority + "; processing successful.");
          }
          else if (company.equals(Company.London_Development )) {
            messages.add("Bean: Operation permitted for the London development office.");
            messages.addElement("Bean invoked by the London development office with");
            messages.addElement("priority of " + priority + "; processing successful.");
          }
          else if (company.equals(Company.NewYork_Development )) {
            messages.add("Bean: Operation permitted for the New York development office.");
            messages.addElement("Bean invoked by the New York development office with");
            messages.addElement("priority of " + priority + "; processing successful.");
          }

          String [] allkeys = userWorkArea.retrieveAllKeys();
          String keyValuePairs = "";
          try{
              for (int i=0; i < allkeys.length; i++) {
                  keyValuePairs = keyValuePairs + "\"" + allkeys[i] + "\"/\"" + userWorkArea.get(allkeys[i]) + "\" ";
              }

              messages.add("");
              messages.add("Bean: All key/value pairs on the UserWorkArea Partition:");
              messages.add("       " + keyValuePairs);
          }
          catch(Exception e){
              messages.add("Bean-ERROR:");
              messages.add("  Could not get context from the UserWorkArea Partition.");
              messages.add("    " + e);
              success = false;
          }

          if (samplePartition != null) {
              allkeys = samplePartition.retrieveAllKeys();
              keyValuePairs = "";
              try{
                  for (int i=0; i < allkeys.length; i++) {
                     keyValuePairs = keyValuePairs + "\"" + allkeys[i] + "\"/\"" + samplePartition.get(allkeys[i]) + "\" ";
                  }
                  messages.add("Bean: All key/value pairs on the " + samplePartitionName + " Partition:");
                  messages.add("       " + keyValuePairs);
              }
              catch(Exception e){
                  messages.add("Bean-ERROR:");
                  messages.add("  Could not get context from the " + samplePartitionName + " Partition.");
                  messages.add("    " + e);
                  success = false;
              }
          }
        }
        else {
          messages.add("Bean-ERROR:");
          messages.add("  WorkArea context non-existent,");
          messages.add("  Operation not permitted without a defined office.");
          success = false;
        }
    }

    if (toContinue) {
      if (userWorkArea != null) {
        try {
          userWorkArea.complete();
        } catch (Exception e) {
          messages.add("Bean-ERROR:");
          messages.add("  Could not complete WorkArea.");
          messages.add("    " + e);
          success = false;
        }
      }
    }

    if (!success) {
      messages.add("Bean-ERROR:");
      messages.add("  The sample was not successfully executed.");
    }

    messages.add("");
    messages.add("Bean: Returning to caller.");
    toReturn = new String[messages.size()];

    for (int ii=0; ii < messages.size(); ii++) {
      toReturn[ii] = (String) messages.elementAt(ii);
    }

    return toReturn;
  }
}
