// ---------------------------------------------------------------------------
//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//  Change History:
//
// Date       Programmer   Defect    Description
// ----       ----------   ------    -----------
// 02-07-11   hthomann     132573.8  Added copyright statement
// ---------------------------------------------------------------------------

package com.ibm.websphere.samples.workarea.companycontextwar;
/**
 * Servlet to access the Increment Enterprise Java Bean.
 */

// General enterprise bean code.
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.FinderException;
import javax.ejb.DuplicateKeyException;


// Servlet related.
import javax.servlet.*;
import javax.servlet.http.*;

// JNDI (naming) for Context, InitialContext, NamingException
import javax.naming.*;  

// Miscellaneous:
import java.util.*;
import java.io.*;

import com.ibm.websphere.workarea.*;
import com.ibm.websphere.samples.workarea.companycontextejb.*;
import com.ibm.websphere.samples.workarea.companycontextlibrary.*;


public class Servlet extends HttpServlet {

  public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
  {

    // the messages will be used to collect messages to return to the client
    Vector messages = new Vector();

    // fatal error? don't do any more work
    boolean toContinue = true;

    // we may continue, but can't call this a complete success
    boolean success = true;

    Company company = null;
    Priority priority = null;
    String submit = null;

    // we want our OWN url so that we can set the urls on the composed html page
    // to set company and priority
    StringBuffer urlSB = req.getRequestURL();
    String url = urlSB.toString();

    // this is were we look up the bean from
    String ejbJndiName = "java:comp/env/ejb/CompanyContext";
    
    // this is our sample partition name
    String samplePartitionName = "SamplePartition";

    UserWorkArea userWorkArea = null;
    UserWorkArea samplePartition = null;
  
    WorkAreaPartitionManager partitionManager = null;

    try {

      Bean bean = null;

      InitialContext jndi = null;

      javax.servlet.http.HttpSession session = req.getSession(true);

      // first, get our submit parameters; there are three possible
      submit = req.getParameter("submit");
      String companysint = req.getParameter("setCompany");
      String prioritysint = req.getParameter("setPriority");

      if (submit == null && companysint == null && prioritysint == null) {
        // add no messages; we'll display the page without results
        toContinue = false;
      }

      // PROCESS COMPANY

      // If we are changing the company, do so here.
      if (companysint != null) {
        Company sc = null;
        int x;
        try {
          x = Integer.parseInt(companysint);
          company = Company.fromInt(x);
        } catch (NumberFormatException e) {
          // we'll catch the error by checking company
        }

        // company will be null if out of range or meaningless
        if (company == null) {
          messages.add("ERROR-servlet:");
          messages.add("  Parameter for company change invalid: ");
          messages.add("    Parameter must be in [0..4]");
          success = false; 
        } else {
          // set into session
          try {
            session.setAttribute("company", company);
          } catch (IllegalStateException e) {
            messages.add("ERROR-servlet:");
            messages.add("  IllegalStateException accessing session: ");
            messages.add("    " + e);
            toContinue = false;
            success = false;
          }
        }
      }

      // if our company is null, look it up or create it;
      if (company == null) {
        // if we haven't been told to change our company, get it from our session.
        // if we don't have it in the session, set it to the default
        try {
          company = (Company) session.getAttribute("company");
        } catch (IllegalStateException e) {
          messages.add("ERROR-servlet:");
          messages.add("  IllegalStateException accessing session: ");
          messages.add("    " + e);
          toContinue = false;
          success = false;
        }
        if (company == null) {
          try {
            company = Company.Main;
            session.setAttribute("company", company);
          } catch (IllegalStateException e) {
            messages.add("ERROR-servlet:");
            messages.add("  IllegalStateException accessing session: ");
            messages.add("    " + e);
            toContinue = false;
            success = false;
          }
        }
      }

      // PROCESS PRIORITY

      // If we are changing the priority, do so here.
      if (prioritysint != null) {
        Priority sp = null;
        int x;
        try {
          x = Integer.parseInt(prioritysint);
          priority = Priority.fromInt(x);
        } catch (NumberFormatException e) {
          // we'll catch the error by checking priority
        }

        // priority will be null if out of range or meaningless
        if (priority == null) {
          messages.add("ERROR-servlet:");
          messages.add("  Parameter for priority change invalid: ");
          messages.add("    Parameter must be in [0..4]");
          success = false; 
        } else {
          // set into session
          try {
            session.setAttribute("priority", priority);
          } catch (IllegalStateException e) {
            messages.add("ERROR-servlet:");
            messages.add("  IllegalStateException accessing session: ");
            messages.add("    " + e);
            toContinue = false;
            success = false;
          }
        }
      }

      // if our priority is null, look it up or create it;
      if (priority == null) {
        // if we haven't been told to change our priority, get it from our session.
        // if we don't have it in the session, set it to the default
        try {
          priority = (Priority) session.getAttribute("priority");
        } catch (IllegalStateException e) {
          messages.add("ERROR-servlet:");
          messages.add("  IllegalStateException accessing session: ");
          messages.add("    " + e);
          toContinue = false;
          success = false;
        }
        if (priority == null) {
          try {
            priority = Priority.Silver;
            session.setAttribute("priority", priority);
          } catch (IllegalStateException e) {
            messages.add("ERROR-servlet:");
            messages.add("  IllegalStateException accessing session: ");
            messages.add("    " + e);
            toContinue = false;
            success = false;
          }
        }
      }

      // If we are going to invoke the bean, do so here.
      if (toContinue) {
        if (submit != null) {
          if (submit.equals("invoke")) {
            // first, get our initialcontext
            try {
              jndi = new InitialContext();
            } catch (Exception e) {
              messages.add("ERROR-servlet:");
              messages.add("  Could not make new InitialContext: ");
              messages.add("    " + e);
              toContinue = false;
              success = false;
            }
            // now get the UserWorkArea
            if (toContinue) {
              try {
                userWorkArea = (UserWorkArea) jndi.lookup("java:comp/websphere/UserWorkArea");
              } catch (Exception e) {
                messages.add("ERROR-servlet:");
                messages.add("  Could not retrieve UserWorkArea");
                messages.add("    " + e);
                toContinue = false;
                success = false;
              }
            }

            if (toContinue) {
                //Lets get the WorkAreaPartitionManager in order to 
                //get our sample partition
                try {
                  partitionManager = (WorkAreaPartitionManager) jndi.lookup("java:comp/websphere/WorkAreaPartitionManager");
                } catch (Exception e) {
                  messages.add("ERROR-Servlet");
                  messages.add("  Could not retrieve the WorkAreaPartitionManager.");
                  messages.add("    " + e);
                  toContinue = false;
                  success = false;
                }
    
                try{
                    samplePartition = partitionManager.getWorkAreaPartition(samplePartitionName);
                }
                catch(NoSuchPartitionException e){
                    messages.add("Servlet: NOTE: the sample tried to get the " + samplePartitionName + " but it");
                    messages.add("was not found.  This sample will continue as normal and use the");
                    messages.add("UserWorkArea Partition only.  Please see the sample documentation");
                    messages.add("on how to create the " + samplePartitionName + " Partition if");
                    messages.add("a demonstartion on WorkArea Partitions is needed.");
                    messages.add("");
                }
            }

            // if we have the UserWorkArea Partition, begin a new WorkArea and set 
            // our company and priority into it
            if (toContinue) {
              if (userWorkArea != null) {
                userWorkArea.begin("Servlet");

                try {
                  userWorkArea.set("company", company, PropertyModeType.fixed_readonly);
                  userWorkArea.set("priority", priority);
                } catch (NoWorkArea e) {
                  messages.add("ERROR-servlet:");
                  messages.add("  Could not set into the UserWorkArea Partition.");
                  messages.add("    " + e);
                  success = false;
                } catch (NotOriginator e) {
                  messages.add("ERROR-servlet:");
                  messages.add("  Could not set into the UserWorkArea Partition.");
                  messages.add("    " + e);
                  success = false;
                } catch (PropertyReadOnly e) {
                  messages.add("ERROR-servlet:");
                  messages.add("  Could not set into the UserWorkArea Partition.");
                  messages.add("    " + e);
                  success = false;
                } catch (Exception e) {
                  messages.add("ERROR-servlet:");
                  messages.add("Could not set into the UserWorkArea Partition.");
                  messages.add("    " + e);
                  success = false;
                }
              }
            }

            // if we have the sample partition, begin a new WorkArea and set our company and 
            // priority into it
            if (toContinue) {
              if (samplePartition != null) {
                samplePartition.begin("Servlet");

                try {
                  samplePartition.set("company", company, PropertyModeType.fixed_readonly);
                  samplePartition.set("priority", priority);
                } catch (NoWorkArea e) {
                  messages.add("ERROR-servlet:");
                  messages.add("  Could not set into Partition: " + samplePartitionName + ".");
                  messages.add("    " + e);
                  success = false;
                } catch (NotOriginator e) {
                  messages.add("ERROR-servlet:");
                  messages.add("  Could not set into Partition: " + samplePartitionName + ".");
                  messages.add("    " + e);
                  success = false;
                } catch (PropertyReadOnly e) {
                  messages.add("ERROR-servlet:");
                  messages.add("  Could not set into Partition: " + samplePartitionName + ".");
                  messages.add("    " + e);
                  success = false;
                } catch (Exception e) {
                  messages.add("ERROR-servlet:");
                  messages.add("  Could not set into Partition: " + samplePartitionName + ".");
                  messages.add("    " + e);
                  success = false;
                }
              }
            }

            // now lookup our bean
            Object o = null;
            if (toContinue) {
              try {
                o = jndi.lookup(ejbJndiName);
                BeanHome home = 
                (BeanHome) javax.rmi.PortableRemoteObject.narrow(o,BeanHome.class); 
                bean = home.create();
              } catch (Exception e) {
                e.printStackTrace();
                messages.add("ERROR-servlet:");
                messages.add("  Could not resolve bean home.");
                messages.add("    " + e);
                toContinue = false;
                success = false;
              }
            }

            // now invoke the bean
            if (toContinue) {
              try {
                String [] rs = bean.test();
                for (int i=0; i < rs.length; i++) {
                  messages.add(rs[i]);
                }
              } catch (Exception e) {
                messages.add("ERROR-servlet:");
                messages.add("  Error invoking bean.");
                messages.add("    " + e);
                toContinue = false;
                success = false;
              }
            }

            // lets list the context after the remote call.  Context on the 
            // UserWorkArea Partition should NOT have propagated back to the client.
            // Context on the sample Partition SHOULD flow back if it is configured
            // to be bidirectional.
            if (toContinue) {
                String [] allkeys;
                String keyValuePairs = "";

                if (userWorkArea != null) {
                    allkeys = userWorkArea.retrieveAllKeys();
                    try{
                        for (int i=0; i < allkeys.length; i++) {
                            keyValuePairs = keyValuePairs + "\"" + allkeys[i] + "\"/\"" + userWorkArea.get(allkeys[i]) + "\" ";
                        }
                        messages.add("");
                        messages.add("Servlet: All key/value pairs on the UserWorkArea Partition:");
                        messages.add("       " + keyValuePairs);
                    }
                    catch(Exception e){
                        messages.add("Bean-ERROR:");
                        messages.add("  Could not get context from the UserWorkArea Partition.");
                        messages.add("    " + e);
                        success = false;
                    }
                }

                if (samplePartition != null) {
                    allkeys = samplePartition.retrieveAllKeys();
                    keyValuePairs = "";
                    try{
                        for (int i=0; i < allkeys.length; i++) {
                            keyValuePairs = keyValuePairs + "\"" + allkeys[i] + "\"/\"" + samplePartition.get(allkeys[i]) + "\" ";
                        }
                        messages.add("Servlet: All key/value pairs on the " + samplePartitionName + " Partition:");
                        messages.add("       " + keyValuePairs);
                    }
                    catch(Exception e){
                        messages.add("Bean-ERROR:");
                        messages.add("  Could not get context from the " + samplePartitionName + " Partition.");
                        messages.add("    " + e);
                        success = false;
                    }
                }
            } 

            // now complete our WorkArea on the UserWorkArea Partition
            if (userWorkArea != null) {
              try {
                userWorkArea.complete();
              } catch (NoWorkArea e) {
                messages.add("ERROR-servlet:");
                messages.add("Could not complete the WorkArea on the UserWorkArea Partition.");
                messages.add("    " + e);
                success = false;
              } catch (NotOriginator e) {
                messages.add("ERROR-servlet:");
                messages.add("Could not complete the WorkArea on the UserWorkArea Partition.");
                messages.add("    " + e);
                success = false;
              } catch (Exception e) {
                messages.add("ERROR-servlet:");
                messages.add("Could not complete the WorkArea on the UserWorkArea Partition.");
                messages.add("    " + e);
                success = false;
              }
            }

            // now complete our WorkArea on the sample Partition
            if (samplePartition != null) {
              try {
                samplePartition.complete();
              } catch (NoWorkArea e) {
                messages.add("ERROR-servlet:");
                messages.add("Could not complete the WorkArea on the " + samplePartitionName + " Partition.");
                messages.add("    " + e);
                success = false;
              } catch (NotOriginator e) {
                messages.add("ERROR-servlet:");
                messages.add("Could not complete the WorkArea on the " + samplePartitionName + " Partition.");
                messages.add("    " + e);
                success = false;
              } catch (Exception e) {
                messages.add("ERROR-servlet:");
                messages.add("Could not complete the WorkArea on the " + samplePartitionName + " Partition.");
                messages.add("    " + e);
                success = false;
              }
            }
          
          } else {
            messages.add("ERROR-servlet:");
            messages.add("  Invalid submit parameter:");
            messages.add("    Parameter must be: invoke");
          }
        }
      }

    } catch (Exception e) {
      messages.add("ERROR-servlet:");
      messages.add("  Unexpected exception:");
      messages.add("    " + e);
      messages.add(e.getMessage());
      e.printStackTrace();

    }

    if (!success) {
      messages.add("ERROR-servlet:");
      messages.add("  The sample was not successfully executed");
    }

    if (submit != null) {
      messages.add("");
      messages.add("Sample complete!");
    }

    // --- Prepare and send HTML response. ---
    res.setContentType("text/html");
    res.setHeader("Pragma", "no-cache");
    res.setHeader("Cache-control", "no-cache");
    PrintWriter out = res.getWriter();


    out.println("<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">");
    out.println("<html>");
    out.println("  <head>");
    out.println("     <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">");
    out.println("     <title>WorkArea Company Context</title>");
    out.println("  </head>");
    out.println("  <body>");

    out.println("<script language=\"JavaScript\" src=\"/WSsamples/scripts/browser_detection.js\"></script>");

    out.println("<NOSCRIPT>");
    out.println("Proper display of samples requires JavaScript to be enabled.");
    out.println("</NOSCRIPT>");
    out.println("  <img alt=\"Samples Gallery\" src=\"/WSsamples/images/success_banner.jpg\" valign=top>");

    out.println("  <table cellpadding=5 cellspacing=5>");
    out.println("    <tr><td><H1 class=\"sampjsp\">WorkArea Company Context</H1>");
    out.println("    </td></tr>");
    out.println("    </table>");
    
    out.println("  <hr>");
    out.println("<br>");
    out.println("<b>Current office and priority:</b>&nbsp; " + company + " - priority: " + priority);
    out.println("<br>");
    out.println("<br>");

    out.println("<b>Change location: &nbsp;</b>");
    out.println("      | <a href=" + url + "?setCompany=" + Company._Main + ">" + Company.Main + "</a>");
    out.println("      | <a href=" + url + "?setCompany=" + Company._NewYork_Development + ">" + Company.NewYork_Development + "</a>");
    out.println("      | <a href=" + url + "?setCompany=" + Company._NewYork_Sales + ">" + Company.NewYork_Sales + "</a>");
    out.println("      | <a href=" + url + "?setCompany=" + Company._London_Development + ">" + Company.London_Development + "</a>");
    out.println("      | <a href=" + url + "?setCompany=" + Company._London_Sales + ">" + Company.London_Sales + "</a>");
    out.println("      |");
    out.println("      <p>");

    out.println("<br>");
    out.println("<b>Change priority:&nbsp;</b>");
    out.println("      | <a href=" + url + "?setPriority=" + Priority._Platinum + ">" + Priority.Platinum + "</a>");
    out.println("      | <a href=" + url + "?setPriority=" + Priority._Gold + ">" + Priority.Gold + "</a>");
    out.println("      | <a href=" + url + "?setPriority=" + Priority._Silver + ">" + Priority.Silver + "</a>");
    out.println("      | <a href=" + url + "?setPriority=" + Priority._Bronze + ">" + Priority.Bronze + "</a>");
    out.println("      | <a href=" + url + "?setPriority=" + Priority._Tin + ">" + Priority.Tin + "</a>");
    out.println("      |");
    out.println("      <p>");
    out.println("      <br>");
      
    out.println("    <a href=" + url + "?submit=invoke>&nbsp;&nbsp;&nbsp; Invoke The Bean</a>");

    out.println("    <hr WIDTH=\"100%\">");

    out.println("    <code>");
    // output messages
    Iterator iterator = messages.iterator();
    while (iterator.hasNext()) {
      out.println( (String) iterator.next());
      out.println("<br>");
    }
    out.println("    </code>");

    out.println("    <hr WIDTH=\"100%\"><br>");
    out.println("    <br>The intent of this sample is to verify the WorkArea service in both the J2EE client and the ejb environments.");
    out.println("    <br>&nbsp;");
    out.println("    <p>When one of the five location or priority links are selected, the client will either");
    out.println("    <ul>");
    out.println("      <li>store locally the chosen company reference, or</li>");
    out.println("      <li>store locally the chosen priority reference</li>");
    out.println("    </ul>");
    out.println("    When the link to invoke the bean is selected, the client will");
    out.println("    <ul>");
    out.println("      <li>begin a WorkArea on the UserWorkArea Partition and the sample");
    out.println("          Partition if it exists</li>");
    out.println("      <li>set the Company and Priority into the WorkArea</li>");
    out.println("      <li>lookup and invoke the Bean via the following exact signature:</li>");
    out.println("      <blockquote>String [] test()</blockquote>");
//    out.println("    </ul>");
//    out.println("      <li>");
    out.println("        The Bean will");
    out.println("        <ul>");
    out.println("          <li>attempt to set a value into the imported WorkArea in the");
    out.println("              UserWorkArea Partition.  This will fail because the UserWorkArea"); 
    out.println("              Partition is defined to not be bidirectional and therefore servers");
    out.println("              cannot write directly into an imported WorkArea; they must begin their");
    out.println("              own WorkArea.</li>");  
    out.println("          <li>also try to write into the sample Partition if the sample Partition");
    out.println("              exists on the server.  This operation will succeed if the sample Partition");
    out.println("              exists and was configured with the \"Bidirectional\" property set to true.");
    out.println("              It will fail if it is configured with the \"Bidirectional\" property set to");
    out.println("              false for the same reasons the attempt failed on the UserWorkArea Partition.</li>");
    out.println("          <li>correctly begin a WorkArea on the UserWorkArea Partition.</li>");
    out.println("          <li>attempt to set a new company into the WorkArea on the UserWorkArea"); 
    out.println("              Partition and the sample Partition should the sample Partition exist.");
    out.println("              However this will cause an exception because the client set the company as");
    out.println("              a fixed/read only value.  This shows the ability to enforce a client-defined");
    out.println("              context via modes.</li>");
    out.println("          <li>successfully change the priority and call another bean (BackendBean).</li>");
    out.println("            The BackendBean will");
    out.println("            <ul>");
    out.println("              <li>begin a WorkArea on the UserWorkArea context and set some new context into");
    out.println("                  it.  Since the UserWorkArea Partition is not, and cannot be, configured");
    out.println("                  to be \"Bidirectional\", this newly added context will not propagate back");
    out.println("                  to the caller bean or the client which originated this call.</li>");
    out.println("              <li>operate on the sample Partition (if it exists) depending on what the");
    out.println("                  sample Partition's \"Bidirectional\" property is set to:"); 
    out.println("                  <ul>");
    out.println("                    <li>if set to \"true\" the BackendBean will set new context into the");
    out.println("                        sample Partition.  This context will propagate back to the caller");
    out.println("                        bean and client.</li>");
    out.println("                    <li>if set to \"false\" the BackendBean will begin a new WorkArea on the");
    out.println("                        sample Partition and set new context into it.  This context will not");
    out.println("                        propagate back to the caller bean and client.</li>");  
    out.println("                    <li>In both cases verify the context that is returned to the client by");
    out.println("                        viewing the key/value pair that are printed on the client.</li>");
    out.println("                  </ul>");
    out.println("              <li>get the company and priority from the WorkArea and process the request.</li>");
    out.println("              <li>return to the caller bean.</li>");
    out.println("            </ul>");
    out.println("          <li>complete the nested WorkArea on the UserWorkArea Partition.</li>");
    out.println("          <li>display a message based upon the identify of the original client.</li>");
    out.println("          <li>return to the client.</li>");
    out.println("        </ul>");
    out.println("      <li>finally, the client will print the key/value pairs resident in the UserWorkArea");
    out.println("          Partition and, if it exists, the sample Partition.</li>");
    out.println("    </ul>");
    out.println("  <br>&nbsp;");
    out.println("  </body>");
    out.println("</html>");
  }
}
