//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.appprofile.accountmanagementejb;

import java.rmi.RemoteException;
import java.util.Properties;
import javax.ejb.*;
import javax.ejb.EJBException;

/**
 * The Account CMP EJB 2.0 Bean implements the EntityBean interface. 
 * It contains the buisness methods necessary to work with an Account. 
 * In addition to buisness methonds, it contains the Home Interface Methods, and Callback Methods.
 *
 * @author Cheng-chieh (Jerry) Cheng
 * @version 1.0
 * @since 04/15/2002
 */
public abstract class AccountBean implements EntityBean {
    /**
     * This abstract CMP field accessor method returns the accountid attribute for an Account.
     *
     * @return The accountid attribute of the Account
    */ 
    public abstract String getAccountid();                             
    /**
     * This abstract CMP field accessor method set the accountid attribute for an Account.
     * 
     * @param accountid String object used as primary key
     */
    public abstract void setAccountid(String accountid);               
    /**
     * This abstract CMP field accessor method returns the type attribute for an Account.
     *
     * @return The type attribute of the Account 
    */ 
    public abstract int getType();                             
    /**
     * This abstract CMP field accessor method set the type attribute for an Account.
     * 
     * @param type set the type attribute of Account to this type parameter
     */
    public abstract void setType(int type);               
    /**
     * This abstract CMP field accessor method returns the balance attribute for an Account.
     *
     * @return The balance attribute of the Account
    */ 
    public abstract double getBalance();                             
    /**
     * This abstract CMP field accessor method set the balance attribute for an Account.
     * 
     * @param balance The value of balance attribute of Account should be after this operation
     */
    public abstract void setBalance(double balance);               
    /**
     * This abstract CMP field accessor method returns the apr attribute for an Account.
     *
     * @return The apr attribute of the Account 
    */ 
    public abstract double getApr();                             
    /**
     * This abstract CMP field accessor method set the apr attribute for an Account.
     * 
     * @param apr The value of apr attribute of Account should be after this operation 
     */
    public abstract void setApr(double apr); 

    /**
     * This Home Interface Method implementation is responsible for the creation of an Account entry.
     * Except for the accountid field, the other fields will be filled with default value.
     * Default type is 1, default balance is 100.0, and default APR is 4.0.
     * 
     * @param accountId
     * @return null String object that is the primary key type; based on spec, this ejbCreate method should return null.
     * @exception javax.ejb.CreateException
     *                   This exception is thrown to indicate a failure to create the entity EJB.
     */
    public String ejbCreate(String accountId) throws CreateException{
        return ejbCreate(accountId,1,(double)100,(double)4);
    }

    /**
     * This Home Interface Method is responsible for post creation steps, 
     * there are none for Account.
     * 
     * @param accountId String object used as the primary key of the Account object, and used to set accountid attribute
     */ 
    public void ejbPostCreate(String accountId) throws CreateException{
    }

    /**
     * This Home Interface Method implementation is responsible for the creation of an Account entry.
     * 
     * @param accountId String object as the primary key of Account bean
     * @param type      used to set type attribute
     * @param balance   used to set balance attribute
     * @param apr       used to set APR attribute
     * @return Account object created by this method
     * @exception CreateException
     */
    public String ejbCreate(String accountId, int type, double balance, double apr) throws CreateException {
        this.setAccountid(accountId);
        this.setType(type);
        this.setBalance(balance);
        this.setApr(apr);
        return null;
    }

    /**
     * This Home Interface Method is responsible for post creation steps, 
     * there are none for Account.
     * 
     * @param accountId String object used as the primary key of the Account object, and used to set accountid attribute
     * @param type      int used to set type attribute
     * @param balance   double used to set balance attribute
     * @param apr       double used to set APR attribute
     * @exception javax.ejb.CreateException
     *                   This exception is thrown to indicate a failure to create the entity EJB.
     */ 
    public void ejbPostCreate(String accountId, int type, double balance, double apr) throws CreateException
    {
    }
    
    /**
     * This remote method is used to add a given amount of money to an Account entry.
     * 
     * @param amount The amount should be added to balance attribute of the Account
     * @return The balance attribute of the Account after this operation
     */
    public double add(double amount) {
        setBalance(getBalance() + amount);
        return getBalance();
    }

    /**
     * This remote method is used to substract a given amount of money from an Account entry.
     * 
     * @param amount The amount should be substracted from balance attribute of the Account
     * @return The balance attribute of the Account after this operation
     */
    public double subtract(double amount) {
        setBalance(getBalance() - amount);
        return getBalance();
    }

    /**
     * Required callback method for Container Managed Persistence.
     */ 
    public void setEntityContext(javax.ejb.EntityContext ctx) 
    {
    }
    /**
     * Required callback method for Container Managed Persistence.
     */ 
    public void unsetEntityContext()  
    {
    }

    /**
     * This required callback method gets called after the EJB is loaded from
     * the database.  Trim to eliminate trailing spaces that can be added by 
     * the database.
     */ 
    public void ejbLoad() { }  

    /**
     * Required callback method for Contaimer Managed Persistence.
     */ 
    public void ejbRemove() { } 

    /**
     * Required callback method for Contaimer Managed Persistence.
     */ 
    public void ejbStore() { }  

    /**
     * This required callback method is used for lifecycle notification. No 
     * actions need to be taken for Movie Review.
     */ 
    public void ejbPassivate() { }    
    /**
     * This required callback method is used for lifecycle notification. No 
     * actions need to be taken for Movie Review.
     */ 
    public void ejbActivate() { }
    
}