//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.appprofile.accountmanagementejb;

import java.util.Collection;
import javax.ejb.*;
import java.rmi.RemoteException;

/**
 * The Remote Interfe for Application Profile Account Management Sample
 * 
 * @author Cheng-chieh (Jerry) Cheng
 * @version 1.0
 * @since 04/15/2002
 */
public interface AccountManager extends javax.ejb.EJBObject
{
    /**
     * This remote method is used to create a number of Account entity beans.
     * 
     * @param number the number of Accounts needed to be created
     * @return the actual number of Account been created
     * @exception RemoteException
     */
    public int createAccounts(int number) throws RemoteException;
    /**
     * This remote method is used to get an Account record in String format, given the primary key account id.
     * 
     * @param accountid String object represent the primary key of an Account
     * @return account String format represent this Account
     * @exception RemoteException
     */
    public String getAccount(String accountid) throws RemoteException;
    /**
     * This remote method is used to get a collection of Account records in String format
     * that meet the minimum balance condition 
     * 
     * @param balance the required minimum balance condition
     * @return accounts String[] object represent the collection of account records meet the requirement
     * @exception RemoteException
     */
    public String[] getLargeAccounts(double balance) throws RemoteException;
    /**
     * This remote method is used to increase the APR by the passed in apr parameter on all large accounts 
     * with balance greater than the passed in balance parameter.
     * 
     * @param balance the required minimum balance condition
     * @param change indicates how much the apr should increase
     * @return the number of Accounts been affected.
     * @exception RemoteException
     */
    public int increaseLargeAccountsAPR(double balance, double change) throws RemoteException;
    /**
     * This remote method is used to get the number of total accounts 
     * 
     * @return number total accounts
     * @exception RemoteException
     */
    public int getAccountsNumber() throws RemoteException;
}
