//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.appprofile.accountmanagementejb;

import java.rmi.RemoteException;
import java.util.Properties;
import javax.ejb.*;
import javax.naming.*;
import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

/**
 * The AccountManagerBean implements the SessionBean interface. 
 * It contains the buisness methods necessary to manage Accounts 
 * In addition to buisness methonds, it contains the Home Interface Methods, and Callback Methods.
 *
 * @author Cheng-chieh (Jerry) Cheng
 * @version 1.0
 * @since 04/15/2002
 */
public class AccountManagerBean implements SessionBean
{
  /**
   * The InitialContext Object used to do jndi lookup 
   */
  protected InitialContext initialContext = null;
  /**
   * The String, java:comp/env/ejb/AppProfileSample-AccountHome, is used to lookup AccountHome. 
   */
  protected String lookupString = "java:comp/env/ejb/AppProfileSample-AccountHome";
  /**
   * The LocalAccountHome object.  
   */
  protected LocalAccountHome accountLocalHome = null;

  /**
   * Required callback method for Contaimer Managed Persistence.
   */ 
  public void ejbActivate() 
  {
    try
    {
      accountLocalHome = lookupAccount();
    } 
    catch (NamingException ex) 
    {
      throw new EJBException(ex.getMessage());
    }
  }

  /**
   * This Home Interface Method implementation is responsible for the creation of an AccountManager.
   * 
   * @exception javax.ejb.CreateException
   *                   This exception is thrown to indicate a failure to create the entity EJB.
   */
  public void ejbCreate() throws javax.ejb.CreateException 
  {
    try
    {
      accountLocalHome = lookupAccount();
    } 
    catch (NamingException ex) 
    {
      throw new CreateException(ex.getMessage());
    }
  }

  /**
   * Required callback method for Contaimer Managed Persistence.
   */ 
  public void ejbPassivate() 
  {
      accountLocalHome = null;
  }

  /**
   * Required callback method for Contaimer Managed Persistence.
   */ 
  public void ejbRemove() {}

  /**
   * Required callback method for Contaimer Managed Persistence.
   */ 
  public void setSessionContext(javax.ejb.SessionContext ctx) {}

  //private method
  private LocalAccountHome lookupAccount() throws NamingException 
  {
      initialContext = new InitialContext();
      java.lang.Object localHomeObject = initialContext.lookup(this.lookupString);
      return (LocalAccountHome) localHomeObject;
   }

  /**
   * This remote method is used to create a number of Account entity beans.
   * 
   * @param number the number of Accounts needed to be created
   * @return the actual number of Account been created
   */
  public int createAccounts(int number){
      Collection accountCollection = null;
      LocalAccount[] accounts = null;
      try
      {
        if(accountLocalHome==null) accountLocalHome = this.lookupAccount();
        if(accountLocalHome != null)
        {
            try
            {
                accountCollection = accountLocalHome.findAllAccounts();
                accounts =  (LocalAccount[])
                                     (accountCollection.toArray(new LocalAccount[accountCollection.size()]));
            }
            catch(Exception e)
            {
                e.printStackTrace();
                return -1;
            }
        }
        else
        {
            return -1;
        }
      }
      catch (Exception ex) 
      {
          ex.printStackTrace();
          return -1;
      }

      int accountNumberCreated = 0;
      if(accounts!=null)
      {
          if(accounts.length==number) return number;
          else
          {
              //delete all previous accounts first
              for(int i=0; i<accounts.length; i++)
              {
                  try
                  {
                      accounts[i].remove();
                  }
                  catch(Exception e)
                  {
                      e.printStackTrace();
                  }
              }
              accounts = null;
          }
      }
      if(accounts==null && number!=0)
      {
          //create accounts
          String accountid = null;
          int type = 1;
          double balance = (double) 0;
          double apr = (double) 4.0;
          int factor = (int) (10000/number)*2;
          for(int i=0; i<number; i++)
          {
              accountid = "" + i;
              balance = (double) i * factor;
              try
              {
                  accountLocalHome.create(accountid,type,balance,apr);
              }
              catch(Exception e)
              {
                  e.printStackTrace();
              }
          }
          accountNumberCreated = number;
      }
      return accountNumberCreated;
  }

  /**
   * This remote method is used to get a collection of Account records in String format
   * that meet the minimum balance condition 
   * 
   * @param balance the required minimum balance condition
   * @return accounts String[] object represent the collection of account records meet the requirement
   */
  public String[] getLargeAccounts(double balance){
      Collection accountCollection = null;
      LocalAccount[] accounts = null;
      String[] accountRecords =  null;
      try
      {
        if(accountLocalHome==null) accountLocalHome = this.lookupAccount();
        if(accountLocalHome != null)
        {
            try
            {
                accountCollection = accountLocalHome.findLargeAccounts(balance);
                accounts =  (LocalAccount[])
                                     (accountCollection.toArray(new LocalAccount[accountCollection.size()]));
                Arrays.sort(accounts,new LocalAccountComparator());
            }
            catch(Exception e)
            {
                e.printStackTrace();
            }
        }        
      }
      catch (Exception ex) 
      {
          ex.printStackTrace();
      }

      String accountRecord = null;
      if(accounts!=null && accounts.length>0)
      {
          accountRecords = new String[accounts.length];
          for(int i=0; i<accounts.length; i++)
          {
              accountRecord = accounts[i].getAccountid() + "\t" + accounts[i].getBalance() + "\t" + accounts[i].getApr();
              accountRecords[i] = accountRecord;
          }
      }
      return accountRecords;
  }

  /**
   * This remote method is used to increase the APR by the passed in apr parameter on all large accounts 
   * with balance greater than the passed in balance parameter.
   * 
   * @param balance the required minimum balance condition
   * @param change indicates how much the apr should increase
   * @return the number of Accounts been affected.
   */
  public int increaseLargeAccountsAPR(double balance, double change){
      Collection accountCollection = null;
      LocalAccount[] accounts = null;
      int accountNumber = 0;
      try
      {
        if(accountLocalHome==null) accountLocalHome = this.lookupAccount();
        if(accountLocalHome != null)
        {
            try
            {
                accountCollection = accountLocalHome.findLargeAccounts(balance);
                accounts =  (LocalAccount[])
                                     (accountCollection.toArray(new LocalAccount[accountCollection.size()]));

                if(accounts!=null)
                {
                    accountNumber = accounts.length; 
                    for(int i=0; i<accounts.length; i++)
                    {
                        accounts[i].setApr(accounts[i].getApr() + change);
                    }
                }
            }
            catch(Exception e)
            {  
                accountNumber = -1;
                e.printStackTrace();
            }
        }
      }
      catch (Exception ex) 
      {
          accountNumber = -1;
          ex.printStackTrace();
      }

      return accountNumber;
  }

  /**
   * This remote method is used to get an Account record in String format, given the primary key account id.
   * This method will try to find an Account with the same account id first. 
   * If it can not find one, it will create a new one with the passed in accountid parameter as primary key.
   * 
   * @param accountid String object represent the primary key of an Account
   * @return account String format represent this Account
   */
  public String getAccount(String accountid) 
  {
    LocalAccount account = null;
    String accountRecord =  null;
    String debug = "";
    try
    {
      if(accountLocalHome==null) accountLocalHome = this.lookupAccount();
      if(accountLocalHome != null)
      {
          try
          {
              account = accountLocalHome.findByAccountId(accountid);
          }
          catch(Exception e)
          {
              debug = debug + "1: FinderException Happened.  ";
              try
              {
                  account = accountLocalHome.create(accountid);
              }
              catch(Exception ex)
              {
                  debug = debug + "2: CreateException Happened.  ";
                  ex.printStackTrace();
              }
          }
      }
    }
    catch (Exception ex) 
    {
        ex.printStackTrace();
    }

    if(account!=null)
    {
        accountRecord = account.getAccountid() + "\t" + account.getBalance() + "\t" + account.getApr();
    }
    else
    {
        accountRecord = "CAN NOT get account bean " + debug;
    }
      
    return accountRecord;
  }

  /**
   * This remote method is used to get the number of total accounts.
   * 
   * @return number total accounts
   */
  public int getAccountsNumber()
  {
      int number = 0;

      Collection accountCollection = null;
      try
      {
        if(accountLocalHome==null) accountLocalHome = this.lookupAccount();
        if(accountLocalHome != null)
        {
            try
            {
                accountCollection = accountLocalHome.findAllAccounts();
            }
            catch(Exception e)
            {
                e.printStackTrace();
                return -1;
            }
        }
        else
        {
            return -1;
        }
      }
      catch (Exception ex) 
      {
          ex.printStackTrace();
          return -1;
      }
      if(accountCollection!=null)
      {
          number = accountCollection.size();
      }
      return number;
  }
}
