//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.appprofile.accountmanagementejb;


import java.util.Collection;
import javax.ejb.*;

/**
 * The Home Local Interfe for Application Profile Account Management Sample
 * 
 * @author Cheng-chieh (Jerry) Cheng
 * @version 1.0
 * @since 04/15/2002
 */
public interface LocalAccountHome extends javax.ejb.EJBLocalHome {
    /**
     * This Local Home Interface Method is responsible for the creation of an Account entry.
     * Except for the accountid field, the other fields will be filled with default value.
     * Default type is 1, default balance is 100.0, and default APR is 4.0.
     * 
     * @param accountid String object used as the primary key of the Account object, and used to set accountid attribute
     * @return Account object created by this method
     * @exception javax.ejb.CreateException
     *                   This exception is thrown to indicate a failure to create the entity EJB.
     */
    public abstract LocalAccount create(String accountid) throws javax.ejb.CreateException;
    /**
     * This Local Home Interface Method is responsible for the creation of an Account entry.
     * 
     * @param accountid String object used as the primary key of the Account object, and used to set accountid attribute
     * @param type      int used to set type attribute
     * @param balance   double used to set balance attribute
     * @param apr       double used to set APR attribute
     * @return Account object created by this method
     * @exception javax.ejb.CreateException
     *                   This exception is thrown to indicate a failure to create the entity EJB.
     */
    public abstract LocalAccount create(String accountid, int type, double balance, double apr) throws javax.ejb.CreateException;
    /**
     * This Home Interface Method is responsible for finding an Account entry based on the account id.
     * 
     * @param accountid String object used as the primary key of Account object.
     * @return Account object found by this method.
     * @exception javax.ejb.FinderException This exception is thrown to indicate a failure to find the entity EJB.
     */
    public abstract LocalAccount findByAccountId (String accountid) throws javax.ejb.FinderException;
    /**
     * This Home Interface Method is responsible for finding a collection of Accounts with balance greater than the balance parameter.
     * 
     * @param balance the minimum value of balance attribute.
     * @return Collection of Account objects found by this method.
     * @exception FinderException
     */
    public abstract Collection findLargeAccounts(double balance) throws javax.ejb.FinderException;   
    /**
     * This Home Interface Method is responsible for finding a collection of all Accounts.
     * 
     * @return Collection of Account objects found by this method.
     * @exception javax.ejb.FinderException
     *                   This exception is thrown to indicate a failure to find the entity EJB.
     */
    public abstract Collection findAllAccounts() throws javax.ejb.FinderException;   
    /**
     * This Home Interface Method is responsible for finding an Account entry based on the accountid .
     * 
     * @param accountid String object used as the primary key of Account object.
     * @return Account object found by this method.
     * @exception javax.ejb.FinderException This exception is thrown to indicate a failure to find the entity EJB.
     */
    public abstract LocalAccount findByPrimaryKey (String accountid)
        throws javax.ejb.FinderException;  
}
