//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Various configuration information and constants for the application.
 */
public class Config 
{
    public static final String CLIENT_COOKIE_NAME= "WebSphereTraderCookie";
    public static final String CLIENT_PARAM_CLIENTID = "TSLoginID";
    public static final String CLIENT_PARAM_TICKERS  = "TSTickers";
    public static final String CLIENT_PARAM_ADDTICKER  = "TSAddTicker";
    public static final String CLIENT_PARAM_RMVTICKER  = "TSRmvTicker";
    public static final String CLIENT_PARAM_STREAM_NAME = "streamName";

    
    public static final String APP_AB_TOPIC_CONNECTION = "AB_AppTopicConnection";
    public static final String APP_AB_LISTENER_REGISTRY = "AB_AppSubscriberRegistry";
    public static final String CLIENT_AB_SERVLET_LISTENER = "AB_ServletListener";
    public static final String CLIENT_AB_TICKERS          = "AB_ActiveTickers";
    public static final String CLIENT_AB_TICKERS_OLD      = "AB_PreviousActiveTickers";
    
    public static final int CLIENT_HEARTBEAT_INTERVAL = 10000;
    public static final int CLIENT_HEARTBEAT_STALE = 2;
    public static final int CLIENT_HEARTBEAT_DEAD = 3;
    public static final int PRICEGEN_HEARTBEAT_INTERVAL = 5000;
    public static final int PRICEGEN_HEARTBEAT_STALE = 2;
    public static final int PRICEGEN_HEARTBEAT_DEAD = 3;
    public static final int STREAMSERVER_HEARTBEAT_INTERVAL = 3000;
    public static final int STREAMSERVER_HEARTBEAT_STALE = 2;
    public static final int STREAMSERVER_HEARTBEAT_DEAD = 10;
    
    public static final String PRICEGEN_PRICETYPE_STATIC = "static";
    public static final String PRICEGEN_PRICETYPE_UP = "up";
    public static final String PRICEGEN_PRICETYPE_DOWN = "down";
    
    private final static String _PropsFileName = "WebSphereTrader.prop";
    private final static String _PropsFileHeader = "WebSphere Trader Properties";
    private static Properties _AppProps = null;

   
    /**
     * Retrieve the application's properties file.
     * @return Properties
     */
    private synchronized static Properties getAppProperties() throws IOException 
    {
        // Load the properties once.
        if(_AppProps == null) {
            ClassLoader cl = Thread.currentThread().getContextClassLoader();

            InputStream is = cl.getResourceAsStream(_PropsFileName);
            _AppProps = new Properties();
            _AppProps.load(is);
        }
        return _AppProps;
    }
    

    /**
     * Retrive a property from the application's configuration file.
     * @param key Property to retrieve
     * @param dftValue Default value to return if the property could not be retrieved.
     * @return Returns the value of the property or the default value.
     */
    public synchronized static String getAppProperty(String key, String dftValue) throws IOException 
    {
        Properties p = getAppProperties();
        return p.getProperty(key,dftValue);
        
    }    
    
    
}
