//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.applet;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

/**
 * Keeps the current client alive by pinging every
 * n seconds
 */
public class PingerHandler extends Thread {

    /**
     * The number of milliseconds to wait between pings.  
     * The HeartBeat interval.
     */
    int hbInterval;

    /**
     * If true, the thread should terminate itself.
     */
    boolean isCancelled;

    /**
     * A reference to the applet
     */
    StreamerApplet mainApp;
    
    static Integer totalPingers = new Integer(0);
    /**
     * @param hbInterval_ms
     * @param clientid
     * @param mainApp
     */
    PingerHandler(int hbInterval_ms, StreamerApplet mainApp) {
        hbInterval = hbInterval_ms;
        this.mainApp = mainApp;
        isCancelled = false;
    }
    
    /**
     * Tell this thread to cancel itself.
     */
    public synchronized void cancel() {
        isCancelled = true;
    }
    
    /**
     * Find out if this thread has been asked to cancel itself.
     */
    public synchronized boolean getCancelled()
    {
        return isCancelled;
    }    
    
    /**
     * Start a thread to continually ping the server.
     */
    public void startPinging()
    {
        this.start();
    }
    
    /**
	 * @see Runnable#run()
	 */
	public void run() {
        synchronized(totalPingers)
        {
            totalPingers = new Integer(totalPingers.intValue() + 1);
            setName("PingerHandler #" + totalPingers.intValue());     
        }
        long lastTime = System.currentTimeMillis();
        while(!getCancelled()) 
        {
            try 
            {
                if(System.currentTimeMillis() - lastTime >= hbInterval)
                {
                    URLSenderHandler.sendPingMessage(mainApp.getCookie());
                    lastTime = System.currentTimeMillis();
                }

                Thread.sleep(500);
            }
	        catch(InterruptedException ex) {
                ex.printStackTrace();
	        }
        }

        synchronized(totalPingers)
        {
            totalPingers = new Integer(totalPingers.intValue() - 1);
        }
	}

}
