//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// 5724-J08, 5724-I63, 5724-H88, 5724-H89, 5655-N02, 5733-W70. COPYRIGHT International Business Machines Corp., 2001, 2008
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.applet;

import java.applet.Applet;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Label;
import java.text.DecimalFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

import com.ibm.websphere.samples.asynchbeans.wstrader.ClientInstruction;
import com.ibm.websphere.samples.asynchbeans.wstrader.Config;

/**
 * The WebSphere Trader Applet client
 */
public class StreamerApplet extends Applet implements NotificationListener
{
    /**
     * A lookup table of ticker symbols and grid labels.
     */
    Hashtable tickerLookup = null;
    
    /**
     * The current active cookie retrieved at login time.
     */
    String cookie;
    
    /**
     * The thread that will actively ping the server and keep
     * our client information alive.
     */
    PingerHandler pinger = null;
    
    /**
     * A reference to the utility class
     */
    Utility utility = new Utility();
    
    /**
     * Each active streamer in the applet has an entry here. Each
     * streamer should have a unique name.
     * NOTE:  This is for future use.  Only one streamer "DEFAULT" is currently used.
     */
    //Hashtable activeStreamers;

    // The messages to display regarding various status information.
    public static final String STATUS_PRICEFEED_ALIVE = "Price Feed = Alive";
    public static final String STATUS_PRICEFEED_STALE = "Price Feed = Stale";
    public static final String STATUS_PRICEFEED_DEAD = "Price Feed = Dead";

    public static final String STATUS_SERVER_ALIVE = "Server Status = Alive";
    public static final String STATUS_SERVER_STALE = "Server Status = Stale";
    public static final String STATUS_SERVER_DEAD = "Server Status = Dead";
    
    /**
     * The default ticker symbols to use when establing a new client session.
     */
    private static final String DEFAULT_TICKERS = "ACTV,BORE,BUYY,COMP,COWS,LEAK";
    
    // The Ticker Symbol Grid dimensions
    private static final int GRID_ROWS = 9;
    private static final int GRID_COLS = 4;
    
    // The Ticker Symbol Grid column numbers.
    public static final int GRID_COL_SYMBOL = 0;
    public static final int GRID_COL_BID = 1;
    public static final int GRID_COL_ASK = 2;
    public static final int GRID_COL_TIME = 3;
    
    // Colors to use for price adjustments
    private static final Color priceDownBackColor = new Color(245,101,104);
    private static final Color priceDownForeColor = Color.black;
    private static final Color priceUpBackColor = new Color(0,128,0);
    private static final Color priceUpForeColor = Color.white;

    // Declare controls
    java.awt.Panel pnlMain = new java.awt.Panel();
    java.awt.Panel pnlGrid = new java.awt.Panel();
    java.awt.Label lblGrid[];
    java.awt.Label[][] gridLabels = new java.awt.Label[GRID_ROWS][GRID_COLS];

    java.awt.Panel pnlTopTop = new java.awt.Panel();
    java.awt.Label lblTitle = new java.awt.Label();

    java.awt.Panel pnlTop = new java.awt.Panel();
    java.awt.Panel pnlLoginOut = new java.awt.Panel();
    java.awt.TextField fldLoginName = new java.awt.TextField();
    java.awt.Button btnDoLoginOut = new java.awt.Button();

    java.awt.Panel pnlTopMid = new java.awt.Panel();

    java.awt.Panel pnlSymbol = new java.awt.Panel();
    java.awt.TextField fldSymbol = new java.awt.TextField();
    java.awt.Button btnAddSymbol = new java.awt.Button();
    java.awt.Button btnRmvSymbol = new java.awt.Button();

    java.awt.Panel pnlBot = new java.awt.Panel();
    java.awt.Label lblStatusPrice = new java.awt.Label();
    java.awt.Label lblStatusServer = new java.awt.Label();
    
    /**
     * A JDK 1.2 compatible version of the GridBagConstraints that
     * will work with JDK 1.0.
     */
    class GridBagConstraints2 extends GridBagConstraints
    {
        /**
         * @see java.awt.GridBagConstraints#GridBagConstraints(int, int, int, int, double, double, int, int, Insets, int, int)
         */
        GridBagConstraints2(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty, int anchor, int fill, Insets insets, int ipadx, int ipady) 
        {
            super();
            this.gridx = gridx;
            this.gridy = gridy;
            this.gridwidth = gridwidth;
            this.gridheight = gridheight;
            this.weightx = weightx;
            this.weighty = weighty;
            this.anchor = anchor;
            this.fill = fill;
            this.insets = insets;
            this.ipadx = ipadx;
            this.ipady = ipady;
        }
        
    }
    
    /**
     * Initialize the Applet
     * @see java.applet.Applet#init()
     */
    public void init()
    {
        try {
	        // Initialize the UI
            
            // Main panel
	        setLayout(new GridLayout(1,1,0,0));
	        setSize(467,238);
	        pnlMain.setLayout(new GridBagLayout());
	        add(pnlMain);
	        
            // Central Grid
	        pnlGrid.setLayout(new GridLayout(10,4,1,1));
	        pnlMain.add(pnlGrid,new GridBagConstraints2(0,2,1,1,1.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
	        pnlGrid.setBackground(java.awt.Color.black);

            
            // Grid column headings
			lblGrid = new Label[4];
			String [] columnTitles = {"Symbol", "Bid", "Offer", "Last Trade Time"};
			Font sansSerifBold = new Font("SansSerif", Font.BOLD, 12);
			Font sansSerifPlain = new Font("SansSerif", Font.BOLD, 12);
			Font monoSpaced = new Font("MonoSpaced", Font.PLAIN, 12);
			
	        for(int i = 0; i < lblGrid.length; ++i)
	        {
		        lblGrid[i] = new Label();
		        lblGrid[i].setText(columnTitles[i]);
		        lblGrid[i].setAlignment(java.awt.Label.CENTER);
		        lblGrid[i].setBackground(java.awt.Color.darkGray);
		        lblGrid[i].setForeground(java.awt.Color.white);
		        lblGrid[i].setFont(sansSerifBold);
		        pnlGrid.add(lblGrid[i]);
	        }
	
	        // The Grid cells
	        for (int r = 0; r < gridLabels.length;r++) 
	        {
	            for (int c = 0; c < gridLabels[r].length;c++)
	            {
	                gridLabels[r][c] = new java.awt.Label();
		            gridLabels[r][c].setAlignment(java.awt.Label.CENTER);
		            pnlGrid.add(gridLabels[r][c]);
		            gridLabels[r][c].setForeground(java.awt.Color.white);
	                gridLabels[r][c].setText("");
	                
	                if (c == GRID_COL_SYMBOL) {
	                    gridLabels[r][c].setFont(sansSerifPlain);
	                }
	                else {
	                    gridLabels[r][c].setFont(monoSpaced);
	                    gridLabels[r][c].setAlignment(java.awt.Label.RIGHT);
	                }
	
	                if (r % 2 == 0) {
	                    gridLabels[r][c].setBackground(new java.awt.Color(0,128,192));
	                }
	                else {
	                    gridLabels[r][c].setBackground(new java.awt.Color(0,64,128));
	                }
	            }
	        }        
	
	
	        // Top title panel
	        pnlTopTop.setLayout(new FlowLayout(FlowLayout.CENTER,0,0));
	        pnlMain.add(pnlTopTop, new GridBagConstraints2(0,0,1,1,0.0,0.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.HORIZONTAL,new Insets(0,0,1,0),0,0));
	        pnlTopTop.setBackground(java.awt.Color.darkGray);
	        lblTitle.setText("WebSphere Trader");
	        pnlTopTop.add(lblTitle);
	        lblTitle.setForeground(java.awt.Color.white);
	        lblTitle.setFont(new Font("Dialog", Font.BOLD, 16));
	
	        // Top toolbar
	        pnlTop.setLayout(new GridBagLayout());
	        pnlMain.add(pnlTop,new GridBagConstraints2(0,1,1,1,1.0,0.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.HORIZONTAL,new Insets(0,0,1,0),0,0));
	        pnlTop.setBackground(java.awt.Color.darkGray);
	        pnlLoginOut.setLayout(new GridBagLayout());
	        pnlTop.add(pnlLoginOut, new GridBagConstraints2(0,0,1,1,0.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));
	        
            // The Login/Logout panel
	        pnlLoginOut.add(fldLoginName,new GridBagConstraints2(0,0,1,1,1.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(0,0,2,0),30,0));
	        fldLoginName.setBackground(java.awt.Color.white);
	        btnDoLoginOut.setLabel("Login");
	        pnlLoginOut.add(btnDoLoginOut,new GridBagConstraints2(1,0,1,1,1.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(0,3,0,0),0,0));
	        btnDoLoginOut.setBackground(java.awt.Color.lightGray);

            // The add/remove symbol panel	        
	        pnlTopMid.setLayout(null);
	        pnlTop.add(pnlTopMid, new GridBagConstraints2(1,0,1,1,1.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));
	        pnlSymbol.setLayout(new GridBagLayout());
	        pnlTop.add(pnlSymbol,new GridBagConstraints2(2,0,1,1,0.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));
	        pnlSymbol.add(fldSymbol,new GridBagConstraints2(1,0,1,1,0.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(0,0,2,0),30,0));
	        fldSymbol.setBackground(java.awt.Color.white);
	        btnAddSymbol.setLabel("Add");
	        pnlSymbol.add(btnAddSymbol,new GridBagConstraints2(2,0,1,1,0.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(0,3,0,0),0,0));
	        btnAddSymbol.setBackground(java.awt.Color.lightGray);
	        btnRmvSymbol.setLabel("Remove");
	        pnlSymbol.add(btnRmvSymbol,new GridBagConstraints2(3,0,1,1,1.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(0,3,0,0),0,0));
	        btnRmvSymbol.setBackground(java.awt.Color.lightGray);
            btnAddSymbol.setEnabled(false);
            btnRmvSymbol.setEnabled(false);
	
            // Bottom status bar
            pnlBot.setLayout(new GridLayout(1,2));
	        pnlMain.add(pnlBot, new GridBagConstraints2(0,10,1,1,0.0,0.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.HORIZONTAL,new Insets(0,0,0,0),0,0));
	        pnlBot.setBackground(java.awt.Color.darkGray);
	        pnlBot.add(lblStatusPrice);
            pnlBot.add(lblStatusServer);
	        lblStatusPrice.setFont(new Font("Dialog", Font.BOLD, 12));
            lblStatusPrice.setText("                                           ");
            lblStatusServer.setFont(new Font("Dialog", Font.BOLD, 12));
            lblStatusServer.setText("                                           ");
	
	        // Register listeners
	        AppletAction appletAction = new AppletAction();
	        btnDoLoginOut.addActionListener(appletAction);
	        btnAddSymbol.addActionListener(appletAction);
	        btnRmvSymbol.addActionListener(appletAction);
        }
        catch (Throwable e)
        {
            e.printStackTrace();
        }
        
    }
    
    /**
     * Start the Applet
     * @see java.applet.Applet#start()
     */    
    public void start()
    {
        try 
        {
            // Set the intial focus to our login field
            fldLoginName.requestFocus();
            
	        // Start a thread that populates our window
	        tickerLookup = new Hashtable();
	        //activeStreamers = new Hashtable();
            
            // Create our singleton URLSender and set the CodeBase to 
            // our Applet's
            URLSenderHandler urlSender = URLSenderHandler.getSelf();
            if(urlSender == null)
            {
                urlSender = URLSenderHandler.create(this.getCodeBase());
            }
            
            // Listen for asynchronous notifications
            urlSender.addNotificationListener(this);
        }
        catch(Throwable e)
        {
            e.printStackTrace();
        }
    }
    
    /**
     * Stop the Applet
     * @see java.applet.Applet#stop()
     */
    public void stop()
    {
        // Stop our Streamer
    	stopStreamer();
        
        // Stop our pinger.
        stopPinger();
        
        // Unregister our notification listener
        URLSenderHandler.getSelf().removeNotificationListener(this);
    }

    /**
     * Gets the cookie.
     * @return Returns a String
     */
    public String getCookie() {
        return cookie;
    }

    /**
     * Sets the cookie.
     * @param cookie The cookie to set
     */
    public void setCookie(String cookie) {
        this.cookie = cookie;
    }

    
    /**
     * Start a ticker streamer in it's own thread and subscribe ourselves
     * as a listener for outgoing events.  Also create a ServerMonitor
     * and add a listener.
     */
    private synchronized void startStreamer(String streamerName)
    {
        // Create our singleton streamer handler.  Pass it our codebase, our ticker to label lookup, streamer name, cookie and
        // the ticker symbols from the grid.  We don't want to send our default ticker symbols because
        // they might be different from a previous login.
        StreamerHandler sh = StreamerHandler.getSelf();
        if (sh == null)
        {
            sh = StreamerHandler.create(getCodeBase(), tickerLookup, streamerName, getTickerSymbolsFromGrid(), cookie);
            //activeStreamers.put("DEFAULT", sc);
        }
        sh.addNotificationListener(this);
        
        // Create a singleton ServerMonitor.  One per JVM for 
        // all potential applet listeners.
        ServerMonitor sm = ServerMonitor.getSelf();
        if (sm == null)
        {
            sm = ServerMonitor.create(Config.STREAMSERVER_HEARTBEAT_INTERVAL, Config.STREAMSERVER_HEARTBEAT_STALE, Config.STREAMSERVER_HEARTBEAT_DEAD);
        }
        
        // The monitor is automatically started and stopped once there is at least one listener registered.
        // In addition, make sure we get notified of the current state
        sm.addNotificationListener(this);
        sm.fireCurState();
    }
    /**
     * Stop all active streamers
     */
	//private synchronized void stopStreamers()
	//{
    //	Enumeration enum = activeStreamers.keys();
    //	while(enum.hasMoreElements())
    //	{
    //		String key = (String)enum.nextElement();
    //        stopStreamer(key);
    //	}
    //	activeStreamers.clear();
	//}
    
    /**
     * Stop our streamer
     */
    private synchronized void stopStreamer()
    {
        ServerMonitor sm = ServerMonitor.getSelf();
        if (sm != null)
        {
            sm.removeNotificationListener(this);
        }

        //StreamerHandler as = (StreamerHandler)activeStreamers.get(streamerName);
        
        StreamerHandler as = StreamerHandler.getSelf();
	    if (as != null)
        {
            as.removeNotificationListener(this);
        }
    }
    

    /**
     * Start our client pinger to keep the client information on the
     * server alive.
     */
    public void startPinger() 
    {
        stopPinger();
        pinger = new PingerHandler(Config.CLIENT_HEARTBEAT_INTERVAL, this);
        pinger.startPinging();
    }

    /**
     * Stop our client pinger.
     */
    public void stopPinger()
    {
        if (pinger != null)
        {
            pinger.cancel();
            pinger = null;
        }
    }
	    
    /**
     * Add a new Ticker Symbol to our grid
     * @param symbol add a single ticker symbol to our grid.
     * @return true if successful
     */
    boolean addNewTickerSymbolToGrid(String symbol)
    {
        if (!tickerLookup.containsKey(symbol))
        {
            // Get the next available grid row and set the symbol name
            if (tickerLookup.size() >= gridLabels.length) 
            {
                showStatus("No more slots available.");
                return false;
            }
            System.out.println("Putting row: symbol=" + symbol + " size: " + tickerLookup.size());
            Integer curRow = new Integer(tickerLookup.size());
            tickerLookup.put(symbol, curRow);
            ((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).setText(symbol);
            return true;
        }
        else
        {
            return false;
        }
    }   

    /**
     * Add a comma-delimited list of Symbols to our grid
     * @param symbols the symbols to add.
     */
    void addTickerSymbolsToGrid(String symbols)
    {
        StringTokenizer stSymbols = new StringTokenizer(symbols, ",");
        while(stSymbols.hasMoreTokens())
        {
            String curSymbol = stSymbols.nextToken();
            addNewTickerSymbolToGrid(curSymbol);
        }
    }


    /**
     * Add a new Ticker Symbol to our grid
     * @param symbol the symbol to remove.
     */
    void removeTickerSymbolFromGrid(String symbol)
    {
        if (tickerLookup.containsKey(symbol))
        {
            System.out.println("Removing row: symbol=" + symbol + " size: " + tickerLookup.size());
            
            // Make sure that only one thread is modifying the grid at a time.
            synchronized(gridLabels)
            {
	            // Lookup the row number for the symbol, remove the row 
	            // and move all rows below it up one.
	            int curRow = ((Integer) tickerLookup.get(symbol)).intValue();
	            
	            for (int i=curRow;i<tickerLookup.size()-1;i++)
	            {
	                swapRows(i,i+1);   
	            }
                clearRow(tickerLookup.size()-1);
                tickerLookup.remove(symbol);
            }
        }
    }   

    /**
     * Clear an entire row of data including the symbol name.
     */
    private void clearRow(int row)
    {
        // Make sure that only one thread is modifying the grid at a time.
        synchronized(gridLabels)
        {
            if (row < gridLabels.length)
            {
    	        ((Label) gridLabels[row][StreamerApplet.GRID_COL_SYMBOL]).setText("");
                clearRowData(row);
            }
        }
    }
    
    /**
     * Clear the data, but not the symbol name from a row in the grid.
     */
    private void clearRowData(int row)
    {
        // Make sure that only one thread is modifying the grid at a time.
        synchronized(gridLabels)
        {
            if (row < gridLabels.length)
            {
                ((Label) gridLabels[row][StreamerApplet.GRID_COL_ASK]).setText("");
                ((Label) gridLabels[row][StreamerApplet.GRID_COL_BID]).setText("");
                ((Label) gridLabels[row][StreamerApplet.GRID_COL_TIME]).setText("");
    
                ((Label) gridLabels[row][StreamerApplet.GRID_COL_ASK]).setBackground(((Label)gridLabels[row][StreamerApplet.GRID_COL_SYMBOL]).getBackground());
                ((Label) gridLabels[row][StreamerApplet.GRID_COL_BID]).setBackground(((Label)gridLabels[row][StreamerApplet.GRID_COL_SYMBOL]).getBackground());
            }
        }
    }
    
    /**
     * Swap two rows in the grid.
     */
    private void swapRows(int row1, int row2)
    {
        // Make sure that only one thread is modifying the grid at a time.
        synchronized(gridLabels)
        {        
            // Clear both rows, then swap the symbols.
	        clearRowData(row1);
	        clearRowData(row2);
            
	        String oldSymbol1 = ((Label) gridLabels[row1][StreamerApplet.GRID_COL_SYMBOL]).getText();
            String oldSymbol2 = ((Label) gridLabels[row2][StreamerApplet.GRID_COL_SYMBOL]).getText();
	        ((Label) gridLabels[row1][StreamerApplet.GRID_COL_SYMBOL]).setText(oldSymbol2);
	        ((Label) gridLabels[row2][StreamerApplet.GRID_COL_SYMBOL]).setText(oldSymbol1);
            
            tickerLookup.remove(oldSymbol1);
            tickerLookup.remove(oldSymbol2);
            tickerLookup.put(oldSymbol1, new Integer(row2));
            tickerLookup.put(oldSymbol2, new Integer(row1));
            
        }
    }
    
    /**
     * Update the status information for all servers that we are monitoring.
     */
    void updateServerStatus(ClientInstruction ci)
    {
        String source = ci.getStatusSource();
        String statusValue = ci.getStatusValue();
        
        if(source.equals(ClientInstruction.RESPONSE_STATUS_SOURCE_PRICEGEN))
        {
            if(statusValue.equals(ClientInstruction.RESPONSE_STATUS_VALUE_ALIVE) ||
               statusValue.equals(ClientInstruction.RESPONSE_STATUS_VALUE_FRESH))
            {
	            lblStatusPrice.setText(STATUS_PRICEFEED_ALIVE);
	            lblStatusPrice.setForeground(Color.green);
            }
            else if(statusValue.equals(ClientInstruction.RESPONSE_STATUS_VALUE_STALE))
            {
                lblStatusPrice.setText(STATUS_PRICEFEED_STALE);
                lblStatusPrice.setForeground(Color.yellow);
            }
            else if(statusValue.equals(ClientInstruction.RESPONSE_STATUS_VALUE_DEAD))
            {
                lblStatusPrice.setText(STATUS_PRICEFEED_DEAD);
                lblStatusPrice.setForeground(Color.red);
            }
            else 
            {
                lblStatusPrice.setText("");
            }
            lblStatusPrice.getParent().validate();
            
        }
        else if(source.equals(ClientInstruction.RESPONSE_STATUS_SOURCE_STREAMSERVER))
        {
            if(statusValue.equals(ClientInstruction.RESPONSE_STATUS_VALUE_ALIVE) ||
               statusValue.equals(ClientInstruction.RESPONSE_STATUS_VALUE_FRESH))
            {
                lblStatusServer.setText(STATUS_SERVER_ALIVE);
                lblStatusServer.setForeground(Color.green);
            }
            else if(statusValue.equals(ClientInstruction.RESPONSE_STATUS_VALUE_STALE))
            {
                lblStatusServer.setText(STATUS_SERVER_STALE);
                lblStatusServer.setForeground(Color.yellow);
            }
            else if(statusValue.equals(ClientInstruction.RESPONSE_STATUS_VALUE_DEAD))
            {
                lblStatusServer.setText(STATUS_SERVER_DEAD);
                lblStatusServer.setForeground(Color.red);
            }
            else 
            {
                lblStatusServer.setText("");
            }
            lblStatusServer.getParent().validate();
            
        }
    }

    /**
     * Update a ticker's information
     * @param symbol the symbol to update
     * @param bid the new bid price
     * @param ask the new asking price
     * @param bidType the type of bid:  up, down or static
     * @param time the time that the server sent the updated price
     */
    void refreshTickerCell(String symbol, double bid, double ask, String bidType, String time)
    {
        // Look for our ticker in our grid.  Each grid cell
        // is is a label.

        // Make sure that only one thread is modifying the grid at a time.
        synchronized(gridLabels)
        {
    
            // Get the ticker row
	        Integer curRow = (Integer)tickerLookup.get(symbol);
	        
            if (curRow != null)
            {
		        // Set the prices
		        java.awt.Label lblBid = gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_BID];
		        java.awt.Label lblAsk = gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_ASK];
		        lblBid.setText(formatNumber(bid, "###,##0.##"));
		        ((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_ASK]).setText(formatNumber(ask, "###,##0.##"));
		        ((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_TIME]).setText(time);
		        
		        // Set the color
		        if(bidType.equals(Config.PRICEGEN_PRICETYPE_UP))
		        {
		            lblBid.setBackground(((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).getBackground());
		            lblAsk.setBackground(priceUpBackColor);
                    lblBid.setForeground(((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).getForeground());
                    lblAsk.setForeground(priceUpForeColor);
		        }
		        else if (bidType.equals(Config.PRICEGEN_PRICETYPE_DOWN))
		        {
		            lblBid.setBackground(priceDownBackColor);
		            lblAsk.setBackground(((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).getBackground());
                    lblBid.setForeground(priceDownForeColor);
                    lblAsk.setForeground(((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).getForeground());
		        }
		        else
		        {
		            lblBid.setBackground(((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).getBackground());
		            lblAsk.setBackground(((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).getBackground());
                    lblBid.setForeground(((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).getForeground());
                    lblAsk.setForeground(((Label) gridLabels[curRow.intValue()][StreamerApplet.GRID_COL_SYMBOL]).getForeground());
		        }
            }
        }
    }

    
    /**
     * Return a comma-delimited list of Ticker Symbols
     * from our grid.
     */
    String getTickerSymbolsFromGrid()
    {
        StringBuffer symbols = new StringBuffer();

        // Make sure that only one thread is modifying the grid at a time.
        synchronized(gridLabels)
        {
            for(Enumeration e = tickerLookup.keys();e.hasMoreElements();)
            {
                String curSymbol = (String)e.nextElement();
                symbols.append(curSymbol);
                if(e.hasMoreElements())
                {
                    symbols.append(",");
                }
            }
        }
        return symbols.toString();
    }
    
    /**
     * Format a double using a given pattern.
     * @param val the value to format
     * @param the pattern to apply to the number
     */
    private String formatNumber(double val, String pattern)
    {
        DecimalFormat f = (DecimalFormat) DecimalFormat.getCurrencyInstance();
        
        f.applyPattern(pattern);
        f.setMinimumFractionDigits(2);
        f.setMaximumFractionDigits(2);
        return f.format(val);
    }    
    /**
     * Login to the WebSphere Trader Streamer servlet using the supplied clientid.  The clientid
     * or loginid can be any string.  The results of the login will be handled in the 
     * handleNotification() method asynchronously.
     * @param clientid any string uniquely identifying the client to the server.
     */
    public void doLogin(String clientid)
    {
        // Asynchronously login
        showStatus("Logging-in");
        URLSenderHandler.getSelf().sendLoginMessage(clientid, DEFAULT_TICKERS);
    }
    
    /**
     * Explicity log-out the current client id on the
     * server.
     */
    private void doLogout()
    {
        // Asynchronously logout
        showStatus("Logging-out");
        if (getCookie() != null)
        {
            // Stop our pinger
            stopPinger();

            // If logged-in cancel stream processing and logout
            stopStreamer();
            
            // Logout from the server
            URLSenderHandler.getSelf().sendLogoutMessage(getCookie());
        }
    }
    
    /**
     * The ActionListener for intercepting all control events.
     */
    class AppletAction implements java.awt.event.ActionListener
    {
        public void actionPerformed(java.awt.event.ActionEvent event)
        {
            Object object = event.getSource();
            if (object == btnDoLoginOut) {
                btnDoLoginOut.setEnabled(false);
                btnDoLoginOut_ActionPerformed(event);
                btnDoLoginOut.setEnabled(true);
            }
            else if (object == btnAddSymbol) {
                btnAddSymbol.setEnabled(false);
                btnAddSymbol_ActionPerformed(event);
                btnAddSymbol.setEnabled(true);
            }
            else if (object == btnRmvSymbol) {
                btnRmvSymbol.setEnabled(false);
                btnRmvSymbol_ActionPerformed(event);
                btnRmvSymbol.setEnabled(true);
            }
                
        }
    }
        
    /**
     * Handle the Login/Logout button events.
     */
    void btnDoLoginOut_ActionPerformed(java.awt.event.ActionEvent event)
    {
        try {
            
            if (fldLoginName.isEnabled())
            {
                // Get the loginid/clientid from the UI
                String clientID = fldLoginName.getText();
                if (clientID.trim().length() > 0)
                {
                    btnDoLoginOut.setEnabled(false);
    
    	            // If not logged-in, then log-in
                    doLogin(clientID);
                }
            }
            else
            {
                btnDoLoginOut.setEnabled(false);

                doLogout();
            }
            
        }
        catch (Exception e)
        {
            showStatus("Error Logging in/out.  See the Java Console for details.");
            e.printStackTrace();
        }
        
             
    }
    
    /**
     * Handle the Add button events.
     */
    void btnAddSymbol_ActionPerformed(java.awt.event.ActionEvent event)
    {
        try
        {
	        // Add the ticker to our local list and update the server
	        String newSymbol = fldSymbol.getText().toUpperCase();
            if (newSymbol.trim().length() > 0)
            {
                fldSymbol.setText(newSymbol);
    	        
    	        if (tickerLookup.containsKey(newSymbol))
    	        {
    	            showStatus("Symbol already exists");
    	            return;
    	        }
    	        
    	        if(addNewTickerSymbolToGrid(newSymbol))
                {
                    URLSenderHandler.getSelf().sendUpdateMessage("DEFAULT", getCookie(), getTickerSymbolsFromGrid());
                }
            }
        }
        catch (Exception e)
        {
            showStatus("Error Adding Symbol.  See the Java Console for details.");
            e.printStackTrace();
        }
    }

    /**
     * Handle the Remove button events.
     */
    void btnRmvSymbol_ActionPerformed(java.awt.event.ActionEvent event)
    {
        try
        {
	        // Remove the ticker from our local list and update the server
            String newSymbol = fldSymbol.getText().toUpperCase();
            if (newSymbol.trim().length() > 0)
            {
                fldSymbol.setText(newSymbol);
    	        
    	        if (!tickerLookup.containsKey(newSymbol))
    	        {
    	            showStatus("Symbol doesn't exist");
    	            return;
    	        }
    	        
    	        removeTickerSymbolFromGrid(newSymbol);
    	        
                URLSenderHandler.getSelf().sendUpdateMessage("DEFAULT", getCookie(), getTickerSymbolsFromGrid());
            }
        }
        catch (Exception e)
        {
            showStatus("Error Removing Symbol.  See the Java Console for details.");
            e.printStackTrace();
        }
             
    }    
	
    /**
     * Handle callbacks from our various HTTP data streams and threads.
	 * @see NotificationListener#handleNotification(Notification)
	 */
	public synchronized void handleNotification(Notification n) 
    {
        // LOGIN REQUEST COMPLETE
        if (n.getType() == Notification.NOTIFICATION_LOGIN)
        {
            LoginInfo loginInfo = (LoginInfo) n.getUserData();
            
            if(loginInfo.success)
            {
                
                // Setup our grid based on our default symbols or those recieved from the server.            
                addTickerSymbolsToGrid(loginInfo.symbols);
                
                // Save the cookie that was received from the login servlet
                setCookie(loginInfo.cookie);

                // Start processing our ticker stream 
                startStreamer("DEFAULT");
                
                // Startup our pinger
                startPinger();
                
                // Update the UI
                fldLoginName.setEnabled(false);
                btnDoLoginOut.setLabel("Logout");
                
                btnAddSymbol.setEnabled(true);
                btnRmvSymbol.setEnabled(true);

                showStatus("Login complete.");
            }
            else
            {
                showStatus("Login failed.");
            }

            btnDoLoginOut.setEnabled(true);
        }
        
        // UPDATE REQUEST COMPLETE
        else if(n.getType() == Notification.NOTIFICATION_UPDATE)
        {
            boolean success = ((Boolean) n.getUserData()).booleanValue();
            if(success)
            {
                fldSymbol.setText("");
                showStatus("Update successful");
            }
            else
            {
                showStatus("Update failed");
            }
        }
        
        // LOGOUT REQUEST COMPLETE
        else if(n.getType() == Notification.NOTIFICATION_LOGOUT)
        {
            boolean success = ((Boolean) n.getUserData()).booleanValue();
            if(success)
            {
                
                // Update the UI.
                tickerLookup.clear();
                for (int r = 0;r < gridLabels.length;r++)
                {
                    // Reset the text in the labels and the colors
                    clearRow(r);    
                }
    
                fldLoginName.setEnabled(true);
                btnDoLoginOut.setLabel("Login");
    
                btnAddSymbol.setEnabled(false);
                btnRmvSymbol.setEnabled(false);
                
                lblStatusPrice.setText("                                    ");
                lblStatusServer.setText("                                    ");
            }
            else
            {
                showStatus("Logout failed.");
            }
            btnDoLoginOut.setEnabled(true);
        }
        
        // SERVER STATUS
        else if (n.getType() == Notification.NOTIFICATION_STATUS)
        {
            ClientInstruction ci = (ClientInstruction) n.getUserData();
            this.updateServerStatus(ci);
            
        }
        
        // PING NOTIFICATION
        else if (n.getType() == Notification.NOTIFICATION_PING)
        {
            ServerMonitor.getSelf().ping();
        }
        
        // PRICE UPDATE
        else if (n.getType() == Notification.NOTIFICATION_PRICEUPDATE)
        {
            ClientInstruction ci = (ClientInstruction) n.getUserData();
            refreshTickerCell(ci.getTickerSymbol(), ci.getBidPrice(), ci.getAskPrice(), ci.getTickerType(), ci.getTickerTime());
        }
        
	}

}
