//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.applet;

import java.awt.Label;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.ibm.websphere.samples.asynchbeans.wstrader.ClientInstruction;
import com.ibm.websphere.samples.asynchbeans.wstrader.Config;

/**
 * The WebSphere Trader Client interface to the remote HTTP server.
 * We use a singleton StreamerHandler so that all applets in a single JVM
 * only have one HTTP connection open.
 */
public class StreamerHandler extends Thread implements NotificationBroadcaster{

	/**
	 * The name of this streamer for the server.
	 */
	private String streamerName;
	
    /**
     * If true, the thread should terminate itself.
     */
    private boolean isCancelled;
    
    /**
     * A cross-reference of ticker symbol names and row numbers in the grid
     */
    private Hashtable tickerLookup;
    
    /**
     * A reference to the applet's codebase
     */
    private URL codeBase;
    
    /**
     * The default symbol list to start with when starting the stream if none active already.
     */
    private String dftSymbols;

    /**
     * The cookie to use when talking to the streamer servlet.
     */
    private String cookie;
        
    /**
     * A reference to the utility class
     */
    private Utility utility = new Utility();

    /**
     * The listeners who have subscribed to notification events from this object.
     */
    private static Vector listeners = new Vector();

    /**
     * The singleton StreamerHandler.
     */
    private static StreamerHandler self = null;    

    /**
     * Main constructor.
     * @param codeBase the URL for the Applet's code base.  It will be used to assemble all servlet URL's.
     * @param name the name of the stream
     * @param dftSymbols the symbols to use upon starting this streamer if the streamer isn't already active.
     * @param cookie the cookie recieved from login.
     */
    private StreamerHandler(URL codeBase, Hashtable tickerLookup, String name, String dftSymbols, String cookie)
    {
        isCancelled = false;
        this.codeBase = codeBase;
        this.tickerLookup = tickerLookup;
        this.streamerName = name;
        this.dftSymbols = dftSymbols;
        this.cookie = cookie;
    }

    /**
     * Create a new singleton ServerMonitor or get the currently created on if one already exists
     */
    public static synchronized StreamerHandler create(URL codeBase, Hashtable tickerLookup, String name, String dftSymbols, String cookie)
    {
        if (self == null)
        {
            self = new StreamerHandler(codeBase, tickerLookup, name, dftSymbols, cookie);
        }
        
        return self;
    }

    /**
     * Get the singleton StreamerHandler
     */
    public static synchronized StreamerHandler getSelf()
    {
        return self;
    }

    /**
     * Cancel this thread.
     */    
    public synchronized void cancel()
    {
        isCancelled = true;
    }
    
    /**
     * Retreive the name of this streamer
     */
    public String getStreamerName()
    {
    	return streamerName;
    }
    
    /**
     * True if we have been asked to cancel the thread.
     */
    public synchronized boolean getCancelled()
    {
    	return isCancelled;
    }
    
    /**
     * Create a URL object for this site using the given servletname
     */
    URL constructURL(String servletName)
        throws MalformedURLException
    {
        URL url = new URL(codeBase.getProtocol(), codeBase.getHost(), codeBase.getPort(), servletName);
        return url;
    }

    /**
     * DEBUG: Dump the header information for a URLConnection.
     */
    void dumpHeaders(URLConnection con)
    {
      int n=1; // n=0 has no key, and the HTTP return status in the value field
      boolean done = false;
      while (!done)
      {
          String headerKey = con.getHeaderFieldKey(n);
          String headerVal = con.getHeaderField(n);
          if (headerKey!=null || headerVal!=null) {
              System.out.println(headerKey+"="+headerVal);        
          } else {
          done = true;
          }
          n++;
      }            
    }
    
    /**
     * Process the input stream for our ClientInstruction objects
     * @param isr the InputStreamReader to parse.
     */
    void processInputStream(InputStreamReader isr)
    {
        // Loop over the InputStreamReader and pull out the 
        // text and find the commands.
        try 
        {
	        while(!getCancelled() && !isInterrupted())
	        {
	            
	            String line = utility.getLineFromInputStream(isr);
                
	            // Process the line by assembling a Java object from the instruction.
                // The server used this same object to assemble the line of text.
	            ClientInstruction resp = ClientInstruction.createResponseFromString(line);
	            if (resp != null)
	            {
	                String output = resp.getTextResponse();
	                if (resp.getResponseType().equals(ClientInstruction.RESPONSE_TYPE_TICKER))
	                {
                        fireNotification(Notification.NOTIFICATION_PRICEUPDATE, resp);
	                }
                    else if (resp.getResponseType().equals(ClientInstruction.RESPONSE_TYPE_STATUS))
                    {
                        if (resp.getStatusSource().equals(ClientInstruction.RESPONSE_STATUS_SOURCE_STREAMSERVER))
                        {
                            fireNotification(Notification.NOTIFICATION_PING, resp);
                        }
                        else if (resp.getStatusSource().equals(ClientInstruction.RESPONSE_STATUS_SOURCE_PRICEGEN))
                        {
                            fireNotification(Notification.NOTIFICATION_STATUS, resp);
                        }
                    }
	            }
	        }
        }
        catch(EOFException ex)
        {
            // This is fine.  The server simply ended.
        }
    }
    
    /**
     * Send a notification to all of our listeners
     */
    private void fireNotification(String type,Object userData)
    {
        for (Enumeration e = listeners.elements();e.hasMoreElements();)
        {
            NotificationListener l = (NotificationListener) e.nextElement();
            Notification n = new Notification(type, this);
            n.setUserData(userData);
            l.handleNotification(n);
        }
    }

    /**
     * Open a HTTP connection to our streaming servlet
     * and pass in our cookie that we received during login.
     * We should continually 
     */
    private void doStreamProcessing()
    {
        try 
        {        
            URL streamURL = constructURL("/asynchbeans/WebSphereTrader/stream?" + Config.CLIENT_PARAM_STREAM_NAME + "=" + java.net.URLEncoder.encode(streamerName) + "&TSTickers="+ java.net.URLEncoder.encode(dftSymbols));
            System.out.println("Connecting to: " + streamURL);
 
            // Start capturing the Ticker Streamer data
            URLConnection conStreamer = (URLConnection) streamURL.openConnection();

            conStreamer.setUseCaches(false); 
            conStreamer.setRequestProperty("Connection", "Keep-Alive");
            conStreamer.setRequestProperty("Cookie", cookie);
            conStreamer.setAllowUserInteraction(false);
            InputStream is = conStreamer.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);        

            // Process the data in our ongoing response.
            processInputStream(isr);
            
            is.close();
            System.out.println("Streamer processing complete: " + streamerName);

        }
        catch(MalformedURLException e)
        {
            System.out.println("StreamerHandler.run() threw MalformedURLException:");
            e.printStackTrace();
        }
        catch(IOException e)
        {
            System.out.println("StreamerHandler.run() threw IOException:");
            e.printStackTrace();
        }
    }
    
    /**
     * @see java.lang.Thread#run()
     */
    public void run()
    {
        setName("StreamerHandler");     
        doStreamProcessing();
    }
    /**
     * @see com.ibm.websphere.samples.asynchbeans.wstrader.applet.NotificationBroadcaster#addNotificationListener(NotificationListener)
     */
    public void addNotificationListener(NotificationListener listener)
    {
        listeners.addElement(listener);
        
        // If this thread isn't started, do so.
        if (listeners.size() == 1)
        {
            start();
        }
    }

    /**
     * @see com.ibm.websphere.samples.asynchbeans.wstrader.applet.NotificationBroadcaster#removeNotificationListener(NotificationListener)
     */
    public void removeNotificationListener(NotificationListener listener)
    {
        listeners.removeElement(listener);

        // If there are no more listeners, cancel ourselves.
        if(listeners.size() == 0)
        {
            _destroy();
        }    
    }

    /**
     * Destroy this instance of the ServerMonitor.
     */
    private synchronized void _destroy()
    {
        // Cancel our thread and then reset our static variables
        cancel();
        self = null;
    }

}
