//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.pricegen;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Various configuration information and constants for the application.
 */
public class PriceGenConfig 
{
    
    /**
     * The name of the Asynch Bean used for this Price Generator
     */
    public final static String AS_NAME = "AS_PRICEGEN";
    
    /**
     * The file name for the Price Generator properties file
     */
    private final static String _PropsFileName = "PriceGenerator.prop";
    
    /**
     * The header for the properties file when it is created 
     */
    private final static String _PropsFileHeader = "WebSphere Trader Price Generator Properties";
    
    /**
     * The cached properties file
     */
    private static Properties _AppProps = null;

    /**
     * If true, automatically start the price generator when the application starts.
     */
    public static String REGISTRY_KEY_PRICEGEN_AUTOSTART = "pricegen.autostart";
    
    
    /**
     * Retrieve the application's properties file.
     * @return Properties
     */
    private synchronized static Properties getAppProperties() 
        throws IOException, FileNotFoundException
    {
        // Load the properties once.
        if(_AppProps == null) {
            ClassLoader cl = Thread.currentThread().getContextClassLoader();
            InputStream is = cl.getResourceAsStream(_PropsFileName);
            if (is == null)
            {
                throw new FileNotFoundException(_PropsFileName);
            }
            _AppProps = new Properties();
            _AppProps.load(is);
        }
        return _AppProps;
    }
    
    /**
     * Retrive a property from the application's configuration file.
     * @param key Property to retrieve
     * @param dftValue Default value to return if the property could not be retrieved.
     * @return Returns the value of the property or the default value.
     */
    public synchronized static String getAppProperty(String key, String dftValue) 
        throws IOException, FileNotFoundException
    {
        Properties p = getAppProperties();
        if (p == null)
        {
            return dftValue;
        }
        return p.getProperty(key,dftValue);
        
    }    
    
    
}
