//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.pricegen.ejb;

import javax.ejb.SessionBean;
import javax.naming.InitialContext;

import com.ibm.websphere.samples.asynchbeans.wstrader.pricegen.PriceGenConfig;



/**
 * This is a startup bean that kicks off the price publishers when
 * the enterprise application starts. 
 */
public class PriceGenBootstrapBean implements SessionBean
{
		
	private javax.ejb.SessionContext mySessionCtx;
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}

	public void ejbPassivate() 
	{
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}


	/**
	 * Start the Price Publishers when the application starts.
	 */	
	public boolean start()
	{
		try
		{
            // Check our configuration and see if we want to auto-start the price generator or not
            String autostart = PriceGenConfig.getAppProperty(PriceGenConfig.REGISTRY_KEY_PRICEGEN_AUTOSTART, "false");
            
            if(autostart.toLowerCase().equals("true"))
            {
                System.out.println("WebSphereTrader: Starting-up the WebSphere Trader Price Generator");
                
	            // Get a local reference to our PriceGenControlBean.
	            InitialContext ic = new InitialContext();
	            PriceGenControlLocalHome cbHome = (PriceGenControlLocalHome) ic.lookup("java:comp/env/ejb/PriceGenControl");
	            PriceGenControlLocal cb = cbHome.create();
	    
	            cb.startPublishers();
            }
            else
            {
                System.out.println("WebSphereTrader: Autostart of the  WebSphere Trader Price Generator has been disabled.");
            }
		}
		catch(Exception e)
		{
			System.out.println("WebSphereTrader: PriceGenBootstrapBean.start(): Exception starting app: " + e.toString());
            e.printStackTrace();
			return false;
		}
		return true;
	}
	
	/**
	 * This stops the publishers when the application stops.
	 */
	public void stop()
	{
		try
		{
            System.out.println("WebSphereTrader: Stopping the WebSphere Trader Price Generator");

            // Get a local reference to our PriceGenControlBean.
            InitialContext ic = new InitialContext();
            PriceGenControlLocalHome cbHome = (PriceGenControlLocalHome) ic.lookup("java:comp/env/ejb/PriceGenControl");
            PriceGenControlLocal cb = cbHome.create();
            
            cb.stopPublishers();
		}
		catch(Exception e)
		{
            System.out.println("WebSphereTrader: PriceGenBootstrapBean.stop(): Exception stopping app " + e.toString());
		}
	}
}
