//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.pricegen.web;

import java.io.IOException;
import java.io.PrintWriter;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.asynchbeans.WorkManager;
import com.ibm.websphere.samples.asynchbeans.wstrader.pricegen.ejb.PriceGenControl;
import com.ibm.websphere.samples.asynchbeans.wstrader.pricegen.ejb.PriceGenControlHome;

/**
 * Price Generator Control Servlet allows starting and stopping
 * the Price Generator
 */
public class ControllerServlet extends HttpServlet 
{


	/**
	 * Constructor for TickerStreamerServlet.
	 */
	public ControllerServlet() 
    {
		super();
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest, HttpServletResponse)
	 */
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException 
    {
        doProcess(req, resp);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest, HttpServletResponse)
	 */
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException 
    {
        doProcess(req,resp);
	}
    

    /**
     * This returns the WorkManager bound with this session bean.
     */
    WorkManager getWorkManager(InitialContext ic)
        throws NamingException
    {
        WorkManager wm = (WorkManager)ic.lookup("java:comp/env/wm/WorkManager");
        return wm;
    }
    
    /**
     * Process all client requests
     * @param req the client request
     * @param resp the response to the client
     */
    protected void doProcess(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException 
    {
        PrintWriter pw = resp.getWriter();

        try 
        {
            if (!resp.isCommitted()) 
            {
                resp.setHeader("Pragma", "No-cache");
                resp.setHeader("Cache-Control", "no-cache");
                resp.setDateHeader("Expires",0);
                resp.setContentType("text/html");
            }


            // Show the page using the WebSphere Samples UI
            pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
            pw.println("<!--");
            pw.println("\"This program may be used, executed, copied, modified and distributed without royalty for the purpose of developing, using, marketing, or distributing.\"");
            pw.println();
            pw.println("Product 5630-A37,  (C) COPYRIGHT International Business Machines Corp., 2001, 2004");
            pw.println("All Rights Reserved * Licensed Materials - Property of IBM");
            pw.println("-->");

            pw.println("<HTML>");
            pw.println("<HEAD>");
            pw.println("<META http-equiv=\"Content-Type\" CONTENT=\"text/html; charset=iso-8859-1\">");
            pw.println("<TITLE>WebSphere Trader Price Generator Control</TITLE>");
            pw.println("<script language=\"JavaScript\" src=\"/WSsamples/scripts/browser_detection.js\"></script>");
            pw.println("<NOSCRIPT>");
            pw.println("Proper display of samples requires JavaScript to be enabled.");
            pw.println("</NOSCRIPT>");
            pw.println("</HEAD>");
            pw.println("<BODY leftmargin=0 topmargin=0 marginwidth=0 marginheight=0>");



            // Show the menu
            pw.println("<img src=\"/WSsamples/images/success_banner.jpg\" valign=top alt=\"WebSphere Samples\">");
            pw.println("<table cellpadding=5 cellspacing=5>");
            pw.println("<tr><td><br>");
            pw.println("<H1 class=\"sampjsp\">Asynchronous Beans - WebSphere Trader Price Generator Control</H1>");
            pw.println("</td></tr></table>");
            
            pw.println("<HR width=100%>");
            pw.println("<table cellpadding=5 cellspacing=5>");
            pw.println("<tr><td>");
            
            pw.println("<p>Start or Stop the Price Generator publishers by clicking one of the following:</p>");
            pw.println("<form name=pgcontrol id=pgcontrol method=get>");
            pw.println("<input name=\"action\" type=\"submit\" value=\"Start Price Generation\">");
            pw.println("<input name=\"action\" type=\"submit\" value=\"Stop Price Generation\">");
            pw.println("</form>");
            pw.println("<p>&nbsp;");



            // Get the command from the user.  Valid commands are:
            // action=[start/stop]

	        // Retrieve the login information from the client.
	        String action= req.getParameter("action");
            
            if (action != null && action.length() > 0) 
            {
                action = action.toLowerCase();

                // Get a remote reference to our PriceGenControlBean.
                InitialContext ic = new InitialContext();
                Object cbHomeObj = ic.lookup("java:comp/env/ejb/PriceGenControl");
                PriceGenControlHome cbHome = (PriceGenControlHome) javax.rmi.PortableRemoteObject.narrow(cbHomeObj, PriceGenControlHome.class);
                PriceGenControl priceGenControl = cbHome.create();
		        
	            if (action.indexOf("start") == 0)
	            {
	                // Start the Price Generation publishers.
                    if (priceGenControl != null)
                    {
	                    if(priceGenControl.startPublishers())
                        {
    	                    pw.println("<p>The Price Generators have been started.");                   
                        }
                        else
                        {
                            pw.println("<p>The Price Generators have already been started.");                   
                        }
                    }
                    else
                    {
                        pw.println("<p>Unable to start the Price Generators.  Check the server log for details.");
                    }
	            }
	            else if (action.indexOf("stop") == 0)
	            {
                    // Stop the Price Generation publishers.
                    if (priceGenControl != null)
                    {
	                    if(priceGenControl.stopPublishers())
                        {
    	                    pw.println("<p>The Price Generators have been stopped.");                   
                        }
                        else
                        {
                            pw.println("<p>The Price Generators have already been stopped.");                   
                        }
                    }
                    else
                    {
                        pw.println("<p>Unable to stop the Price Generators.  Check the server log for details.");
                    }
	            }
            }
                
        }
        catch(Exception e)
        {
            pw.println("<font color=\"red\">");
            pw.println("An unexpected exception occurred:");
            e.printStackTrace(pw);
            pw.println("</font>");
        }
        finally 
        {
            pw.println("</td></tr></table>");
            pw.println("</body></html>");
        }
        
    }

	/**
	 * @see javax.servlet.GenericServlet#init()
	 */
	public void init() throws ServletException
	{
		super.init();
        
	}

}
