//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.tickstream;

import java.util.LinkedList;

/**
 * A queue of the most current objects.  All old ones
 * drop off the front when the queue gets full.  In addition,
 * when the queue is empty, the deQueue method blocks until there
 * are items added to the queue.
 */
public class CurrentQueue
{
    LinkedList entries;
    int maxSize;
    boolean isClosed;
    
    /**
     * Create a queue that can have maxSize elements in it.
     * 
     */
    public CurrentQueue(int maxSize)
    {
        entries = new LinkedList();
        this.maxSize = maxSize;
        isClosed = false;
    }
    
    /**
     * Remove an item from the front of the queue.  If the 
     * queue is empty, just wait.
     * @param timeoutMS The maximum time to wait.
     */
    public synchronized Object deQueue(int timeoutMS)
    {
        if(entries.isEmpty())
        {
            try {
                wait(timeoutMS);
            }
            catch (InterruptedException e){
                
            }
        }
        if (entries.isEmpty() || isClosed)
        {
            return null;
        }
        return entries.removeFirst();
    }
    
    /**
     * Add an item to the end of the queue.  If the 
     * number of items have reaced maxSize, then
     * remove and item from the front.
     * Wake-up when finished.
     */
    public synchronized void enQueue(Object o)
    {
        if(entries.size() == maxSize)
        {
            entries.removeFirst();
        }
        entries.addLast(o);
        notify();
    }
    
	/**
	 * Rreturn the first item from the queue.
	 */
	public Object peekFirst() 
    {
        if (entries.size() >0) 
        {
    		return entries.getFirst();
        }
        else
        {
            return null;
        }
	}

	/**
	 * Return the last item fromt he queue.
	 */
	public Object peekLast() 
    {
        if (entries.size() >0) 
        {
            return entries.getLast();
        }
        else
        {
            return null;
        }
        
	}

	/**
	 * Return the size of the queue
	 */
	public long size() 
    {
        return entries.size();
	}

    /**
     * Close down the queue and stop all blocking
     */    
    public synchronized void close()
    {
        isClosed = true;
        notify();
    }

	public synchronized boolean isClosed()
	{
		return isClosed;
	}
}
