//
// "This program may be used, executed, copied, modified and distributed without royalty for the
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.tickstream;

import java.util.logging.*;
import com.ibm.websphere.asynchbeans.Alarm;
import com.ibm.websphere.asynchbeans.AlarmListener;
import com.ibm.websphere.asynchbeans.EventSource;
import com.ibm.websphere.samples.asynchbeans.wstrader.ClientInstruction;

/**
 * A simple alarm that fires StreamHeartBeatEvent events to all listeners of
 * the input EventSource.  The purpose of this alarm is to
 * ultimately send Ping messages to our clients on a regular basis.  They can
 * therefore have a better understanding of the state of their connection.
 */
public class StreamHeartBeatAlarm implements AlarmListener
{
    static final Logger logger = Logger.getLogger(StreamHeartBeatAlarm.class.getName());
    static final String className = StreamHeartBeatAlarm.class.getName();
    int interval;
    StreamHeartBeatEvents eventProxy;

    /**
     * Construct a new StreamHeartBeatAlarm listener
     * @param es the EventSource used to publish our ping events on.
     * @param interval_ms the number of milliseconds between pings.
     */
    StreamHeartBeatAlarm(EventSource es, int interval_ms)
    {
        logger.logp(Level.FINE, className, "StreamHeartBeatAlarm", new Integer(interval_ms).toString());
        interval = interval_ms;
        eventProxy = (StreamHeartBeatEvents)es.getEventTrigger(StreamHeartBeatEvents.class);
    }

    /**
     * @see com.ibm.websphere.asynchbeans.AlarmListener#fired(Alarm)
     */
    public void fired(Alarm alarm)
    {
        String methodName = "fired";
        logger.entering(className, methodName);

        // Always send an alive instruction.  The client will ignore this anyway when it's
        // a STREAMSERVER type, since they will simply be monitoring for any status at a
        // given interval.
        eventProxy.handleHeartBeat(ClientInstruction.createStatusResponse(ClientInstruction.RESPONSE_STATUS_SOURCE_STREAMSERVER, ClientInstruction.RESPONSE_STATUS_VALUE_ALIVE));

		try
		{
        	alarm.reset(interval);
		}
		catch(IllegalStateException e)
		{
			// swallow it to avoid any exception traces. This is expected
			// if the alarm fires during or after the owning AsynchScope is destroyed.
		}
    }

}
