//
// "This program may be used, executed, copied, modified and distributed without royalty for the
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.ejb;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import java.util.logging.*;
import com.ibm.websphere.asynchbeans.WorkManager;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.StreamApplicationState;

/**
 * Bean implementation class for Enterprise Bean: PriceGenHeartBeatMDB
 */
public class PriceGenHeartBeatMDBBean implements javax.ejb.MessageDrivenBean, javax.jms.MessageListener
{
    static final Logger logger = Logger.getLogger(PriceGenHeartBeatMDBBean.class.getName());
    static final String className = PriceGenHeartBeatMDBBean.class.getName();

    private javax.ejb.MessageDrivenContext fMessageDrivenCtx;
    private WorkManager wm;

	/**
	 * getMessageDrivenContext
	 */
	public javax.ejb.MessageDrivenContext getMessageDrivenContext()
    {
		return fMessageDrivenCtx;
	}

	/**
	 * setMessageDrivenContext
	 */
	public void setMessageDrivenContext(javax.ejb.MessageDrivenContext ctx)
    {
		fMessageDrivenCtx = ctx;
	}

	/**
	 * ejbCreate
	 */
	public void ejbCreate()
    {
        String methodName = "ejbCreate";
        logger.entering(className, methodName);

        try
        {
            InitialContext ic = new InitialContext();

            wm = (WorkManager)ic.lookup("java:comp/env/wm/WorkManager");
        }
        catch(NamingException e)
        {
            logger.logp(Level.SEVERE, className, methodName, "Naming exception initializing PriceGenHeartBeatMDBBean", e);
        }
        finally
        {
            logger.exiting(className, methodName);
        }
	}

	/**
	 * onMessage
	 */
	public void onMessage(Message msg)
    {
        String methodName = "onMessage";
        logger.entering(className, methodName);

        try
        {
            // For each ping message we get, call the Ticker Streamer's Application State Asynch Bean
            // and ping it's subsystem monitor
            if (msg.getStringProperty("type").equals("ping"))
            {
                // Lookup the application state.  If it's null, that's OK... There simply isn't anyone
                // there to listen for it.  The application must be shutting down or hasn't been started yet.
                StreamApplicationState appState = StreamApplicationState.getApplicationState(wm, false);
                if (appState != null)
                {
                    logger.logp(Level.FINE, className, methodName, "Calling ping() " + msg.getStringProperty("time"));
                    appState.handlePriceGenPing();
                }
                else
                {
                    logger.logp(Level.FINE, className, methodName, "ApplicationState is null.");
                }
            }
        }
        catch(NamingException e)
        {
            logger.logp(Level.SEVERE, className, methodName, "Naming exception", e);
        }
        catch(JMSException e)
        {
            logger.logp(Level.SEVERE, className, methodName, "JMSException in onMessage()", e);
        }
	}

	/**
	 * ejbRemove
	 */
	public void ejbRemove()
    {
	}
}
