//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.ejb;

import javax.ejb.SessionBean;
import javax.naming.InitialContext;

import com.ibm.websphere.asynchbeans.WorkManager;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.StreamApplicationState;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.TickStreamConfig;



/**
 * This is a startup bean that starts and stops the Ticker Streamer
 * application when the server starts.
 */
public class TickerStreamBootstrapBean implements SessionBean
{
		
	private javax.ejb.SessionContext mySessionCtx;
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}

	public void ejbPassivate() 
	{
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}


	/**
	 * Start the Ticker Streamer Application when the application starts.
	 */	
	public boolean start()
	{
		try
		{
            // Check our configuration and see if we want to auto-start the ticker streamer application or not
            String autostart = TickStreamConfig.getAppProperty(TickStreamConfig.REGISTRY_KEY_TICKSTREAM_AUTOSTART,  "false");
            
            if(autostart.toLowerCase().equals("true"))
            {
                System.out.println("WebSphereTrader: Starting-up the WebSphere Trader Ticker Streamer");
                InitialContext ic = new InitialContext();
                WorkManager wm = (WorkManager)ic.lookup("java:comp/env/wm/WorkManager");

                // Start-up the StreamApplicationState by creating one.  It will
                // exist in an AsynchBean, so we don't need to do anything with it here.  Just
                // call the get method.
                StreamApplicationState.getApplicationState(wm, true);
                
            }
            else
            {
                System.out.println("WebSphereTrader: Autostart of the  WebSphere Trader Ticker Streamer has been disabled.");
            }
		}
		catch(Exception e)
		{
			System.out.println("WebSphereTrader: TickerStreamBootstrapBean.start(): Exception starting app: " + e.toString());
            e.printStackTrace();
			return false;
		}
		return true;
	}
	
	/**
	 * This stops the Ticker Streamer Application when the application stops
	 */
	public void stop()
	{
		try
		{
            System.out.println("WebSphereTrader: Stopping the WebSphere Trader Ticker Streamer");
            InitialContext ic = new InitialContext();
            WorkManager wm = (WorkManager)ic.lookup("java:comp/env/wm/WorkManager");

            // Destroy the StreamApplicationState by looking up the existing one. If one is returned,
            // then destroy it.  This will effectively stop all of the running resources
            // of this application
            StreamApplicationState appState = StreamApplicationState.getApplicationState(wm, false);
            if (appState != null)
            {
                appState.destroy();
            }
		}
		catch(Exception e)
		{
            System.out.println("WebSphereTrader: stop(): Exception stopping app " + e.toString());
		}
	}
}
