//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.web;

import javax.jms.JMSException;
import javax.naming.*;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.util.StringTokenizer;
import java.util.logging.*;

import com.ibm.websphere.asynchbeans.WorkManager;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.StreamApplicationState;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.StreamClientState;

/**
 * @author bnewport
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 */
public class BaseAsynchServlet extends HttpServlet 
{
      static final Logger logger = Logger.getLogger(BaseAsynchServlet.class.getName());
      static final String className = BaseAsynchServlet.class.getName();

	StreamApplicationState appState;

    /**
     * Initialize the servlet.   Just get a reference to our Application State object.
     * @see javax.servlet.GenericServlet#init()
     */
	public void init()
	{
            String methodName = "init";
            logger.entering(className, methodName);

		try
		{
			InitialContext ic = new InitialContext();
			
                  WorkManager wm = (WorkManager)ic.lookup("java:comp/env/wm/WorkManager");
			appState = StreamApplicationState.getApplicationState(wm, true);
		}
		catch(JMSException e)
		{
                  logger.logp(Level.SEVERE, className, methodName, "JMSException creating application state", e);
		}
		catch(NamingException e)
		{
                  logger.logp(Level.SEVERE, className, methodName, "Naming exception initializing servlet", e);
		}
		finally
		{
			logger.exiting(className, methodName);
		}
	}
	
	public StreamClientState getClientState(String loginid)
	{
		return StreamClientState.getClientState(appState, loginid, false);   
	}

	public StreamClientState createClientState(String loginid)
	{
		return StreamClientState.getClientState(appState, loginid, true);   
	}
	
	public void setTextResponse(HttpServletResponse resp, boolean flushbuffer)
		throws IOException
	{
        if (!resp.isCommitted()) 
        {
            resp.setHeader("Pragma", "No-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setDateHeader("Expires",0);
            resp.setContentType("text/plain");
            if (flushbuffer)
            {
                resp.flushBuffer();
            }
        }        
	}

    String getReqParameter(HttpServletRequest req, String name)
    {
        String rc = req.getParameter(name);
        if (rc == null) rc = "";

        String methodName = "getReqParameter";
        logger.logp(Level.FINER, className, methodName, name + " = " + rc);
        return rc;
    }

    /**
     * Returns a cookie value for a given array of cookies, a cookie name and a key.
     */
    public String getCookieValue(Cookie cookies[], String cookieName, String cookieKey)
    {

        String methodName = "getCookieValue";

        if (cookies != null) 
        {
            logger.logp(Level.FINE, className, methodName, "Total cookies: " + cookies.length);
            for(int i = 0;i < cookies.length;i++)
            {
                Cookie cookie = cookies[i];
                if (cookie.getName().equals(cookieName))
                {
                    logger.logp(Level.FINE, className, methodName, "Cookie: " + cookieName + " found");

                    // Our cookie value is a combination of 
                    // key=value,[key=value] pairs
                    StringTokenizer stCookie = new StringTokenizer(cookie.getValue(), ",");

                    while (stCookie.hasMoreTokens())
                    {
                        StringTokenizer kvPair = new StringTokenizer(stCookie.nextToken(), "=");
                        String key = kvPair.nextToken();
                        String value = kvPair.nextToken();
                            
                        if (key.equals(cookieKey))
                        {
                            logger.logp(Level.FINE, className, methodName, cookieKey + " = " + value);
                            return value;
                        }
                    }
                    break;
                }
            }
        }  
        else
        {
        if(logger.isLoggable(Level.FINE))
            logger.logp(Level.FINE, className, methodName, "Total cookies: null");
        }

        if(logger.isLoggable(Level.FINE))
            logger.logp(Level.FINE, className, methodName, cookieKey + " = null");

        return null;      
    }

}
