//
// "This program may be used, executed, copied, modified and distributed without royalty for the
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.web;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.logging.*;
import com.ibm.websphere.samples.asynchbeans.wstrader.ClientInstruction;
import com.ibm.websphere.samples.asynchbeans.wstrader.Config;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.StreamClientState;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.StreamListener;

/**
 * Handle TickerStreamer login requests
 */
public class LoginServlet extends BaseAsynchServlet
{
      static final Logger logger = Logger.getLogger(LoginServlet.class.getName());
      static final String className = LoginServlet.class.getName();

	/**
	 * Constructor for TickerStreamerServlet.
	 */
	public LoginServlet()
    {
		super();
	}

    /**
     * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest, HttpServletResponse)
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException
    {
        doProcess(req, resp);
    }

    /**
     * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest, HttpServletResponse)
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException
    {
        doProcess(req,resp);
    }

    /**
     * Process all client requests
     * @param req the client request
     * @param resp the response to the client
     */
    protected void doProcess(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException
    {
        String methodName = "doProcess";
        logger.entering(className, methodName);
        PrintWriter pw = resp.getWriter();

        try
        {
            // Setup our text response, but don't flush the buffer
        	setTextResponse(resp, false);

	        // Retrieve the login information from the client.
	        String loginid = getReqParameter(req, Config.CLIENT_PARAM_CLIENTID);

                if(loginid.length() > 0)
                {
                logger.logp(Level.FINE, className, methodName, "Client ID=" + loginid);

                // Create a Client State object for our login id.
                // If we are able to get an existing StreamClientState, then
                // the client is re-logging in.
                StreamClientState state = getClientState(loginid);

                String tickers = null;

                if (state != null)
                {
                    // See if the default streamer for this client already has some
                    // defined symbols from a disconnected session
                    // tickers = state.getStreamerSubscription("DEFAULT");
                    StreamListener streamer = state.findStreamer("DEFAULT");
                    tickers = streamer.getSubscribedTickers();
                    logger.logp(Level.FINE, className, methodName, "Existing tickers=" + tickers);

                    // Destroy our ClientState and recreate
                    state.destroy();

                }

                state = createClientState(loginid);

	            // Return a cookie to the client with the login information.
	            Cookie cookie = new Cookie(Config.CLIENT_COOKIE_NAME, Config.CLIENT_PARAM_CLIENTID + "=" + loginid);
	            resp.addCookie(cookie);

	            // Print our results
                pw.print(ClientInstruction.createCommandStatusResponse(ClientInstruction.RESPONSE_COMMAND_LOGIN, ClientInstruction.RESPONSE_COMMANDSTATUS_SUCCESS, tickers).getTextResponse());
			}
            else
            {
                  logger.logp(Level.FINE, className, methodName, "Invalid Client ID");
	            pw.print(ClientInstruction.createCommandStatusResponse(ClientInstruction.RESPONSE_COMMAND_LOGIN, ClientInstruction.RESPONSE_COMMANDSTATUS_FAIL).getTextResponse());
            }
        }
        catch(Exception e)
        {
            pw.print(ClientInstruction.createCommandStatusResponse(ClientInstruction.RESPONSE_COMMAND_LOGIN, ClientInstruction.RESPONSE_COMMANDSTATUS_FAIL).getTextResponse());
            pw.print(ClientInstruction.createErrorResponse(e.toString()).getTextResponse());
            logger.logp(Level.SEVERE, className, methodName, "doProcess", e);
        }
    	finally
    	{
    		logger.exiting(className, methodName);
    	}
    }
}
