//
// "This program may be used, executed, copied, modified and distributed without royalty for the
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.web;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.logging.*;

import com.ibm.websphere.samples.asynchbeans.wstrader.ClientInstruction;
import com.ibm.websphere.samples.asynchbeans.wstrader.Config;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.StreamClientState;
import com.ibm.websphere.samples.asynchbeans.wstrader.tickstream.StreamListener;

/**
 * Servlet used to update a current streamer's ticker symbols.
 */
public class UpdateServlet extends BaseAsynchServlet
{
    static final Logger logger = Logger.getLogger(UpdateServlet.class.getName());
    static final String className = UpdateServlet.class.getName();

	/**
	 * Constructor for TickerStreamerServlet.
	 */
	public UpdateServlet()
    {
		super();
	}

    /**
     * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest, HttpServletResponse)
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException
    {
        doProcess(req, resp);
    }

    /**
     * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest, HttpServletResponse)
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException
    {
        doProcess(req,resp);
    }

    /**
     * Process all client requests
     * @param req the client request
     * @param resp the response to the client
     */
    protected void doProcess(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException
    {
        String methodName = "doProcess";
        logger.entering(className, methodName);

        PrintWriter pw = resp.getWriter();
        try
        {
        	setTextResponse(resp, true);

	        // Get the client ID from the user and the updated tickers
	        String loginid = getCookieValue(req.getCookies(), Config.CLIENT_COOKIE_NAME, Config.CLIENT_PARAM_CLIENTID);

            // If we get a new string of tickers, then replace the existing list.
	        String tickers = getReqParameter(req, Config.CLIENT_PARAM_TICKERS);

            // If our tickers are empty, check for the add or remove
            String addTicker = getReqParameter(req, Config.CLIENT_PARAM_ADDTICKER);

            String rmvTicker = getReqParameter(req, Config.CLIENT_PARAM_RMVTICKER);

			String streamName = getReqParameter(req, Config.CLIENT_PARAM_STREAM_NAME);

	        // Find the asynch bean
	        StreamClientState clientState = StreamClientState.getClientState(appState, loginid, false);

	        if (clientState == null)
	        {
                logger.logp(Level.FINE, className, methodName, "Invalid client id");
	            pw.print(ClientInstruction.createErrorResponse(ClientInstruction.RESPONSE_ERRORMSG_NOLOGIN));
	            return;
	        }

			if(streamName.length() == 0)
			{
                logger.logp(Level.FINE, className, methodName, "Invalid stream name");
                pw.print(ClientInstruction.createErrorResponse(ClientInstruction.RESPONSE_ERRORMSG_NOSTREAMER));
                return;
			}

			StreamListener listener = clientState.findStreamer(streamName);
			if(listener == null)
			{
                logger.logp(Level.FINE, className, methodName, "Unable to find streamer for client", new Object[]{loginid, streamName});
                pw.print(ClientInstruction.createErrorResponse(ClientInstruction.RESPONSE_ERRORMSG_NOSTREAMER));
                return;
			}
			else
			{
                logger.logp(Level.FINE, className, methodName, "Updating tickers", new Object[]{loginid, tickers, streamName});
				listener.updateTickers(tickers, false);
			}

            // Print our results
            pw.print(ClientInstruction.createCommandStatusResponse(ClientInstruction.RESPONSE_COMMAND_UPDATE, ClientInstruction.RESPONSE_COMMANDSTATUS_SUCCESS).getTextResponse());
        }
        catch (Exception e)
        {
            pw.print(ClientInstruction.createCommandStatusResponse(ClientInstruction.RESPONSE_COMMAND_UPDATE, ClientInstruction.RESPONSE_COMMANDSTATUS_FAIL).getTextResponse());
            pw.print(ClientInstruction.createErrorResponse(e.toString()).getTextResponse());
            logger.logp(Level.SEVERE, className, methodName, "Exception processing client request: " + e);
        }
        finally
        {
        	pw.flush();
            logger.exiting(className, methodName);
        }
    }
}
