//-----------------------------------------------------------------------------------
// "This program may be used, executed, copied, modified and distributed without 
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//-----------------------------------------------------------------------------------

package com.ibm.websphere.samples.i18nctx.currencyexchangeejb;

import java.rmi.RemoteException;
import java.util.Properties;
import javax.ejb.*;
import javax.ejb.EJBException;

/**
 * An Entity Bean class with CMP (2.0) fields.
 */
public abstract class CustomerBean implements EntityBean {
    /*
     * cmp fields account, balance, name, password will
     * be specified in the deployment descriptor.
     *
     * NOTE: we do not implement ejbCreate and ejbPostCreate methods 
     * since we do not insert or delete a row into the database.
     */

    // Access methods for persistent fields
    abstract public String getAccount (); 
    abstract public void setAccount (String account);
    abstract public double getBalance ();  
    abstract public void setBalance (double newValue); 
    abstract public java.lang.String getName (); 
    abstract public void setName (java.lang.String newValue);  
    abstract public java.lang.String getPassword (); 
    abstract public void setPassword (java.lang.String newValue); 
  
    private final static long serialVersionUID = 3206093459760846163L;
    private javax.ejb.EntityContext entityContext = null;

    // EntityBean  methods

    /**
     *
    public String ejbCreate (String account, String name, String password, double balance) 
        throws CreateException {
        setAccount(account);
        setName(name);
        setPassword(password);
        setBalance(balance);
        return null;
    }
     */
    /**
     *
    public void ejbPostCreate (
            String account, String name, String password, double balance) 
        throws CreateException {}
     */
    public javax.ejb.EntityContext getEntityContext () {
        return entityContext;
    }
    public void setEntityContext (javax.ejb.EntityContext ctx) {
        entityContext = ctx;
    }
    public void unsetEntityContext () {
        entityContext = null;
    }
    public void ejbRemove () {}    
    public void ejbLoad () {}    
    public void ejbStore () {}    
    public void ejbPassivate () {}    
    public void ejbActivate () {}
    
    // Business methods
    
    /**
     * Add depositAmt to Customer balance in database
     * @param depositAmt double
     * @exception javax.ejb.EJBException
     */
    public void deposit(double depositAmt) throws EJBException {
        if (depositAmt <= 0)
          throw new EJBException ("Amount must be greater than zero.");
        
        double tempBalance = getBalance ();
        tempBalance = tempBalance + depositAmt;
        setBalance (tempBalance);
    }
    
    /**
     * Substract withdrawAmt from Customer balance in database
     * @param withdrawAmt double
     * @exception javax.ejb.EJBException
     */
    public void withdraw (double withdrawAmt) throws EJBException {
        double tempBalance = getBalance ();
    
        if (withdrawAmt <= 0)
          throw new EJBException ("Amount must be greater than zero.");
        if (withdrawAmt > tempBalance)
          throw new EJBException ("Transaction failed due to insufficient funds.");
        tempBalance = tempBalance - withdrawAmt;
        setBalance (tempBalance);
    }
}
