//-----------------------------------------------------------------------------------
// "This program may be used, executed, copied, modified and distributed without 
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//-----------------------------------------------------------------------------------

package com.ibm.websphere.samples.i18nctx.currencyexchangeejb;

import java.rmi.RemoteException;
import java.util.Properties;
import javax.ejb.*;
import javax.naming.*;
import java.io.*;
import java.util.*;
import java.text.DecimalFormat;
import com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.Resources;
import com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.AccInfoJBean;
import com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.CustomerJBean;

/**
 * A remote Session Bean.
 */
public class ExchangeBean implements SessionBean {
  
    // EJB(s)
    protected LocalCustomerHome customerHome = null;
 
    // JNDI name for Customer EJB
    public static final String CUSTOMER_EJBHOME = "java:comp/env/ejb/Customer";

    /********* Session EJB lifecycle callback methods *********/

    public void ejbActivate ()  {}
    public void ejbPassivate () {}    

    public void ejbCreate() throws javax.ejb.CreateException {
        try {
            customerHome = lookupCustomer ();
        } catch (NamingException ex) {
            throw new CreateException (ex.getMessage ());
        }
    }

    public ExchangeBean () {}
    public void ejbRemove () {}

    public void setSessionContext (javax.ejb.SessionContext ctx) {}

    private LocalCustomerHome lookupCustomer () throws NamingException {
        Context initialContext = new InitialContext ();

        // Lookup the Customer local home interface using the appropriate JNDI name.  
        java.lang.Object localHomeObject = initialContext.lookup (CUSTOMER_EJBHOME);
        return (LocalCustomerHome)localHomeObject;
    }

    //Business methods

    /**
     * Verify Customer's principal
     * @return com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.CustomerJBean
     * @param account java.lang.String
     * @param password java.lang.String
     */
    public CustomerJBean verifyPrincipal (String account, String password) {
        LocalCustomer customer = null;
        AccInfoJBean accInfo = null;
        CustomerJBean customerInfo = null;
        try {
            customer = customerHome.findByPrincipal(account, password);
            accInfo = new AccInfoJBean(customer);
            customerInfo = accInfo.getAccountInfo();
        } catch (Exception e) {
            // throw new EJBException(e);
            //throw new RemoteException("Login failed: " + e);
            return customerInfo;
        }
        return customerInfo;
    }

    /**
     * Add depositAmt to Customer balance when exchange other currencies 
     * to US dollars.
     * @return com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.CustomerJBean
     * @param account java.lang.String
     * @param depositAmt double
     */
    public CustomerJBean deposit (String account, double depositAmt) {
        AccInfoJBean accInfo = null;
        CustomerJBean customerInfo = null;
        LocalCustomer customer = null;  
        try {
            customer = customerHome.findByPrimaryKey(account);
            customer.deposit(depositAmt);
            accInfo = new AccInfoJBean(customer);
            customerInfo = accInfo.getAccountInfo();
        } catch (Exception e) {
            // throw new EJBException(e);
            //throw new RemoteException("Transaction failed due to: " + e);
           return  customerInfo;
        }
        return  customerInfo;
    }

    /**
     * Withdraw withdrawAmt from Customer balance when exchange 
     * US dollars to other currencies
     * @return com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.CustomerJBean
     * @param account java.lang.String
     * @param withdrawAmt double
     */
    public CustomerJBean withdraw (String account, double withdrawAmt) {
        AccInfoJBean accInfo = null;
        CustomerJBean customerInfo = null;
        LocalCustomer customer = null;
        try {
            customer = customerHome.findByPrimaryKey(account);
            customer.withdraw(withdrawAmt);
            accInfo = new AccInfoJBean(customer);
            customerInfo = accInfo.getAccountInfo();
        } catch (Exception e) {
            return customerInfo;
        } 
        return customerInfo;
    }

    /**
     * update the customer information when the user want to do more exchange
     * @return com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.CustomerJBean
     * @param account java.lang.String
     */
    public CustomerJBean updateCustomer (String account) {
        AccInfoJBean accInfo = null;
        CustomerJBean customerInfo = null;
        LocalCustomer customer = null;
        try {
            customer = customerHome.findByPrimaryKey(account);
            accInfo = new AccInfoJBean(customer);
            customerInfo = accInfo.getAccountInfo();
        } catch (Exception e) {
            // throw new EJBException(e);
            //throw new RemoteException("Transaction failed due to: " + e);
            return  customerInfo;
        }
        return  customerInfo;
    }

}
