//-----------------------------------------------------------------------------------
// "This program may be used, executed, copied, modified and distributed without 
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//-----------------------------------------------------------------------------------

package com.ibm.websphere.samples.i18nctx.currencyexchangelibrary;

import java.text.MessageFormat;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * Class Resources provides support for retrieving messages from
 * the sampi18nctx resource bundle(s).
 */
public class Resources {
    private static ResourceBundle bundle = null;
    
    public static final String SAMPLE_RESOURCE_BUNDLE =
            "com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.sampi18nctx";
    public static final String [] MSGKEYS = {
            "it_IT", "fr_FR", "de_DE", "es_ES", "en_GB", "pt_BR", "ja_JP", "ko_KR", "zh_CN"
    };
    static {
        // Get the resource bundle for en_US
        try {
            bundle = ResourceBundle.getBundle(SAMPLE_RESOURCE_BUNDLE, java.util.Locale.ENGLISH);
        } catch (MissingResourceException e) {
            System.out.println ("Error: Exception occurred while setting resource bundle: " + e);
            e.printStackTrace ();
        }
    }
    
    /**
     * Retrieve a message from  resource bundle.
     * @return java.lang.String
     */
    public static String getMsg (String key) {
        String st = null;
        try {
            st = bundle.getString (key);
        } catch (Exception e) {
            System.out.println ("Error: Exception occurred while getting message: " + e);
            e.printStackTrace ();
        }
        return st;
    } // getMsg
    
    /**
     * Retrieve message from resource bundle.
     * @return java.lang.String []
     */
    public static String [] getMsgs (String [] key) {
        String [] strings = null;
        try {
            strings = new String[key.length];
            for (int i=0; i<key.length; i++)
                strings [i] = bundle.getString (key [i]);
        } catch (Exception e) {
            System.out.println ("Error: Exception occurred while getting message: " + e);
            e.printStackTrace ();
        }
        return strings;
    } // getMsg
    
    /**
     * Retrieve exchange rates from the specified resource bundle.
     * @return double[]
     */
    public static double [] getRates () {
        double [] rates;
        String [] ratesStr;
        rates = new double [9];
        ratesStr = new String [9];
        for (int i = 0; i < 9; i++) {
            rates [i] = 0;
            ratesStr [i] = null;
        }
        try {
            for (int n = 0; n < 9; n++) {
                ratesStr [n] = bundle.getString (Resources.MSGKEYS [n]);
                rates [n] = Double.parseDouble (ratesStr [n]);
            }
        } catch (Exception e) {
            System.out.println("Error: Exception occurred while getting exchange rates: " + e);
            e.printStackTrace();
        }
        return rates;
    } // getRates
      
    /**
     * Reset the resource bundle to one for a specific locale.
     */
    public static void setResourceBundle (Locale locale) {
        if (null != locale)
        try {
            bundle = ResourceBundle.getBundle(SAMPLE_RESOURCE_BUNDLE, locale);
        } catch (MissingResourceException e) {
            System.out.println("Error: Exception occurred while setting resource bundle: " + e);
            e.printStackTrace();
        }
    } // setResourceBundle 

}
