//-----------------------------------------------------------------------------------
// "This program may be used, executed, copied, modified and distributed without 
// royalty for the purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//-----------------------------------------------------------------------------------

package com.ibm.websphere.samples.i18nctx.currencyexchangewar;

import java.io.*;
import java.util.*;
import javax.naming.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

import com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.Resources;
import com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.AccInfoJBean;
import com.ibm.websphere.samples.i18nctx.currencyexchangelibrary.CustomerJBean;
import com.ibm.websphere.samples.i18nctx.currencyexchangeejb.Exchange;
import com.ibm.websphere.samples.i18nctx.currencyexchangeejb.ExchangeHome;

//------------------------------------------------------------
// INTERNATIONALIZATION SERVICE: Imports.
//------------------------------------------------------------
import com.ibm.websphere.i18n.context.UserInternationalization;
import com.ibm.websphere.i18n.context.Internationalization;
import com.ibm.websphere.i18n.context.InvocationInternationalization;

public class CurrencyServlet extends HttpServlet {
  
    //------------------------------------------------------------
    // INTERNATIONALIZATION SERVICE: API references.
    //------------------------------------------------------------
    protected UserInternationalization userI18n = null;
    //protected Internationalization i18n = null;
    protected InvocationInternationalization invI18n = null;
    
    //------------------------------------------------------------
    // INTERNATIONALIZATION SERVICE: JNDI name. 
    //------------------------------------------------------------
    public static final String USERI18N_URL = "java:comp/websphere/UserInternationalization";
     
    // JNDI name for Exchange EJB
    public static final String EXCHANGE_EJBHOME = "java:comp/env/ejb/Exchange";
      
    protected static Exchange exc = null;
    protected Locale iLocale;
    
    protected ExchangeHome exchangeHome = null;
    protected AccInfoJBean accInfo = null;
    protected CustomerJBean customerInfo = null;
    
    protected String account;
    protected String name;
    protected String password;
    
    //Exception message for EJB lookup and I18n API resolve
    private String excMsg = null;
    
    private DecimalFormat twoDigits = new DecimalFormat("0.00");
    private DecimalFormatSymbols en = new DecimalFormatSymbols();
    
    //texts need to be localized
    private String[] localizeStr = {
          "MSG1",    //localizeStr[0]
          "MSG2",    //localizeStr[1]
          "MSG3",    //localizeStr[2]
          "MSG4",    //localizeStr[3]
          "MSG5",    //localizeStr[4]
          "MSG6",    //localizeStr[5]
          "MSG7",    //localizeStr[6]
          "MSG8",    //localizeStr[7]
          "MSG9",    //localizeStr[8]
          "MSG10",   //localizeStr[9]
          "MSG12"};  //localizeStr[10]
    
    //Message keys passed to resource bundle to get localized texts
    private final static String[] MSGKEYS = {
          "MSG1",   //Please supply an account number     
          "MSG2",   //Please enter your password      
          "MSG3",   //Login incorrect, Please try again   
          "MSG4",   //Back to login        
          "MSG5",   //Login failed, please check log file to find out exception    
          "MSG6",   //Please enter units    
          "MSG7",   //Units must be greater than zero   
          "MSG8",   //Please supply a numeric units       
          "MSG9",   //Insufficient funds. Please try again     
          "MSG10",  //Transaction failed
          "MSG12"}; //Insufficient transaction amount. Please increase amount to exchange.          
   
    /**
     * Initialize this servlet.
     *  <p>Resolve all references enabling remote operation,
     * including the JNDI initial context, any EJBs, and the
     * Internationalization service API
     */
    public void init () throws ServletException {
        try {
            // Resolve EJB(s)
            Context initialContext = new InitialContext ();
            java.lang.Object homeObject = initialContext.lookup (EXCHANGE_EJBHOME);
            exchangeHome = (ExchangeHome)javax.rmi.PortableRemoteObject.narrow (
                    (org.omg.CORBA.Object)homeObject, ExchangeHome.class);
        } catch (Exception e) {
            excMsg = "ERROR: Cannot look-up Exchange EJB home:\n" + e;
        }
        
        //------------------------------------------------------------
        // INTERNATIONALIZATION SERVICE: Resolve the API.
        //------------------------------------------------------------
        try {
            Context initialContext = new InitialContext();
            userI18n = (UserInternationalization) initialContext.lookup (USERI18N_URL);
            invI18n = userI18n.getInvocationInternationalization ();
        } catch (NamingException ne) {
            excMsg =  "Error: Cannot resolve UserInternationalization: Exception:\n" + ne;
        } catch (IllegalStateException ise) {
            excMsg =  "Error: UserInternationalization is not available:\n" + ise;
        }
    } // init
    
    /**
     * Process incoming HTTP GET requests.
     * @param  request  Object that encapsulates the request to the servlet
     * @param  response  Object that encapsulates the response from the servlet
     */
    public void doGet (HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {
        doPost (request, response);
    } // doGet
    
    /**
     * Process incoming HTTP POST requests.
     * @param request  Object that encapsulates the request to the servlet.
     * @param response  Object that encapsulates the response from the servlet.
     */
    public void doPost (HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {
    
        // If any exception occurred, pass it to error.jsp
        if (null != excMsg) {
            request.setAttribute ("excMsg", excMsg);
            RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/error.jsp");
            rd.forward (request, response);
        } else {
            //------------------------------------------------------------
            // INTERNATIONALIZATION SERVICE: Get Invocation context/locale
            //------------------------------------------------------------
            try {
                iLocale = invI18n.getLocale (); 
    		
                // If the locale is not supported in this application, set it to US locale by default
                if (!AccInfoJBean.isMyLocale (iLocale))
                    iLocale = new Locale ("en", "US");
    
                // NOTE: Browsers may propagate accept-languages that contain a language code, 
                // but lack a country code, like "fr" to indicate "French as spoken in France."  
                // The following code supplies a default country code in such cases.
                if (iLocale.getCountry ().equals (""))
                    iLocale = AccInfoJBean.getCompleteLocale (iLocale);
    
                // set resource bundle according to the locale we get using INTERNATIONALIZATION SERVICE
                Resources.setResourceBundle (iLocale);
            
                // localize all display texts in this class
                localizeStr = Resources.getMsgs (MSGKEYS);
    
                // Get the hidden field named command from jsp form
                // decide which action to take
    
                String command = request.getParameter ("command");
    
                if (command.equals ("login")) {
                    // Verify user's principal(account and password)
                    verifyPrincipal (response, request);
                } else if (command.equals ("buyother")) {
                    // Use the US dollar to purchase the selected currency
                    // withdraw certain amount from customer database
                    withdraw (request, response);
                } else if (command.equals ("buyusd")) {
                    // Use the selected currency to purchase US dollar
                    // deposit certain amount to customer database
                    deposit (request, response);
                } else if (command.equals ("anotherExc")) {
                    // let user do another exchange
                    moreExchange (request, response);
                } else  {
                    request.setAttribute ("excMsg", "ERROR: Internal Error - command not set");
                    RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/error.jsp");
                    rd.forward (request, response);
                }
            } catch (IllegalStateException ise) {
                excMsg = "An anomaly occurred while accessing Invocation context:\n" + ise;
    		  
                request.setAttribute ("excMsg", excMsg);
                RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/error.jsp");
                rd.forward (request, response);
            }
        }
    } // doPost
    
    /**
     * Verify the customer's login and password.
     * @param response javax.servlet.http.HttpServletResponse
     * @param request javax.servlet.http.HttpServletRequest
     * @exception javax.servlet.ServletException 
     * @exception java.io.IOException 
     */
    public void verifyPrincipal (
            HttpServletResponse response, 
            HttpServletRequest request)
        throws javax.servlet.ServletException, java.io.IOException {
        
        String error = ""; // Message if any problems are found.
    
        // Get the parmeters from the form.
        account = request.getParameter ("account").trim ();
        password = request.getParameter ("password").trim ();
    
        // Check account and password.
        if (account == null || account.length() == 0) {
    	    error = localizeStr [0];
            response.setContentType ("text/html;charset=UTF-8");
            PrintWriter out = response.getWriter ();
            out.println ("<HTML>");
            out.println (" <script language=\"JavaScript\" src=\"/WSsamples/scripts//browserStyle.js\"></script> ");
            out.println ("<HEAD><TITLE>" + localizeStr[2] + "</TITLE></HEAD>");
            out.println (error + "</center></font><p>");
            out.println ("<BR><BR><A href=\"login.jsp\"><B>" + localizeStr[3] + "</B></A>");
            out.println ("</BODY></HTML>");
        } else if(password == null || password.length() == 0) {
            error = localizeStr[1];
            response.setContentType("text/html;charset=UTF-8");
            PrintWriter out = response.getWriter();
            out.println ("<HTML>");
            out.println (" <script language=\"JavaScript\" src=\"/WSsamples/scripts//browserStyle.js\"></script> ");
            out.println ("<HEAD><TITLE>" + localizeStr[2] + "</TITLE></HEAD>");
            out.println (error + "</center></font><p>");
            out.println ("<BR><BR><A href=\"login.jsp\"><B>" + localizeStr[3] + "</B></A>");
            out.println ("</BODY></HTML>");
    	} else {
            try {
                Exchange exc = exchangeHome.create ();
                customerInfo = exc.verifyPrincipal (account, password);
                if (customerInfo != null) {
                    exc.remove ();
    
                    request.setAttribute ("customerInfo", customerInfo);
                    request.setAttribute ("error", "");
                    RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/purchase.jsp");
                    rd.forward (request, response);
                } else {
                    // Login failed, let user try again
                    response.setContentType ("text/html;charset=UTF-8");
                    PrintWriter out = response.getWriter ();
                    out.println ("<HTML>");
                    out.println (" <script language=\"JavaScript\" src=\"/WSsamples/scripts//browserStyle.js\"></script> ");
                    out.println ("<HEAD><TITLE>" + localizeStr[2] + "</TITLE></HEAD>");
                    out.println ("<BODY BGCOLOR=\"#FFFFEE\"><font size = 5>" + localizeStr[2]);
                    out.println ("</font><p>");
                    out.println ("<BR><BR><A href=\"login.jsp\"><B>" + localizeStr[3] + "</B></A>");
                    out.println ("</BODY></HTML>");
                 }
            } catch (Exception e) {
                // Login failed, let user try again
                response.setContentType ("text/html;charset=UTF-8");
                PrintWriter out = response.getWriter ();
                out.println ("<HTML><HEAD><TITLE>" + localizeStr[2] + "</TITLE></HEAD>");
                out.println ("<BODY BGCOLOR=\"#FFFFEE\"><font color = red size = 5>" + localizeStr[2]);
                out.println ("</font><p>");
                out.println (localizeStr [4]);
                e.printStackTrace (out);
                out.println ("<BR><BR><A href=\"login.jsp\"><B>" + localizeStr[3] + "</B></A>");
                out.println ("</BODY></HTML>");
            }
        }
    } // verifyPrincipal
    
    /**
     * This method lets customers exchange other currencies by using US dollars.
     * @param  request  javax.servlet.http.HttpServletRequest
     * @param  response  javax.servlet.http.HttpServletResponse
     * @exception  javax.servlet.ServletException  
     */
    protected void withdraw (
            HttpServletRequest request, 
            HttpServletResponse response)
        throws javax.servlet.ServletException, IOException {

        double withdrawAmt = 0;
        double excAmt = 0;
        String error = ""; // String to hold error message if any problems are found.
    
        // Get the parmeters from the form.
        String units = request.getParameter ("usamount").trim ();
        String currency = request.getParameter ("currency").trim ();
    
        parmcheck : {
            // Check units.
            if (units == null || units.length () == 0) {
                error = localizeStr [5];
                break parmcheck;
            } else {
                try {
                    withdrawAmt = Double.parseDouble (units);
                    if (withdrawAmt <= 0) {
                        error = localizeStr [6];
                        break parmcheck;
                    }
                } catch (NumberFormatException e) {
                    error = localizeStr [7];
                    break parmcheck;
                }
            } //else
    
            //get exchange rate from properties file according to the selected currency
            double excRate = getRate (currency);
            //calculate withdraw amount
	    en.setDecimalSeparator('.');
	    twoDigits.setDecimalFormatSymbols(en);
            excAmt = withdrawAmt * excRate; 
            excAmt = Double.parseDouble (twoDigits.format (excAmt));
            withdrawAmt = Double.parseDouble (twoDigits.format (withdrawAmt));
            if (excAmt < 1) {
                error = localizeStr [10];
                break parmcheck;
            }
        } //parmcheck
    
        if (withdrawAmt > customerInfo.getBalance ())
            error = localizeStr [8];
    
        if (error.length () > 0) { // An error was detected.
            // Pass information - error message to JSP (go back to purchase.jsp)
            try {
                // request.setAttribute ("exchangeRate", Double.toString (excRate) );
                request.setAttribute ("customerInfo", customerInfo);
                request.setAttribute ("error", error);
                RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/purchase.jsp");
                rd.forward (request, response);
            } catch (Exception e) {
                response.setContentType ("text/html;charset=UTF-8");
                PrintWriter out = response.getWriter ();
                out.println ("<HTML><HEAD><TITLE>" + localizeStr [9] + "</TITLE></HEAD>");
                out.println ("<BODY><font color = red>" + localizeStr [9] + "</font><br>");
                e.printStackTrace (out);
                out.println ("</BODY></HTML>");
            }
        } else {
            try {
                Exchange exc = exchangeHome.create ();
                customerInfo = exc.withdraw (customerInfo.getAccount (), withdrawAmt);
                exc.remove ();
    
                /*
                 * If the transaction succeeds, return a JSP page with the
                 * new account information to the browser.
                 */
                request.setAttribute ("type", "withdraw");
                request.setAttribute ("usAmt", Double.toString (withdrawAmt));
                request.setAttribute ("excAmt", Double.toString (excAmt));
                request.setAttribute ("customerInfo", customerInfo);
                String currencyName = getCurrencyName (currency);
    
                request.setAttribute ("currency", currencyName);
                RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/accountInfo.jsp");
                rd.forward (request, response);
            } catch (Exception e) {
                // Transaction failed.
                response.setContentType ("text/html;charset=UTF-8");
                PrintWriter out = response.getWriter ();
                out.println("<HTML><HEAD><TITLE>" + localizeStr [9] + "</TITLE></HEAD>");
                out.println ("<BODY><font color = red>" + localizeStr [9] + "</font><br>");
                e.printStackTrace (out);
                out.println ("</BODY></HTML>");
            }
        }
    } // withdraw
    
    /**
     * This method lets customers exchange US dollars by using other currencies.
     * @param  request  javax.servlet.http.HttpServletRequest
     * @param  response  javax.servlet.http.HttpServletResponse
     * @exception  javax.servlet.ServletException
     */
    protected void deposit (
            HttpServletRequest request, 
            HttpServletResponse response)
        throws javax.servlet.ServletException, IOException {
        
        double otherAmt = 0;
        double depositAmt = 0;
        String error = ""; // String to hold error message if any problems are found.
    
        // Get the parmeters from the form.
        String units = request.getParameter ("otheramount").trim ();
        String currency = request.getParameter ("currency").trim ();
    
    
        parmcheck : {
            // Check units.
            if (units == null || units.length () == 0) {
                error = localizeStr [5];
                break parmcheck;
            } else {
                try {
                    otherAmt = Double.parseDouble (units);
                    if (otherAmt <= 0) {
                        error = localizeStr [6];
                        break parmcheck;
                    }
                } catch (NumberFormatException e) {
                    error = localizeStr [7];
                    break parmcheck;
                }
            }

            //get exchange rate from properties file according to the selected currency
            double excRate = getRate (currency);
            //calculate deposit amount
            depositAmt = otherAmt * (1.0 / excRate);
	     
	    en.setDecimalSeparator('.');
	    twoDigits.setDecimalFormatSymbols(en);
            
            otherAmt = Double.parseDouble (twoDigits.format (otherAmt));
            depositAmt = Double.parseDouble (twoDigits.format (depositAmt));
            if (depositAmt < 1) {
                error = localizeStr [10];
                break parmcheck;
            }
        } //End of parmcheck
    
        if (error.length () > 0) { // An error was detected.
            // Pass error information to purchase.jsp, let user try again
            try {
                request.setAttribute ("error", error);
                request.setAttribute ("customerInfo", customerInfo);
    
                RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/purchase.jsp");
                rd.forward (request, response);
                return;
            } catch (Exception e) {
                response.setContentType ("text/html;charset=UTF-8");
                PrintWriter out = response.getWriter ();
                out.println ("<HTML><HEAD><TITLE>" + localizeStr [9] + "</TITLE></HEAD>");
                out.println ("<BODY><font color = red>" + localizeStr [9] + "</font><br>");
                e.printStackTrace (out);
                out.println ("</BODY></HTML>");
            }
        } else {
            try {
                Exchange exc = exchangeHome.create ();
    
                customerInfo = exc.deposit (customerInfo.getAccount (), depositAmt);
                exc.remove ();
    
                // If the transaction succeeds, return a JSP page with the
                // new account information to the browser.
                request.setAttribute ("type", "deposit");
                request.setAttribute ("usAmt", Double.toString (depositAmt));
                request.setAttribute ("excAmt", Double.toString (otherAmt));
                request.setAttribute ("customerInfo", customerInfo);
                String currencyName = getCurrencyName (currency);
    
                request.setAttribute("currency", currencyName);
                RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/accountInfo.jsp");
                rd.forward (request, response);
            } catch (Exception e) {
                // Transaction failed.
                response.setContentType ("text/html;charset=UTF-8");
                PrintWriter out = response.getWriter ();
                out.println ("<HTML><HEAD><TITLE>" + localizeStr [9] + "</TITLE></HEAD>");
                out.println ("<BODY><font color = red>" + localizeStr [9] + "</font><br>");
                e.printStackTrace (out);
                out.println ("</BODY></HTML>");
            }
        }
    } // deposit
    
    protected void moreExchange (HttpServletRequest request, HttpServletResponse response)
        throws javax.servlet.ServletException, IOException {
        try {
            Exchange exc = exchangeHome.create ();
            //update customerInfo
            customerInfo = exc.updateCustomer (customerInfo.getAccount ());
            exc.remove ();
           
            request.setAttribute ("error", "");
            request.setAttribute ("customerInfo", customerInfo);
            RequestDispatcher rd = getServletContext ().getRequestDispatcher ("/purchase.jsp");
            rd.forward (request, response);
        } catch (Exception e) {
            response.setContentType ("text/html;charset=UTF-8");
            PrintWriter out = response.getWriter ();
            out.println ("<HTML><HEAD><TITLE>Unexcepted Error</TITLE></HEAD>");
            out.println ("<BODY><font color = red>" +localizeStr[9] + "</font><br>");
            out.println (e.getMessage () + "</BODY></HTML>");
        }
    } // moreExchange
    
    //get exchange rate from the properties file
    public double getRate (String localeStr) {
        return Double.parseDouble (Resources.getMsg (localeStr));
    }
    
    //get currency name from the properties file
    private String getCurrencyName (String currency) {
        String currencyKey = null;
        String result = null;
        if (currency.equals ("it_IT"))
            currencyKey = "ITL";
        else if (currency.equals ("fr_FR"))
            currencyKey = "FRF";
        else if (currency.equals ("de_DE"))
            currencyKey = "DEM";
        else if (currency.equals ("es_ES"))
            currencyKey = "ESP";
        else if (currency.equals ("en_GB"))
            currencyKey = "GBP";
        else if (currency.equals ("pt_BR"))
            currencyKey = "PTE";
        else if (currency.equals ("ja_JP"))
            currencyKey = "JPY";
        else if (currency.equals ("ko_KR"))
            currencyKey = "KRW";
        else if (currency.equals ("zh_CN"))
            currencyKey = "CNY";
        return Resources.getMsg (currencyKey);
    }
    
    /**
     * Return the servlet info string.
     */
    public String getServletInfo () {
        return "Currency Exchange Servlet";
    }

} // CLASS CurrencyServlet
