//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//

package com.ibm.websphere.samples.ejbquery.ejbmediatorwar;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.RequestDispatcher;
import javax.transaction.UserTransaction;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;


import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.sdo.impl.EDataGraphImpl;

import commonj.sdo.DataGraph;
import commonj.sdo.DataObject;

import com.ibm.websphere.sdo.mediator.ejb.Mediator;
import com.ibm.websphere.sdo.mediator.ejb.MediatorFactory;

/**
 * This is a sample program showing how to call the EJB Data Mediator Service. The simplest way to run the
 * sample is to open a browser to the url http://localhost:9080/DynamicQuery/EjbMediatorWeb/index.html,
 * enter EJB query select statements and press the "Submit" button.
 * The EJB query can be on multiple lines but the select statements must be separated by semicolon (";") characters. 
 * Example: select d.deptno, d.name from DeptBean as d; select e.* from in (d.emps) as e
 * @version 1.1
 */

public class EjbMediatorSample extends HttpServlet implements Servlet {


	private InitialContext ic = null;

	private UserTransaction utx = null;

	public EjbMediatorSample() {
		super();
	}
	

	   /**
	    * The _processQuery method uses the input query to create a mediator and then calls getGraph()
	    *  on the EJB Mediator.  
	    * The createMediator and getGraph methods must be run in a transaction context.
	    * @return void
	    * @param req javax.servlet.HttpServletRequest
	    * @param resp javax.servlet.HttpServletResponse
	    * @throws java.lang.Exception
	    */
	public void _processQuery(HttpServletRequest req, HttpServletResponse resp)
			throws Exception {

			HttpSession thisSession = req.getSession(true);
			resp.setBufferSize(8192);
			resp.setContentType("text/xml");
			PrintWriter out = resp.getWriter();

			String q = (req.getParameter("query"));
			String[] querys = null;
			querys = parse(q);

			/*
			 * EJB Mediator methods must always be called within the scope of a
			 * transaction however, createMediator, getGraph and applyChanges
			 * may be called in different tranactions.
			 * 
	 		 * EJB Mediator uses optimistic caching using the EJB optimistic
			 * predicate fields defined in the EJB mapping to maintain data integrity.
			 */

			utx.begin();
			/*
			 * Create a mediator instance from the input querys. 
			 * In this example the second argument to createMediator is null indicating 
			 * that there are no query input parameters being used. 
			 * After create, execute the query by calling 
			 * getGraph to create a DataGraph. 
			 */
			Mediator med = MediatorFactory.getInstance().createMediator(querys,
					null);

			DataGraph g = med.getGraph().getDataGraph();
			utx.commit();
			
			try {
				
			/* **************************************************************
			 * the following code is for you to use to modify the DataGraph *
			 ****************************************************************/

			/*
			 * Create a Service Data Object (SDO): 
			 * The following shows an example of how to create an SDO. A new 
			 * Employee SDO is created but has null values for all attributes. 
			 * The primary key field must be updated before running applyChanges because 
			 * it cannot be null in the database. 
			 * SDO creates are recorded in the DataGraph's change log for later processing 
			 * by the EJB Mediator. 
			 */
			//g.getRootObject().createDataObject("EmpBean");

			/*
			 * Update a Service Data Object (SDO): 
			 * The following shows an example of how to update an SDO. The first 
			 * Employee SDO is updated to a new salary of 50000. SDO updates are 
			 * recorded in the DataGraph's change log for later processing by the EJB Mediator. 
			 */
			//g.getRootObject().set("/EmpBean.0/salary", new Double(50000.00));


				/*
			 * Delete a Service Data Object (SDO): 
			 * The following shows an example of how to delete an SDO. The first 
			 * Employee SDO is deleted. SDO deletes are 
			 * recorded in the DataGraph's change log for later processing by the EJB Mediator. 
			 */
			//((DataObject) g.getRootObject().get("EmpBean.0")).delete();
			//((DataObject) g.getRootObject().get("EmpBean.0/dept")).delete();
	
			} catch (Exception qe) {
				throw new Exception("SDO Create,Update, or Delete failed with "+qe+
						", probably because of an incorrect SDO path or value."); 
			}

			
			/* *************************************************************
			 * the following code is for you to use to modify the database *
			 ***************************************************************/

			/*
			 * Apply changes to database directly: 
			 * Note: to see the effect of applyChanges you must reissue the 
			 * getGraph() command to rebuld the DataGraph from the database. 
			 * The following code shows how to run applyChanges to write 
			 * back DataGraph changes directly to datasource using EJB mapping 
			 * but bypassing the EJB Container. 
			 */
//						utx.begin();
//						med.applyChanges(g.getRootObject());
//					    utx.commit();
			
			/*
			 * Apply changes to database through EJB Container: 
			 * Alternatively, you may want to write back the changes through the EJB
 			 * Container and EJB instances. This would take advantage of any EJB
			 * caching by the container and Persistence Manager as well as taking
			 * advantage of application logic within the EJB.
			 */
//					      utx.begin();
//					      med.applyChanges(g.getRootObject(), null);
//					      utx.commit();
			

			/*
			 * Display the datagraph: 
			 * You may display the entire DataGraph, including the schema and changelog, 
			 * or display SDO values only. 
			 * For details see the displayGraph method. 
			 */
			
			displayGraph(thisSession, req, out,  g);

			return;
	}


	   /**
	   /**
	    * The displayGraph method shows the DataGraph that is returned after the mediator query 
		* and any subsequent DataObject commands are issued.  
	    * You can choose to display the entire DataGraph, including the schema and change log, 
	    * or choose to display SDO values only.
	    * @return java.lang.String
	    * @param thisSession javax.servlet.HttpSession
	    * @param req javax.servlet.HttpServletRequest
	    * @param out java.io.PrintWriter
	    * @throws java.lang.Exception
	    */
	public void displayGraph(HttpSession thisSession, HttpServletRequest req,
			PrintWriter out, DataGraph g) throws Exception {

		if (g == null) {
			throw new Exception("Please enter a query first.");
		}
		
		/*
		 * to display the datagraph use the save method on the Resouce object.
		 */

		/*
		 * to display entire DataGraph, including the schema and changelog,
		 * uncomment the next line and comment the line following.
		 */
		//	Resource gres = ((EDataGraphImpl) g).getDataGraphResource();

		/*
		 * to display SDO values only,
		 * uncomment the next line and comment the previous line.
		 */
			Resource gres = ((EDataGraphImpl) g).getRootResource();

		try {
			ByteArrayOutputStream buf = new ByteArrayOutputStream();
			gres.save(buf, null);
			buf.flush();
			buf.close();
			out.println(buf.toString());

		} catch (IOException e) {
			out.println("IOException during xml display. " + e);
			throw new Exception(e.getMessage());
		}

	}


	 /**
	 * The doPost method checks if the ejb application is installed and started then processes the query.
	 * It forwards Exceptions to error.jsp.
	 * @return void
	 * @param req javax.servlet.HttpServletRequest
	 * @param resp javax.servlet.HttpServletResponse
	 * @throws javax.servlet.ServletException, java.io.IOException
	 */
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		try {
			installCheck(); 
			_processQuery(req, resp);
		} catch (Exception ex) {
			req.setAttribute("javax.servlet.jsp.jspException", ex);
			RequestDispatcher rd = getServletContext().getRequestDispatcher(
			"/error.jsp");
			rd.forward(req, resp);
		}
		
	}


	
	/**
	 * The init method is used to retrieve the InitialContext for the application server namespace
	 * and the UserTransaction from the application server namespace.
	 * @return void
	 * @throws javax.servlet.ServletException
	 */
	public void init() throws ServletException {
		super.init();
		try {
			ic = new InitialContext();
		} catch (NamingException e) {

		}
		utx = (UserTransaction) lookup("java:comp/UserTransaction",
				"JNDI Lookup of 'java:comp/UserTransaction' failed.  Can not continue.");

	}

	/**
	 * The installCheck method checks if the datasource and ejb home for the sample are installed correctly.
	 * It runs in doPost so exceptions can be returned to the web browser.
	 * @return void
	 * @throws java.lang.Exception
	 */
		public void installCheck() throws Exception {

			/**
			 * check if datasource and ejb home for sample are installed
			 * correctly.
			 */
			lookup(
					"jdbc/QuerySampDatasource",
					"JNDI Lookup of DataSource jdbc/QuerySampDatasource failed. Check if the DynamicQuery sample database is installed correctly.");
			lookup(
					"samples/ejbquery/employeefinderejb/dept",
					"JNDI Lookup of EJB Homes ejb/Dept failed. Check if the DynamicQuery Sample Application is installed correctly.");
			lookup(
					"samples/ejbquery/employeefinderejb/emp",
					"JNDI Lookup of EJB Homes ejb/Emp failed. Check if the DynamicQuery Sample Application is installed correctly.");

		}

	/**
	 * The lookup method is a utility method to retrieve references from the application server namespace.
	 * @return Object
	 * @param name String Name of Application Object to be retrieved.
	 * @param message String Error message to return
	 * @throws javax.servlet.ServletException
	 */
	protected Object lookup(String name, String message)
			throws ServletException {
		Object r = null;
		try {
			r = ic.lookup(name);
		} catch (Exception e) {
			throw new ServletException(message, e);
		}
		if (r == null)
			throw new ServletException(message);
		return r;
	}

	   /**
	    * The parse method converts a select query String which may contain several select
	    * statements separated by semicolons on one or more lines into an array of single select
	    * statements.
	    * @return java.lang.String[]
	    * @param qs java.lang.String
	    * @throws java.lang.Exception
	    */
	protected String[] parse(String qs) throws Exception {
		java.util.ArrayList ql = new java.util.ArrayList();
		int i, startPos = 0;
		String tmp = null;
		while ((i = qs.indexOf(';', startPos)) != -1) {
			tmp = qs.substring(startPos, i).trim();
			if (tmp.length() > 0){
				ql.add(tmp);
			}
			startPos = i + 1;
		}
		tmp = qs.substring(startPos).trim();
		if (tmp.length() > 0){
			ql.add(tmp);
		}
		if (ql.size()==0) {
			throw new Exception("Please enter a query command.");
		}
		String[] r = new String[ql.size()];
		return (String[]) ql.toArray(r);
	}


}
