//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.ejbquery.employeefinderejb;

import java.rmi.RemoteException;
import java.util.Collection;
import javax.ejb.*;
import java.math.*;
import java.sql.*;
import java.util.Calendar;
import java.util.GregorianCalendar;
/**
 * The EmpBean CMP EJB Bean implements the EntityBean interface.  It 
 * contains the accessor and business methods necessary to work with an Emp.  In
 * addition, it contains the Home Interface Methods, and Callback Methods. 
 */
public abstract class EmpBean implements EntityBean
{
	//CMP fields

	protected EntityContext entityContext = null;

    //
    // CMP Field accessor methods           Added for EJB 2.0 CMP field support
    //
 
 /**
 * This abstract CMP field accessor method gets the empid attribute.
 * @return java.lang.Integer
 */
	abstract public Integer getEmpid();

 /**
 * This abstract CMP field accessor method sets the empid attribute.
 * @param empid java.lang.Integer
 */
	abstract public void setEmpid(Integer empid);

 /**
 * This abstract CMP field accessor method gets the name attribute.
 * @return java.lang.String
 */
	abstract public String getName();

 /**
 * This abstract CMP field accessor method sets the name attribute.
 * @param name java.lang.String
 */
	abstract public void setName(String name);

 /**
 * This abstract CMP field accessor method gets the salary attribute.
 * @return double
 */
        abstract public double getSalary();

 /**
 * This abstract CMP field accessor method sets the salary attribute.
 * @param salary double
 */
	abstract public void setSalary(double salary);

 /**
 * This abstract CMP field accessor method gets the bonus attribute.
 * @return double
 */
        abstract public double getBonus();

 /**
 * This abstract CMP field accessor method sets the bonus attribute.
 * @param bonus double
 */
  	abstract public void setBonus(double bonus);

 /**
 * This abstract CMP field accessor method gets the hireDate attribute.
 * @return java.sql.Date
 */
	abstract public java.sql.Date getHireDate();

 /**
 * This abstract CMP field accessor method sets the hireDate attribute.
 * @param hireDate java.sql.Date
 */
	abstract public void setHireDate(java.sql.Date hireDate);

 /**
 * This abstract CMP field accessor method gets the hireTime attribute.
 * @return java.sql.Time
 */
        abstract public Time getHireTime();

 /**
 * This abstract CMP field accessor method sets the hireTime attribute.
 * @param hireTime java.sql.Time
 */
	abstract public void setHireTime(Time hireTime);

 /**
 * This abstract CMP field accessor method gets the hireTimestamp attribute.
 * @return java.sql.Timestamp
 */
        abstract public Timestamp getHireTimestamp();

 /**
 * This abstract CMP field accessor method sets the hireTimestamp attribute.
 * @param hireTimestamp java.sql.Timestamp
 */
	abstract public void setHireTimestamp(Timestamp hireTimestamp);

 /**
 * This abstract CMP field accessor method gets the isManager attribute.
 * @return boolean
 */
        abstract public boolean getIsManager();

 /**
 * This abstract CMP field accessor method sets the isManager attribute.
 * @param isManager boolean
 */
  	abstract public void setIsManager(boolean isManager);
	
	//CMR (many-to-1)
/**
 * This abstract CMR field accessor method gets the dept attribute.
 * @return DeptLocal
 */
	abstract public DeptLocal getDept();

/**
 * This abstract CMR field accessor method sets the dept attribute.
 * @param newDept DeptLocal
 */
	abstract public void setDept(DeptLocal newDept);

   //Bean business methods

 /**
 * add1 method returns empid + 1. <BR><BR>
 * eg. select e.add1() from EmpBean e <BR>
 * <strong>WARNING.</strong>&nbsp For methods to work, the server's "Application classloader policy" must be set to "Single".  The default is "Multiple" which can be reset using the Administrative Console.
 *
 * @return int
 */
   public int add1() {
      return getEmpid().intValue() + 1;
   }

 /**
 * calcBonus method calculates an employee's annual bonus given a positive profitFactor. <BR>
 * For non managers this is calculated by multiplying 
 * the yearsWorked, the profitFactor, and the profitUnit. <BR>
 * For managers, the previous result is further multiplied by
 * the mgrFactor if the profitFactor is greater than 0.1. <BR>
 * In the method, profitUnit is fixed at 1000 and mgrFactor at 2. <BR><BR>
 * eg. select e.calcBonus(0.2) from EmpBean e <BR>
 * <strong>WARNING.</strong>&nbsp For methods to work, the server's "Application classloader policy" must be set to "Single".  The default is "Multiple" which can be reset using the Administrative Console.
 * @param profitFactor the fractional increase of the value of the company during the current the year
 * @return double
 */
   public double calcBonus(double profitFactor) {
      int profitUnit = 1000;
      int mgrFactor = 2;
	double newBonus = 0;	
      Calendar rightNow = Calendar.getInstance(); 
      Calendar hireCal = Calendar.getInstance(); 
//      Calendar hireCal = (new GregorianCalendar());
      hireCal.setTime((java.util.Date)getHireDate());
      if (profitFactor > 0) {
         int yrsWorked = (rightNow.get(Calendar.YEAR) - hireCal.get(Calendar.YEAR));
         newBonus = yrsWorked * profitUnit * profitFactor;
         if (getIsManager()) {
            if (profitFactor > .1 ) {
               newBonus = newBonus * mgrFactor ;
	      }
         }
	}
      return (newBonus);
   }    

   //framework methods


    // 
    // Home Interface Methods
    //


/**
 * ejbCreate method responsible for the creation of a skeleton Emp.
 * @param empid java.lang.Integer
 * @return EmpKey  the Primary Key Object for Emp, it must be unique.
 * @exception javax.ejb.CreateException  This exception is thrown to 
 *            indicate a failure to create the entity EJB.
 */
   public EmpKey ejbCreate(Integer empid) throws CreateException {
      setEmpid(empid);
      return new EmpKey(empid); 
   } 

/**
 * ejbCreate method responsible for the creation of an Emp.
 * @param empid java.lang.Integer
 * @param name java.lang.String
 * @param salary double
 * @param bonus double
 * @param hireDate java.sql.Date
 * @param hireTime java.sql.Time
 * @param hireTimestamp java.sql.Timestamp
 * @param isManager boolean
 * @return EmpKey  the Primary Key Object for Emp, it must be unique.
 * @exception javax.ejb.CreateException  This exception is thrown to 
 *            indicate a failure to create the entity EJB.
 */
   public EmpKey ejbCreate(Integer empid, String name, double salary, double bonus, Date hireDate, Time hireTime, Timestamp hireTimestamp, boolean isManager) throws CreateException {
      setEmpid(empid);
      setName(name);
      setSalary(salary);
      setBonus(bonus);
      setHireDate(hireDate);
      setHireTime(hireTime);
      setHireTimestamp(hireTimestamp);
      setIsManager(isManager);	
      return new EmpKey(empid); 
   }

/**
 * ejbPostCreate method responsible for the post creation of a skeleton Emp.
 * @param empid java.lang.Integer
 * @exception javax.ejb.CreateException  This exception is thrown to 
 *            indicate a failure to post create the entity EJB.
 */
   public void ejbPostCreate(Integer empid) throws CreateException {
   } 

/**
 * ejbPostCreate method responsible for the post creation of an Emp.
 * @param empid java.lang.Integer
 * @param name java.lang.String
 * @param salary double
 * @param bonus double
 * @param hireDate java.sql.Date
 * @param hireTime java.sql.Time
 * @param hireTimestamp java.sql.Timestamp
 * @param isManager boolean
 * @exception javax.ejb.CreateException  This exception is thrown to 
 *            indicate a failure to post create the entity EJB.
 */
   public void ejbPostCreate(Integer empid, String name, double salary, double bonus, Date hireDate, Time hireTime, Timestamp hireTimestamp, boolean isManager) throws CreateException {
   }

/**
 * ejbPostCreate Home Interface Method responsible for post creation steps, there
 * are none for Emp.
 * @param key EmpKey
 * @exception javax.ejb.CreateException    This exception is thrown to 
 *            indicate a failure to post create the entity EJB.
 */
   public void ejbPostCreate(EmpKey key) throws CreateException {}


       //
    // Required callback methods
    //

/**
 * setEntityContext required callback method for Container Managed Persistence.
 * @param ctx javax.ejb.EntityContext
 * @exception javax.ejb.EJBException    This exception is thrown to 
 *            indicate a failure to create the entity EJB.
 */ 
    public void setEntityContext (EntityContext ctx) throws EJBException {
      entityContext = ctx;
   }
/**
 * unsetEntityContext required callback method for Container Managed Persistence. 
 * @exception javax.ejb.EJBException   Signals that an EJB exception can occur.
 */ 
    public void unsetEntityContext() throws EJBException {
       entityContext = null;
   }

/**
 * ejbLoad required callback method gets called after the EJB is loaded from
 * the database.
 * @exception javax.ejb.EJBException   Signals that an EJB exception can occur. 
 */
   public void ejbLoad() throws EJBException {}

/**
 * ejbStore required callback method for Contaimer Managed Persistence.
 * @exception javax.ejb.EJBException   Signals that an EJB exception can occur. 
 */
   public void ejbStore() throws EJBException {}

/**
 * ejbRemove required callback method for Contaimer Managed Persistence.
 * @exception javax.ejb.EJBException   Signals that an EJB exception can occur. 
 * @exception javax.ejb.RemoveException   Signals that a Remove exception can occur. 
 */
   public void ejbRemove() throws EJBException, RemoveException {}

/**
 * ejbActivate required callback method for lifecycle notification.
 * No actions need to be taken for Emp.
 * @exception javax.ejb.EJBException    Signals that an EJB exception can occur.
 */
   public void ejbActivate() throws EJBException {}

/**
 * ejbPassivate required callback method for lifecycle notification.
 * No actions need to be taken for Emp.
 * @exception javax.ejb.EJBException    Signals that an EJB exception can occur.
 */
   public void ejbPassivate() throws EJBException {}

}
