//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.ejbquery.employeefinderwar;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.*;
import java.rmi.*;
import java.math.BigDecimal;
import javax.transaction.UserTransaction;

import com.ibm.websphere.ejbquery.*;

/**
 * The ProcessQueryBean class provides the "run Query" implementation for DynamicQuery.jsp
 */

public class ProcessQueryBean {

   // Attributes of a ProcessQueryBean.

/**
 * This is the index position of the predefined query selector radio button used in the associated DynamicQuery.jsp.
 */
   public String selectorIndex;

/**
 * This is the query created in the associated DynamicQuery.jsp.
 */
   public String query = null;

/**
 * This holds the results of the query.
 */
   public String[] results = null;

/**
 * This holds the correct answer for the query.
 */
   public String[] answer = null;
/**
 * This indicates if DynamicQuery is using the Local Query interface
 */
   public String local = null;
/**
 * This holds the starting row in the database for the query.
 */
   public String startRow = "1";

/**
 * This holds the ending row in the database for the query.
 */
   public String endRow = "100";

/**
 * This indicates if DynamicQuery is printing the Query Plan
 */
   public String checkPlan = null;

/**
 * The answerCheck attribute indicates if the query results are correct.
 */
   public Boolean answerCheck = null;


   // package variables

   String diag;
   QueryHome      aJQSHome;
   Query    JQueryService;
   QueryLocalHome       aLQSHome;
   QueryLocal           LQueryService; 
   UserTransaction       ut=null;
   Object [] parms = null;
   String parmString = null;
   String [] _typeArray = null;




   /**
  * This accessor method gets the selectorIndex attribute.
  * @return java.lang.String
  */
   public String getSelectorIndex(){return selectorIndex;}
   /**
   * This accessor method sets the selectorIndex attribute.
   * @param s java.lang.String
   */
   public void setSelectorIndex(String s){selectorIndex = s;}
   /**
   * This accessor method gets the local attribute.
   * @return java.lang.String
   */
   public String getLocal(){return local;}
   /**
   * This accessor method sets the local attribute.
   * @param s java.lang.String
   */
   public void setLocal(String s){local = s;}
   /**
   * This accessor method gets the checkPlan attribute.
   * @return java.lang.String
   */
   public String getCheckPlan(){return checkPlan;}
   /**
   * This accessor method sets the checkPlan attribute.
   * @param s java.lang.String
   */
   public void setCheckPlan(String s){checkPlan = s;}
   /**
   * This accessor method gets the startRow attribute.
   * @return java.lang.String
   */
   public String getStartRow(){return startRow;}
   /**
   * This accessor method sets the startRow attribute.
   * @param s java.lang.String
   */
   public void setStartRow(String s){
      try {
         Integer.parseInt(s);
         startRow = s;
      } catch (java.lang.NumberFormatException e) {
         startRow = "1";
      }
   }
   /**
    * This accessor method gets the endRow attribute.
    * @return java.lang.String
    */
   public String getEndRow(){return endRow;}
   /**
   * This accessor method sets the endRow attribute.
   * @param s java.lang.String
   */
   public void setEndRow(String s){
      try {
         Integer.parseInt(s);
         endRow = s;
      } catch (java.lang.NumberFormatException e) {
         endRow = "100";
      }
   }
   /**
   * This accessor method gets the query attribute.
   * @return java.lang.String
   */
   public String getQuery(){return query;}
   /**
   * This accessor method sets the query attribute.
   * @param s java.lang.String
   */
   public void setQuery(String s){query = s;}
   /**
   * This accessor method gets the results attribute.
   * @return java.lang.String[]
   */
   public String[] getResults(){return results;}
   /**
   * This accessor method sets the results attribute.
   * @param s java.lang.String[]
   */
   public void setResults(String[] s){results = s;}
   /**
   * This accessor method gets the answer attribute.
   * @return java.lang.String[]
   */
   public String[] getAnswer(){return answer;}
   /**
   * This accessor method sets the answer attribute.
   * @param s java.lang.String[]
   */
   public void setAnswer(String[] s){answer = s;}
   /**
   * This accessor method gets the answerCheck attribute.
   * @return java.lang.Boolean
   */
   public Boolean getAnswerCheck(){return answerCheck;}
   /**
   * This accessor method sets the answerCheck attribute.
   * @param s java.lang.Boolean
   */
   public void setAnswerCheck(Boolean s){answerCheck = s;}
   /**
   * compareResults method compares the result String[] with the answer String[]
   * to setAnswerCheck.
   */
   public void compareResults(){
      setAnswerCheck(new Boolean(results[0].equals(answer[0])));
   }

/**
 * This method acts like main and processes the query attribute returning, if successful, an html table containing the query results, and if unsuccessful, an html text area error message.
 * To do this it uses doQuery() and print().
 * @return java.lang.String
 */
   public String processQuery(){
      String st = null;
      StringBuffer error = new StringBuffer();

      //decode a possible parameter string to the parameter list expected by executeQuery and prepareQuery
      parseInput(query);
     // System.out.println("processQuery() - query = " + query);
      if (_typeArray != null) {
         processSet(_typeArray);
      }

      if (checkPlan != null ) {
         // show the query plan
         String plan = null;
         if (local == null)
            plan = doPlan(query, error);
         else
            plan = doLocalPlan(query, error);
         if (plan == null) {
            st = new String(error);
         } else {
            st = new String("<table><table cellpadding=5 cellspacing=1 border=0><tr><td><TEXTAREA rows=\"15\" cols=\"74\" name=\"plan\">" + plan + "</TEXTAREA></td></tr></table>");
         }
      } else {

         if (local == null) {
            QueryIterator iter = doQuery ( query, error);
            if (iter == null ) {
               st = new String(error);
            } else {
               st = new String(print(iter));
            }
         } else { // do local interface query
            QueryLocalIterator iter = doLocalQuery ( query, error );
            if (iter == null ) {
               st = new String(error);
            } else {
               st = new String(localprint(iter));
            }
            try {
               ut.commit();
            } catch (Exception e) {
            }

         }

      }//if checkplan 
      return st;

   }






/**
 * This method looks up the com.ibm.websphere.ejbquery.QueryBean's remote interface, executes its create(), executes its executeQuery(), and returns, if successful, an html table containing the query results, and if unsuccessful,  fills error with a html text area error message
 * @return com.ibm.websphere.ejbquery.QueryIterator
 * @param qstmt java.lang.String
 * @param error java.lang.StringBuffer
 */
   public QueryIterator doQuery(String qstmt, StringBuffer error )
   {

      QueryIterator iter;
      diag = "";


      int skip = Integer.parseInt(startRow); 
      if (skip > 0) {
         skip = skip -1;
      } else {
         skip = 0;
      } 
      int max = Integer.parseInt(endRow) - skip; 

      try {
         if (JQueryService == null) {
            diag="InitialContext";
            InitialContext ic = new InitialContext();
            diag = "lookup";


            Object homeObject = ic.lookup("com/ibm/websphere/ejbquery/Query");
            diag = "narrow";
            aJQSHome = (QueryHome)javax.rmi.PortableRemoteObject.narrow(homeObject,
                                                                        QueryHome.class);
            diag = "create";
            JQueryService = aJQSHome.create();
         }

         diag = "executeQuery";
         iter = JQueryService.executeQuery(qstmt, parms, null, skip, max);
         return iter;

      } catch (QueryException ex) {
         //System.out.println(ex.getMessage());
         error.append("<table><table cellpadding=5 cellspacing=1 border=0><tr>");
         error.append("<td>"+"<B>DynamicQuery Error:  </B>"+"</td>"+"</tr>"+"<tr>");
         error.append("<td>");
         error.append("\n <TEXTAREA NAME=\"results\" COLS=\"80\" ROWS =\"3\" > "); 
         error.append(ex.getMessage());
         error.append("</TEXTAREA>");
         error.append("</td>");
         error.append("</tr>");
         error.append("</table>");
         //System.err.println(error);
         return null;
      } catch (Exception ex) {
         System.out.println(diag);
         ex.printStackTrace();
         //add enclosing table for spacing
         error.append("<table><table cellpadding=5 cellspacing=1 border=0><tr>");
         error.append("<td>"+"<B>DynamicQuery Error:  </B>"+"</td>"+"</tr>"+"<tr>");
         error.append("<td>");
         error.append("\n <TEXTAREA NAME=\"results\" COLS=\"80\" ROWS =\"3\" >" ); 
         error.append(diag +"\n"+ ex.toString());
         error.append("</TEXTAREA>");
         error.append("</td>");
         error.append("</tr>");
         error.append("</table>");
         return null;
      }

   }

/**
 * This method looks up the com.ibm.websphere.ejbquery.QueryBean's local interface, executes its create(), executes its executeQuery(), and returns, if successful, an html table containing the query results, and if unsuccessful,  fills error with a html text area error message
 * @return com.ibm.websphere.ejbquery.QueryIterator
 * @param qstmt java.lang.String
 * @param error java.lang.StringBuffer
 */

   public QueryLocalIterator doLocalQuery(String qstmt, StringBuffer error ) 
   {
      int prefetchCnt = 5;
      QueryLocalIterator iter;
      String diag = "";



      try {
         diag="InitialContext";
         InitialContext ic = new InitialContext();

         if (LQueryService == null) {
            diag = "lookup";
            Object homeObject = ic.lookup("java:comp/env/ejb/LocalQuery");
            diag = "narrow";
            aLQSHome = (QueryLocalHome) homeObject;
            diag = "create";
            LQueryService = aLQSHome.create();
         }

         if (ut == null) {
            diag="getTransaction";
            ut = (UserTransaction) ic.lookup("java:comp/UserTransaction");
            if (ut == null) {
               error.append(" Failed to obtain UserTransaction");
               return null;
            }
         }

         ut.begin();
         diag = "executeQuery";
         iter = LQueryService.executeQuery(qstmt, parms, null);
         return iter;

      } catch (QueryException ex) {
         //System.out.println(ex.getMessage());
         error.append("<table><table cellpadding=5 cellspacing=1 border=0><tr>");
         error.append("<td>"+"<B>DynamicQuery Error:  </B>"+"</td>"+"</tr>"+"<tr>");
         error.append("<td>");
         error.append("\n <TEXTAREA NAME=\"results\" COLS=\"80\" ROWS =\"3\" > "); 
         error.append(ex.getMessage());
         error.append("</TEXTAREA>");
         error.append("</td>");
         error.append("</tr>");
         error.append("</table>");
         //System.err.println(error);
         return null;
      } catch (Exception ex) {
         System.out.println(diag);
         ex.printStackTrace();
         //add enclosing table for spacing
         error.append("<table><table cellpadding=5 cellspacing=1 border=0><tr>");
         error.append("<td>"+"<B>DynamicQuery Error:  </B>"+"</td>"+"</tr>"+"<tr>");
         error.append("<td>");
         error.append("\n <TEXTAREA NAME=\"results\" COLS=\"80\" ROWS =\"3\" >" ); 
         error.append(diag +"\n"+ ex.toString());
         error.append("</TEXTAREA>");
         error.append("</td>");
         error.append("</tr>");
         error.append("</table>");
         return null;
      }
   }


/**
 * This method iterates through the passed in QueryIterator formatting an html table structure returning, if successful, an html table containing the query results,
 and if unsuccessful, an html text area error message
 * @return java.lang.StringBuffer
 * @param iter com.ibm.websphere.ejbquery.QueryIterator
 */
   public StringBuffer print(QueryIterator iter)   {
      int rowid=1;
      int ncols=0;

      diag = "print";
      StringBuffer sb = new StringBuffer();
      try {

         if (iter== null) return sb;

         // create result table
         sb.append("<table cellpadding=5 cellspacing=1 border=1>\n");
         
   //gfh-will get better col hdr for ejb ref when getFieldName() displays object name instead of col no       

         // create labels for columns
         ncols = iter.getFieldsCount();
         sb.append("<tr>");
         sb.append("<th>"+"Row") ;
         for (int j=1; j<=ncols; j++) {
            sb.append("<th>"+ (iter).getFieldName(j)) ;
         }
         sb.append("</tr>\n");
         // create data rows
         while (iter.hasNext()) {
            IQueryTuple outTuple = (IQueryTuple)iter.next();


            sb.append("<tr>");
            sb.append("<td>"+ rowid++);
            for (int j=1; j<=ncols; j++) {

               Object o1 = null;  
               try {
                  o1 = outTuple.getObject(j);  
                  if (o1 instanceof EJBObject ) {
                     EJBObject o2 = (EJBObject) o1;  
                     o1 = o2.getPrimaryKey();
                     String s1 = o1.toString();
//                     System.out.println(" s1 = "+s1);
                     int i1 = s1.lastIndexOf(".");
                     int i3 = s1.lastIndexOf("@");
//                     System.out.println("i1 "+i1+" i3 "+i3);

                     if (i1 != -1  && i3 != -1   && i1<i3 ) {
                        String s2 = s1.substring(i1+1);
                        i3 = s2.lastIndexOf("@");
                        int i2 = s2.lastIndexOf("Key");
                        if (i2 != -1 ) {
                           o1 = new String(s2.substring(0,i2) +"."+ s2.substring(i3+1));
                        } else {
                           o1 = new String(s2.substring(0,i3) +"."+ s2.substring(i3+1));
                        }
                     } else {
                        o1 = new String("ejbObject()");
//                        o1 = new String(s1);
                     }
                  }
               } catch (Exception ex) {
                  o1 = ex.getMessage();
               }
               sb.append("<td>"+ o1); 
            } //end for
            sb.append("</tr>\n");
         } //end while
         sb.append("</table>");

	//put result title and row count on top
         sb.insert(0,(rowid -1)+ " row(s)\n");
         sb.insert(0,"<b>DynamicQuery Result:  </b>\n"+"<br>");
         
         return sb;

      } catch (Exception ex) {
         System.out.println(diag);
         ex.printStackTrace();
         //System.out.println(ex.toString());
         sb = new StringBuffer();
         //add enclosing table for spacing
         sb.append("<table><table cellpadding=5 cellspacing=1 border=0><tr>");
         sb.append("<td>"+"<B>DynamicQuery Error:  </B>"+"</td>"+"</tr>"+"<tr>");
         sb.append("<td>");
         sb.append("\n <TEXTAREA NAME=\"results\" COLS=\"80\" ROWS =\"3\" >" ); 
         sb.append(diag +"\n"+ ex.toString());
         sb.append("</TEXTAREA>");
         sb.append("</td>");
         sb.append("</tr>");
         sb.append("</table>");
         return sb;
      }
   }

   /**
    * This method iterates through the passed in QueryLocalIterator formatting an html table structure returning, if successful, an html table containing the query results,
    and if unsuccessful, an html text area error message
    * @return java.lang.StringBuffer
    * @param iter com.ibm.websphere.ejbquery.QueryLocalIterator
    */
   public StringBuffer localprint(QueryLocalIterator iter) 
   {
      int rowidx=0;
      int rowid=1;
      int ncols=0;

// local query is demand driven and has no start/max parms in executeQuery so I am using the start and
//end counts to decide what to output.
      int strow = Integer.parseInt(startRow); 
      if (strow < 1) {
         strow = 1;
      }
      int erow = Integer.parseInt(endRow); 

      diag = "localprint";
      StringBuffer sb = new StringBuffer();
      try {

         if (iter== null) return sb;

         // create result table
         sb.append("<table cellpadding=5 cellspacing=1 border=1>\n");
         
   //gfh-will get better col hdr for ejb ref when getFieldName() displays object name instead of col no       

         // create labels for columns
         ncols = iter.getFieldsCount();
         sb.append("<tr>");
         sb.append("<th>"+"Row") ;
         for (int j=1; j<=ncols; j++) {
            sb.append("<th>"+ (iter).getFieldName(j)) ;
         }
         sb.append("</tr>\n");

         // create data rows
         while (iter.hasNext()) {
            IQueryTuple outTuple = (IQueryTuple)iter.next();

	   rowidx++;
           if (rowidx >= strow && rowidx <= erow) {

            sb.append("<tr>");
            sb.append("<td>"+ rowid++);
            for (int j=1; j<=ncols; j++) {

               Object o1 = null;  
               try {
                  o1 = outTuple.getObject(j);  
                  if (o1 instanceof EJBLocalObject ) {
                     EJBLocalObject o2 = (EJBLocalObject) o1;  
                     String s1 = o2.toString();
//                     System.out.println(" s1 = "+s1);
                     int i1 = s1.lastIndexOf(".");
                     int i3 = s1.lastIndexOf("@");
                     int i4 = s1.lastIndexOf(")");

                     if (i1 != -1  && i3 != -1   && i4 != -1  && i1<i3 && i3<i4) {
                        String s2 = s1.substring(i1+1);
                        i4 = s2.lastIndexOf(")");
                        i3 = s2.lastIndexOf("@");
//                        System.out.println("i4 "+i4+" s2.length()"+s2.length()+" ");
                        if (i4 == s2.length()-1) {
                           int i2 = s2.lastIndexOf("Key");
                           if (i2 != -1 ) {
                              o1 = new String(s2.substring(0,i2) +"("+ s2.substring(i3+1,i4));
                           } else {
                              o1 = new String(s2.substring(0,i3) +"("+ s2.substring(i3+1,i4));
                           }
                        } else {
                           o1 = new String(s2);
                        }
                     }
                  }
               } catch (Exception ex) {
                  o1 = ex.getMessage();
               }
               sb.append("<td>"+ o1); 
            }//for
            sb.append("</tr>\n");
          }//if rowidx in range
         }//while
         sb.append("</table>");

	//put result title and row count on top
         sb.insert(0,(rowid -1)+ " row(s)\n");
         sb.insert(0,"<b>DynamicQuery Result:  </b>\n"+"<br>");
         
         return sb;

      } catch (Exception ex) {
         System.out.println(diag);
         ex.printStackTrace();
         //System.out.println(ex.toString());
         sb = new StringBuffer();
         //add enclosing table for spacing
         sb.append("<table><table cellpadding=5 cellspacing=1 border=0><tr>");
         sb.append("<td>"+"<B>DynamicQuery Error:  </B>"+"</td>"+"</tr>"+"<tr>");
         sb.append("<td>");
         sb.append("\n <TEXTAREA NAME=\"results\" COLS=\"80\" ROWS =\"3\" >" ); 
         sb.append(diag +"\n"+ ex.toString());
         sb.append("</TEXTAREA>");
         sb.append("</td>");
         sb.append("</tr>");
         sb.append("</table>");
         return sb;
      }

   }

   /*
    * This method looks up the com.ibm.websphere.ejbquery.QueryBean's remote interface, executes its create(), executes its prepareQuery(), and returns, if successful, a query plan, and if unsuccessful,  fills error with a html text area error message
    * @return java.lang.String
    * @param qstmt java.lang.String
    * @param error java.lang.StringBuffer
    */
   public String doPlan( String query, StringBuffer error) {

      String diag = "";
      String plan;

      try {
         diag="InitialContext";
         InitialContext ic = new InitialContext();
         diag = "lookup";

         Object homeObject = ic.lookup("com/ibm/websphere/ejbquery/Query");
         diag = "narrow";
         aJQSHome = (QueryHome)javax.rmi.PortableRemoteObject.narrow(homeObject,
                                                                     QueryHome.class);
         diag = "create";
         JQueryService = aJQSHome.create();
         diag = "prepareQuery";
         plan = JQueryService.prepareQuery( query, parms, null );
         return plan;

      } catch (Exception ex) {

          //add enclosing table for spacing
         error.append("<table><table cellpadding=5 cellspacing=1 border=0><tr>");
         error.append("<td>"+"<B>DynamicQuery Error:  </B>"+"</td>"+"</tr>"+"<tr>");
         error.append("<td>");
         error.append("\n <TEXTAREA NAME=\"results\" COLS=\"80\" ROWS =\"3\" >" ); 
         error.append(" System Exception in doPlan::"+diag+ " msg="+ex.toString());
//         error.append(diag +"\n"+ ex.toString());
         error.append("</TEXTAREA>");
         error.append("</td>");
         error.append("</tr>");
         error.append("</table>");
         return null; 
      }

   }

   /*
    * This method looks up the com.ibm.websphere.ejbquery.QueryBean's local interface, executes its create(), executes its prepareQuery(), and returns, if successful, a query plan, and if unsuccessful,  fills error with a html text area error message
    * @return java.lang.String
    * @param qstmt java.lang.String
    * @param error java.lang.StringBuffer
    */
   public String doLocalPlan( String query, StringBuffer error) {

      String diag = "";
      String plan;

      try {
         diag="InitialContext";
         InitialContext ic = new InitialContext();
         diag = "lookup";

         Object homeObject = ic.lookup("java:comp/env/ejb/LocalQuery");

         diag = "narrow";
         aLQSHome = (QueryLocalHome) homeObject;
         diag = "create";
         LQueryService = aLQSHome.create();
         diag = "prepareQuery";
         plan = LQueryService.prepareQuery( query, parms, null );
         return plan;

      } catch (Exception ex) {
          //add enclosing table for spacing
         error.append("<table><table cellpadding=5 cellspacing=1 border=0><tr>");
         error.append("<td>"+"<B>DynamicQuery Error:  </B>"+"</td>"+"</tr>"+"<tr>");
         error.append("<td>");
         error.append("\n <TEXTAREA NAME=\"results\" COLS=\"80\" ROWS =\"3\" >" ); 
         error.append(" System Exception in doLocalPlan::"+diag+ " msg="+ex.toString());
//         error.append(diag +"\n"+ ex.toString());
         error.append("</TEXTAREA>");
         error.append("</td>");
         error.append("</tr>");
         error.append("</table>");
         return null; 
      }

   }


   /**
    * This method provides additional information for specified error messages.
    * @return void
    * @param error java.lang.StringBuffer
    * @param origErr java.lang.String
    */
   private void augmentErrorMsg( StringBuffer sb, String origErr) {
	String lineSep = System.getProperty("line.separator");
      if (origErr.substring(0,9).equals("WQRY0037E")) {
   		sb.append(origErr);
		sb.append(" Probable Causes:"+lineSep);
		sb.append("  The server's \"Application classloader policy\" is not set to \"Single\""+lineSep);
		sb.append("  Method entered does not match Method Signature in EJB"+lineSep);
      } else {
   		sb.append(origErr);
      }
   }

   /**
    * This method changes the array of strings containing type value pairs  to an array of java objects of the specified type and having the specified value.
    * @return java.lang.Object[]
    * @param parmStringArray java.lang.String[]
    */
   public Object processSet( String[] parmStringArray) {                      
       String qstmt = null;
       int len = parmStringArray.length;
       Object[] temp = new Object[len];
       String diag = "";
      for (int k = 0; k < len; k++) {
         qstmt =  parmStringArray[k];
         //System.out.println("processSet() - parmStringArray[k] " + k + " = " + parmStringArray[k]);
      try {
	   if (qstmt.toLowerCase().startsWith("bigdecimal")) {
           qstmt = qstmt.substring(11).trim();
           temp[k] = new java.math.BigDecimal(qstmt);
           //System.out.println("processSet() - temp[k] " + k + " = " + (BigDecimal)temp[k]);
	   } else if (qstmt.toLowerCase().startsWith("boolean")) {
            qstmt = qstmt.substring(8).trim();
            temp[k] = Boolean.valueOf(qstmt);
            //System.out.println("processSet() - temp[k] " + k + " = " + (Boolean)temp[k]);
         } else if (qstmt.toLowerCase().startsWith("byte")) {
            qstmt = qstmt.substring(5).trim();
            temp[k] = Byte.valueOf(qstmt);
         } else if (qstmt.toLowerCase().startsWith("calendar")) {
            qstmt = qstmt.substring(9).trim();
            java.sql.Timestamp ts= new java.sql.Timestamp(java.sql.Timestamp.valueOf(qstmt).getTime() ) ;
            java.util.Calendar cd = Calendar.getInstance();
            cd.setTime(ts);
            temp[k]= cd;
         } else if (qstmt.toLowerCase().startsWith("character")) {
            qstmt = qstmt.substring(10).trim();
            if (qstmt.substring(0,1).equals("'") || qstmt.substring(0,1).equals("\"") ) {
               temp[k] = new Character(qstmt.charAt(1));
            }  else {
               temp[k] = new Character(qstmt.charAt(0));
            }
            //System.out.println("processSet() - temp[k] " + k + " = " + (Character)temp[k]);
         } else if (qstmt.toLowerCase().startsWith("char ")) {
            qstmt = qstmt.substring(5).trim();
            if (qstmt.substring(0,1).equals("'") || qstmt.substring(0,1).equals("\"") ) {
               temp[k] = new Character(qstmt.charAt(1));
            }  else {
               temp[k] = new Character(qstmt.charAt(0));
            }
            //System.out.println("processSet() - temp[k] " + k + " = " + (Character)temp[k]);
        } else if (qstmt.toLowerCase().startsWith("date")) {
           qstmt = qstmt.substring(5).trim();
           if (qstmt.indexOf("'") == 0 && qstmt.lastIndexOf("'") == qstmt.length() -1) {
		  qstmt = qstmt.substring(1,qstmt.length()-1);
	     }
           temp[k] = new java.sql.Date(java.sql.Date.valueOf(qstmt).getTime() );
       } else if (qstmt.toLowerCase().startsWith("double")) {
          qstmt = qstmt.substring(7).trim();
          temp[k] = Double.valueOf(qstmt);
        } else if (qstmt.toLowerCase().startsWith("float")) {
          qstmt = qstmt.substring(6).trim();
          temp[k] = Float.valueOf(qstmt);
        } else if (qstmt.toLowerCase().startsWith("integer")) {
           qstmt = qstmt.substring(8).trim();
           temp[k] = Integer.valueOf(qstmt);
           //System.out.println("processSet() - temp[k] " + k + " = " + (Integer)temp[k]);
        } else if (qstmt.toLowerCase().startsWith("long")) {
           qstmt = qstmt.substring(5).trim();
           temp[k] = Long.valueOf(qstmt);
        } else if (qstmt.toLowerCase().startsWith("short")) {
           qstmt = qstmt.substring(6).trim();
           temp[k] = Short.valueOf(qstmt);
        } else if (qstmt.toLowerCase().startsWith("string")) {
           qstmt=qstmt.substring(7);
           temp[k]=qstmt;
        } else if (qstmt.toLowerCase().startsWith("timestamp")) {
           qstmt = qstmt.substring(10).trim();
           if (qstmt.indexOf("'") == 0 && qstmt.lastIndexOf("'") == qstmt.length() -1) {
	       qstmt = qstmt.substring(1,qstmt.length()-1);
           }
           temp[k] = new java.sql.Timestamp(java.sql.Timestamp.valueOf(qstmt).getTime() );
        } else if (qstmt.toLowerCase().startsWith("time ")) {
           qstmt = qstmt.substring(5).trim();
           if (qstmt.indexOf("'") == 0 && qstmt.lastIndexOf("'") == qstmt.length() -1) {
	       qstmt = qstmt.substring(1,qstmt.length()-1);
           }
           temp[k] = java.sql.Time.valueOf(qstmt);
//  the following gives the same result
//         } else if (qstmt.toLowerCase().startsWith("time ")) {
//            qstmt = qstmt.substring(5).trim();
//           if (qstmt.indexOf("'") = 0 && qstmt.lastIndexOf("'") = qstmt.length() -1) {
//	       qstmt = qstmt.substring(1,qstmt.length()-1);
//           }
//            temp[k] = new java.sql.Time(java.sql.Time.valueOf(qstmt).getTime() );
        } else {
           System.out.println(" data type entered unknown");
           return null;
        }
         /*
        } catch (NamingException ex) {
             //System.out.println(ex.toString());
             error.append(" Naming Exception in doQuery::"+diag+ " msg="+ex.toString());
              //System.err.println(error);
         */
      } catch (Exception ex) {
         //System.out.println(ex.toString());
         System.out.println(" System Exception in doQuery::"+diag+ " msg="+ex.toString());
         //System.err.println(error);
      }

 //     System.out.println("processSet() - temp[k] = " ); //+ temp[k]);
      }  // for
      parms=temp;
      return parms;
   }

   /**
    * This method separates query part of the input string from parameter string part, saves the query part as an object attribute, and turns the parameter string into an array of strings each containing a TypeName value pair which it returns.  
    * The parameter string part of the input is surrounded by {} and contains TypeName, value pairs separated by commas.  
    * example input   select e from EmpBean e where e.dept.deptno = ?1 and e.isManager = ?2  {Integer 10,  boolean true}.
    * example query output   select e from EmpBean e where e.dept.deptno = ?1 and e.isManager = ?2.
    * example parameter output   Integer 10.
    * example parameter output   boolean true.
    * @return java.lang.String[]
    * @param osqlNormalized java.lang.String
    */

    public String[] parseInput(String osqlNormalized){

      //System.out.println("parseInput() - osqlNormalized = " +  osqlNormalized);
              int typeBegPos = osqlNormalized.indexOf("{");
              int typeEndPos = osqlNormalized.indexOf("}");
              if (typeBegPos > 0) {
                      query = new String(osqlNormalized.substring(0, typeBegPos));
              } else { //case of in_s_quote or in_d_quote or no terminator
                      query = new String(osqlNormalized);
              //        osqlNormalized = osqlNormalized + "{}"; //add brackets if not there
              }
              if (typeEndPos - typeBegPos > 1) {
                      String tas = new String(osqlNormalized.substring(typeBegPos + 1, typeEndPos));
                      StringTokenizer st = new StringTokenizer(tas, ",");
                      ArrayList aTypeArrayList = new ArrayList();
                      while (st.hasMoreTokens()) {
                              aTypeArrayList.add(st.nextToken());
                      }
                      _typeArray = new String[aTypeArrayList.size()];
                      for (int i = 0; i < aTypeArrayList.size(); i++) {
                              _typeArray[i] = ((String) aTypeArrayList.get(i)).trim();
                              //System.out.println("parseInput() - parmString = " + _typeArray[i]);
                      }
              }
              //System.out.println("parseInput() - query = " + query);
              return _typeArray;
   }
}

