//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.scheduler.accountreportejb;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;

/**
 * Bean implementation for the AccountReport entity bean
 * 
 * @see com.ibm.websphere.samples.scheduler.accountreportejb.AccountReportHome
 * @see com.ibm.websphere.samples.scheduler.accountreportejb.AccountReport
 */
public abstract class AccountReportBean implements EntityBean
{
    /**
     * The entity context associated with this bean instance
     */
    private EntityContext ejbContext;

    /**
     * Container invoked callback. 
     */
    public void ejbPostCreate(){
    }

    /**
     * Container invoked callback.
     * 
     * @param key    The key
     */
    public void ejbPostCreate(Long key){
    }

    /**
     * Container invoked callback.
     * 
     * @return The primary key for the row
     * @exception CreateException
     */
    public Long ejbCreate() throws CreateException{
        Long key = new Long(System.currentTimeMillis());
        setTotalBalance(new Double(0));
        return(ejbCreate(key));
    }

    /**
     * Container invoked callback.
     * 
     * @param key    The key
     * @return The primary key for the row
     * @exception CreateException
     */
    public Long ejbCreate(Long key) throws CreateException{
        setTimeReportGenerated(key);
        setTotalBalance(new Double(0));
        return(key);
    }

    /**
     * A container invokes this method when the instance is
     * taken out of the pool of available instances to become
     * associated with a specific EJB object.
     * 
     * @exception EJBException
     *                   Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbActivate() throws EJBException
    {
    }

    /**
     * A container invokes this method on an instance before
     * the instance becomes disassociated with a specific EJB object.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbPassivate() throws EJBException
    {
    }

    /**
     * A container invokes this method before it removes the EJB object
     * that is currently associated with the instance.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbRemove() throws EJBException
    {
    }

    /**
     * A container invokes this method to instruct the instance to
     * synchronize its state by loading it state from the underlying
     * database.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbLoad() throws EJBException
    {
    }

    /**
     * A container invokes this method to instruct the instance to
     * synchronize its state by storing it to the underlying database.
     * 
     * @exception EJBException
     *                   Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbStore() throws EJBException
    {
    }

    /**
     * Set the associated entity context.
     * 
     * @param ctx    An EntityContext interface for the instance. The instance should store the reference
     *               to the context in an instance variable.
     * @exception EJBException
     *                   Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void setEntityContext(EntityContext ctx) throws EJBException
    {
        ejbContext = ctx;
    }

    /**
     * Unset the associated entity context.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void unsetEntityContext() throws EJBException
    {
        ejbContext = null;
    }

    /**
     * Gets the total balance for this record.
     * 
     * @return The balance of all accounts at the time this report was generated.
     */
    public abstract Double getTotalBalance();

    /**
     * Sets the total balance for this record
     * 
     * @param totalBalance
     *               The new value of the balance
     */
    public abstract void setTotalBalance(Double totalBalance);

    /**
     * Gets the time this report was generated in milliseconds since
     * the epoch, GMT
     * 
     * @return time in milliseconds
     */
    public abstract Long getTimeReportGenerated();

    /**
     * Sets the time this report was generated. Time is in milliseconds
     * since the epoch, GMT
     * 
     * @param timeReportGenerated
     *               The new value for the time field
     */
    public abstract void setTimeReportGenerated(Long timeReportGenerated);
}
