//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.scheduler.accountreportejb;

import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import com.ibm.websphere.samples.scheduler.Utils;
import com.ibm.websphere.scheduler.TaskStatus;

/**
 * Implements the ReportHandler Enterprise Java Bean. The ReportHandler
 * EJB is a TaskHandler session bean which is the target of a scheduled
 * task.
 */
public class ReportGeneratorBean implements SessionBean
{
    /**
     * Accumulates the current account balances and inserts a row
     * into the report table.
     * 
     * This method is called whenever the scheduled task runs.
     * 
     * @param status The current status of the task. This parameter is ignored.
     */
    public void process(TaskStatus status){
        try {
            double totalBalance = 0;

            Object accountHomeObj = initialContext.lookup("java:comp/env/ejb/Account");
            AccountHome accountHome = (AccountHome) PortableRemoteObject.narrow(accountHomeObj, AccountHome.class);
            Collection accounts = accountHome.findAllAccounts();
            Iterator iterator = accounts.iterator();
            
            // Sum-up the account balance of all of the accounts
            while (iterator.hasNext()) 
            {
                Account account = (Account)iterator.next();
                totalBalance += account.getBalance().doubleValue();
            }

            //create a new report, using the current time as the primary key
            Object accountReportHomeObj = initialContext.lookup("java:comp/env/ejb/AccountReport");;
            AccountReportHome accountReportHome = (AccountReportHome) PortableRemoteObject.narrow(accountReportHomeObj, AccountReportHome.class);

            AccountReport report = accountReportHome.create(new Long(System.currentTimeMillis()));
            report.setTotalBalance(new Double(totalBalance));
        } catch (Exception e) {
            Utils.log(e);
        }
    }

    private Context initialContext;
    private SessionContext ejbContext;


    /**
     * A container invokes this method when it creates the EJB object     
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbCreate() throws CreateException
    {
        try {
            initialContext = new InitialContext();
        } catch (javax.naming.NamingException ne) {
            Utils.log(ne);
            throw new CreateException(ne.getMessage());
        }
    }


    /**
     * A container invokes this method when the instance is
     * taken out of the pool of available instances to become
     * associated with a specific EJB object.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbActivate()
    {
    }

    /**
     * A container invokes this method on an instance before
     * the instance becomes disassociated with a specific EJB object.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbPassivate()
    {
    }

    /**
     * A container invokes this method before it removes the EJB object
     * that is currently associated with the instance.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbRemove()
    {
    }

    /**
     * Set the associated session context.
     *
     * @param ctx A SessionContext interface for the instance. The instance should store the reference
     *            to the context in an instance variable.
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void setSessionContext(SessionContext ctx)
    {
        ejbContext = ctx;
    }
}
