//
// "This program may be used, executed, copied, modified and distributed without royalty for the 
// purpose of developing, using, marketing, or distributing."
//
// Product 5724i63,  (C) COPYRIGHT International Business Machines Corp., 2001,2004
// All Rights Reserved * Licensed Materials - Property of IBM
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
package com.ibm.websphere.samples.scheduler.accountreportejb;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import com.ibm.websphere.samples.scheduler.Utils;
import com.ibm.websphere.scheduler.BeanTaskInfo;
import com.ibm.websphere.scheduler.Scheduler;
import com.ibm.websphere.scheduler.SchedulerException;
import com.ibm.websphere.scheduler.TaskHandlerHome;

/**
 * The TaskCreatorBean class implements the TaskCreator Enterprise Java Bean.
 */
public class TaskCreatorBean implements SessionBean
{

    /**
     * Method scheduleTask.
     * @param startInterval
     * @param numberOfRepeats
     * @param repeatInterval
     * @throws SchedulerException
     */
    public void scheduleTask(String startInterval,
                             int numberOfRepeats,
                             String repeatInterval) throws SchedulerException
    {
        try {
            // Get the Scheduler
            Object schedulerObj = initialContext.lookup("java:comp/env/Scheduler");
            Scheduler scheduler = (Scheduler)javax.rmi.PortableRemoteObject.narrow(schedulerObj,Scheduler.class);

            // Get the handler which will be called when the task runs.
            // This EJB will use the home and remote interfaces that are provided with the Scheduler's
            // TaskHandler interfaces.  Because we have bound our ReportGenerator to the real EJB 
            // implementation in our bindings, we will get the correct implementation.
            Object reportGenHomeObj = initialContext.lookup("java:comp/env/ejb/ReportGenerator");
            TaskHandlerHome reportGenHome = (TaskHandlerHome) PortableRemoteObject.narrow(reportGenHomeObj,TaskHandlerHome.class);
            
            // Create our Schedule Task Info for our Report Generator Home EJB
            // Specify our UserCalendar EJB Home and the "seconds" calendar within it.
            BeanTaskInfo taskInfo = (BeanTaskInfo) scheduler.createTaskInfo(BeanTaskInfo.class);
            taskInfo.setTaskHandler(reportGenHome);
            taskInfo.setStartTimeInterval(startInterval);
            taskInfo.setRepeatInterval(repeatInterval);
            taskInfo.setNumberOfRepeats(numberOfRepeats);
            taskInfo.setUserCalendar("java:comp/env/ejb/SampleCalendar", "seconds");
            scheduler.create(taskInfo);            
        } 
        catch (javax.naming.NamingException ne) 
        {
            throw new EJBException(ne);
        } 
        catch (RemoteException re) 
        {
            throw new EJBException(re);
        }


    }

    private Context initialContext;
    private SessionContext ejbContext;


    /**
     * A container invokes this method when it creates the EJB object
     * 
     * @exception EJBException
     *                   Thrown by the method to indicate a failure caused by a system-level error.
     * @exception CreateException
     *                   thrown if the instance cannot be created
     */
    public void ejbCreate() throws CreateException
    {
        try {
            initialContext = new InitialContext();
        } catch (javax.naming.NamingException ne) {
            Utils.log(ne);
            throw new CreateException(ne.getMessage());
        }
    }


    /**
     * A container invokes this method when the instance is
     * taken out of the pool of available instances to become
     * associated with a specific EJB object.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbActivate()
    {
    }

    /**
     * A container invokes this method on an instance before
     * the instance becomes disassociated with a specific EJB object.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbPassivate()
    {
    }

    /**
     * A container invokes this method before it removes the EJB object
     * that is currently associated with the instance.
     *
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void ejbRemove()
    {
    }

    /**
     * Set the associated session context.
     *
     * @param ctx A SessionContext interface for the instance. The instance should store the reference
     *            to the context in an instance variable.
     * @exception EJBException Thrown by the method to indicate a failure caused by a system-level error.
     */
    public void setSessionContext(SessionContext ctx)
    {
        ejbContext = ctx;
    }
}
