//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. "
//
// Product 5630-A36,  (C) COPYRIGHT International Business Machines Corp., 2001, 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//

package com.ibm.websphere.samples.technologysamples.basiccalcclient;


import java.util.*;
import java.text.*;
//import java.io.*;
//import javax.naming.*;
//import javax.servlet.*;
//import javax.servlet.http.*;
//import java.rmi.RemoteException;

import com.ibm.websphere.samples.technologysamples.basiccalcclient.common.*;

public class BasicCalculatorClientJ2EEMain {

    /**
     * Displays a message to the user
     */
    private void printMessage(String message) {
        System.out.println(message);
    }

    /**
     * Displays standard program usage syntax.
     */
    private void printUsageSyntax() {
        printUsageSyntax(null);
    }

    /**
     * Displays standard program usage syntax with a user message.
     */
    private void printUsageSyntax(String message) {
        if(message == null) {
            message = "Incorrect syntax.";
        }
        
        printMessage(message);
        printMessage("  Correct syntax: <Launch Arguments> <Operation> Operand1 Operand2");
        printMessage("     <Operation>: add | subtract | multiply | divide");
        printMessage("  Example: launchClient.bat/sh TechnologySamples.ear -CCjar=BasicCalculatorClient.jar -CCBootstrapHost=MyServer add 2 3");
    }

    /**
     * Handles all of the user-interaction for the BasicCalculator EJB.
     * Takes an array of strings in the order {<Operation>,<Operand1>,<Operand2>}
     * and invokes the BasicCalculatorClient.calculate() method to perform the 
     * actual calculation.  The results of the calculation are returned in a 
     * BasicCalculatorClientResultBean object and are displayed to the user.
     */
    public void doCalculation(String[] args) {
        String operation;
        double operand1;
        double operand2;
        
		//get the number format for the default local
		NumberFormat numberFormat = NumberFormat.getNumberInstance( Locale.getDefault() );

        // Validate the arguments
        // Arguments are in the form:  Operation, Operand1, Operand2
        if (args.length != 3) {
            printUsageSyntax();
            return;
        }
        
        operation = args[0];
        try {

	        //retrieve operands respective of the locales
            operand1 = numberFormat.parse(args[1]).doubleValue();
            operand2 = numberFormat.parse(args[2]).doubleValue();
        }
        catch(ParseException pex)
        {
            printUsageSyntax("Parse exception on number input. Please enter a valid number.");
            return;
        }

        catch(NumberFormatException nfex)
        {
            printUsageSyntax("HEIDI Invalid number entered.  Please enter a valid number.");
            return;
        }

        // Create the BasicCalculatorClient object and the object to hold the result
        BasicCalculatorClientJ2EE bcc = new BasicCalculatorClientJ2EE();
        BasicCalculatorClientResultBean calcResult = null;
        
        try {
            // Initialize the BasicCalculatorClient object
            // This will find and create home interface
            bcc.init();

            // Call the calculate method
            calcResult = bcc.calculate(operation, (double)operand1, (double)operand2);

            // Print the result
            printMessage("Result: " + numberFormat.format( new Double (calcResult.getOperand1())) + calcResult.getOperation() + numberFormat.format(new Double(calcResult.getOperand2())) + " = " + numberFormat.format(new Double (calcResult.getResult())) );

        }
        catch(javax.naming.NamingException ne) {
            printMessage("Unable to initialize the BasicCalculatorClient object.");
            printMessage("The server may not be setup correctly.");
            ne.printStackTrace();
        }
        catch(BasicCalculatorClient.CalcException_DivideByZero dz) {
            printMessage("Division by zero is not a valid operation.");
        }
        catch(BasicCalculatorClient.CalcException_InvalidOperation io) {
            printUsageSyntax("Operation: " + operation + " is not a valid operation.");
        }
        catch(Throwable e) {
            printMessage("Unable to initialize the BasicCalculatorClient.  Refer to the following Stack Trace for details.");
            e.printStackTrace();
        }
    }

    /**
     * Entry point to the program used by the J2EE Application Client Container
     */
    public static void main(String[] args) {
        
        BasicCalculatorClientJ2EEMain calcclient = new BasicCalculatorClientJ2EEMain();
        calcclient.doCalculation(args);
        System.exit(0);
    }
}