//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. "
//
// Product 5630-A36,  (C) COPYRIGHT International Business Machines Corp., 2001, 2004
// All Rights Reserved * Licensed Materials - Property of IBM
// 

package com.ibm.websphere.samples.technologysamples.ejb.stateless.basiccalculatorwar;

import java.io.IOException;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.websphere.samples.technologysamples.ejb.stateless.basiccalculatorejb.BasicCalculator;
import com.ibm.websphere.samples.technologysamples.ejb.stateless.basiccalculatorejb.BasicCalculatorHome;

/**
 * Servlet that is used to process the incoming requests from the JSP, invoke an instance of the BasicCalculator EJB, and forward
 * the response back to the JSP.
 */
public class BasicCalculatorServlet extends HttpServlet {

   private BasicCalculator bc = null;
   private BasicCalculatorHome bcHome = null;

/**
 * Process incoming HTTP GET requests
 * 
 * @param req Object that encapsulates the request to the servlet
 * @param res Object that encapsulates the response from the servlet
 * @throws javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
 * @throws java.io.IOException Signals that an I/O exception of some sort has occurred.
 */
   public void doGet (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
      performTask(req, res);
   }

/**
 * Process incoming HTTP POST requests
 * 
 * @param req Object that encapsulates the request to the servlet
 * @param res Object that encapsulates the response from the servlet
 * @throws javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
 * @throws java.io.IOException Signals that an I/O exception of some sort has occurred.
 */
   public void doPost (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
      performTask(req, res);
   }

/**
 * Main service method for BasicCalculatorServlet
 * 
 * @param req Object that encapsulates the request to the servlet
 * @param res Object that encapsulates the response from the servlet
 * @throws javax.servlet.ServletException This exception is thrown to indicate a servlet problem.
 * @throws java.io.IOException Signals that an I/O exception of some sort has occurred.
 */
   public void performTask (HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
      String message = "";
      String operation = "";
      double operand1 = 0;
      double operand2 = 0;
      double result = 0; 


      // get the locale of the request
      Locale locale = req.getLocale();

      //get the number format for the default local
      NumberFormat numberFormat = NumberFormat.getNumberInstance(locale);

      try {

         operation = req.getParameter("operation");

         //retrieve operands respective of the locales
         operand1 = numberFormat.parse(req.getParameter("operand1")).doubleValue();
         operand2 = numberFormat.parse(req.getParameter("operand2")).doubleValue();

         InitialContext initCtx = new InitialContext( );
         Object homeObject = initCtx.lookup("java:comp/env/WSsamples/BasicCalculator");
         bcHome = (BasicCalculatorHome) javax.rmi.PortableRemoteObject.narrow(homeObject, BasicCalculatorHome.class);
         bc = bcHome.create( );

         if (operation.equals("add")) {
            result = bc.makeSum(operand1, operand2);
         }
         else if (operation.equals("subtract")) {
            result = bc.makeDifference(operand1, operand2);
         }
         else if (operation.equals("multiply")) {
            result = bc.makeProduct(operand1, operand2);
         }
         else if (operation.equals("divide")) {
            if (operand2 == 0) {
               message = "Division by 0 is not a valid operation";
            }
            else {
               result = bc.makeQuotient(operand1, operand2);
            }
         }

      }
      catch (NumberFormatException e) {
         message = "Not a valid number, please enter a valid number";
      }
      catch (ParseException e) {
         message = "Not a valid number based on the locale, please enter a valid number";
      }
      catch (NamingException e) {
         e.printStackTrace( );
         throw new ServletException(e);
      }
      catch (Exception e) {
         e.printStackTrace( );
         throw new ServletException(e);
      }

      // create the JSP Bean
      BasicCalculatorJspBean bcJspBean = new BasicCalculatorJspBean( );

      //set the result to match the locale
      bcJspBean.setResult(numberFormat.format((new Double(result))));

      //set the operands to match the locale
      bcJspBean.setOperand1(numberFormat.format((new Double (operand1))));
      bcJspBean.setOperand2(numberFormat.format((new Double (operand2))));

      bcJspBean.setOperation(operation);
      bcJspBean.setMessage(message);

      HttpSession session = req.getSession(true);
      session.setAttribute("BCJspBean", bcJspBean);

      RequestDispatcher rd = getServletContext( ).getRequestDispatcher("/BasicCalculator.jsp" );
      rd.forward(req, res);
   }
}