//
//"This sample program is provided AS IS and may be used, executed, copied and
//modified without royalty payment by customer (a) for its own instruction and
//study, (b) in order to develop applications designed to run with an IBM
//WebSphere product, either for customer's own internal use or for
//redistribution by customer, as part of such an application, in customer's
//own products. " 
//
//Product 5724i63, (C) COPYRIGHT International Business Machines Corp., 2008
//All Rights Reserved * Licensed Materials - Property of IBM
//

package com.ibm.websphere.samples.techsamp.security.jaas.jaasloginejb;

/**
 * Bean implementation class for Enterprise Bean: SampleBean.
 * The SampleBean class is a stateless session Enterprise JavaBean (EJB)
 * that returns the caller's security identity. The EJB is secured by a
 * J2EE security role.
 * 
 * @see com.ibm.websphere.samples.techsamp.security.jaas.jaasloginejb.SampleBean
 * @see com.ibm.websphere.samples.techsamp.security.jaas.jaasloginejb.SampleBeanHome
 */
public class SampleBeanBean implements javax.ejb.SessionBean {

    private static final long serialVersionUID = 200805121227L;
    private javax.ejb.SessionContext mySessionCtx;
    
    /**
     * Gets the associated session context.
     *
     * @return The context for this session.
     */
    public javax.ejb.SessionContext getSessionContext() {
        return mySessionCtx;
    }
    
    /**
     * Sets the associated session context.
     * The container calls this method after instance creation.
     *
     * @param ctx Specifies the context for this session.
     */
    public void setSessionContext(javax.ejb.SessionContext ctx) {
        mySessionCtx = ctx;
    }
    
    /**
     * Executed upon bean instance creation.
     * <p>
     * No action is required by the SampleBean bean.
     */
    public void ejbCreate() throws javax.ejb.CreateException {
    }
    
   /**
    * The activate method is called when the instance is activated
    * from its "passive" state. The instance should acquire any
    * resource that it has released earlier in the <code>ejbPassivate</code> method.
    * <p>
    * No action is required by the SampleBean bean.
    */
    public void ejbActivate() {
    }
    
    /**
     * The passivate method is called before the instance enters the
     * "passive" state. The instance should release any resources that
     * it can re-acquire later in the <code>ejbActivate</code> method.
     * <p>
     * No action is required by the SampleBean bean.
     */
    public void ejbPassivate() {
    }
    
   /**
    * A container invokes this method before it ends the life of
    * the session object. This happens as a result of a client
    * invoking a remove operation, or when a container decides to
    * terminate the session object after a timeout.
    * <p>
    * No action is required by the SampleBean bean.
    */
    public void ejbRemove() {
    }
    
    /**
     * Returns the caller's security identity.
     * 
     * @return the caller's security identity.
     * @throws RemoteException Thrown if the remote method call fails.
     */
    public String getCallerInfo() {
        // retrieve the caller's security identification information
        java.security.Principal principal = mySessionCtx.getCallerPrincipal();
        return principal.toString();
    }
}
