//
//"This sample program is provided AS IS and may be used, executed, copied and
//modified without royalty payment by customer (a) for its own instruction and
//study, (b) in order to develop applications designed to run with an IBM
//WebSphere product, either for customer's own internal use or for
//redistribution by customer, as part of such an application, in customer's
//own products. " 
//
//Product 5724i63, (C) COPYRIGHT International Business Machines Corp., 2004
//All Rights Reserved * Licensed Materials - Property of IBM
//

package com.ibm.websphere.samples.techsamp.security.jaas.jaasloginwar;

import java.io.IOException;
import java.security.Principal;
import java.security.PrivilegedActionException;
import java.util.*;

import javax.naming.InitialContext;
import javax.security.auth.login.LoginException;
import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// WebSphere security and JAAS extensions
import com.ibm.websphere.security.WSSecurityHelper;
import com.ibm.websphere.security.auth.callback.*;
import com.ibm.websphere.security.cred.WSCredential;

// Stateless session bean SampleBean
import com.ibm.websphere.samples.techsamp.security.jaas.jaasloginejb.*;

/**
 * This Servlet performs the server-side JAAS login. If the authentication is
 * successful, it invokes a J2EE role-based protected SampleBean session EJB
 * using the authenticated subject. The servlet retrieves all Principals and
 * public Credentials of the authenticated user, and sends the information 
 * to the dspCred.jsp page.
 * @version 	1.0
 * @author      IBM
 */
public class JAASLoginServlet extends HttpServlet {

    // Keys for the request parameters
    public final static String USERNAME = "username";
    public final static String PASSWORD = "password";

    // JAAS Login JNDI
    public final static String JAAS_LOGIN_JNDI = "java:comp/env/WSsamples/JAASLogin";

    // JAAS Login URLs
    public final static String DSP_CRED = "/dspCred.jsp";
    public final static String CONFIG_SEC = "/configSecurity.jsp";
    public final static String LOGINERROR = "/loginError.jsp";

    // request variables
    public final static String PRINCIPALS = "Principals"; 
    public final static String CREDENTIALS = "Credentials"; 
    public final static String EJB_MSG = "EJB_MSG";
    public final static String ERR_MSG = "ERR_MSG";

    // cache the EJB Home here
    private SampleBeanHome sampleBeanHome = null;

    /**
     * Initializes the servlet.
     *
     * @throws javax.servlet.ServletException Thrown if the servlet could not be
     *                                        initialized.
     */
    public void init() throws ServletException {
        super.init();
        javax.naming.InitialContext ic = null;
        try {
            ic = new InitialContext();
            Object homeObject = ic.lookup(JAAS_LOGIN_JNDI);
            sampleBeanHome =
                (SampleBeanHome) javax.rmi.PortableRemoteObject.narrow(
                    homeObject,
                    SampleBeanHome.class);
        } catch (Exception e) {
            e.printStackTrace(); 
            throw new ServletException("JAAS Sample SampleBean init failed: "
                + e.getMessage());
        }
    }

    /**
     * Sends HTTP GET requests to processRequest to do the work.
     * 
     * @param req  an object that encapsulates the request to the servlet.
     * @param resp an object that encapsulates the response from the servlet.
     * @throws javax.servlet.ServletException This exception is thrown to
     *                                        indicate a servlet problem.
     * @throws java.io.IOException            Signals that an I/O exception of
     *                                        some sort has occurred.
     */
    public void doGet(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException {
        this.doWork(req, resp);
    }
    
   /**
    * Sends HTTP POST requests to processRequest to do the work.
    *  
    * @param req  an object that encapsulates the request to the servlet.
    * @param resp an object that encapsulates the response from the servlet.
    * @throws javax.servlet.ServletException This exception is thrown to
    *                                        indicate a servlet problem.
    * @throws java.io.IOException            Signals that an I/O exception of
    *                                        some sort has occurred.
    */
    public void doPost(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException {
        this.doWork(req, resp);
    }

    /**
     * Performs server-side JAAS Authentication and Authorization.
     * 
     * The request parameters are defined as follows:
     *
     * <dl>
     *   <dt><b>username</b></dt>
     *   <dd>Specifies the user ID.</dd>
     *   <dt><b>password</b></dt>
     *   <dd>Specifies the password.</dd>
     * </dl>
     * 
     * @param req  an object that encapsulates the request to the servlet.
     * @param resp an object that encapsulates the response from the servlet.
     * @throws javax.servlet.ServletException This exception is thrown to
     *                                        indicate a servlet problem.
     * @throws java.io.IOException            Signals that an I/O exception
     *                                        of some sort has occurred.
     */
    private void doWork(HttpServletRequest req, HttpServletResponse resp)
        throws ServletException, IOException {

        // get the session from the request
        javax.servlet.http.HttpSession session = req.getSession(true);

        // get the ServletContext object 
        resp.setContentType("text/html"); 
        ServletContext ctx = getServletConfig().getServletContext();

        // If WebSphere global security is disabled. redirects the user to the
        // Configure and Run page to configure global security.	
        if (!com.ibm.websphere.security.WSSecurityHelper.isGlobalSecurityEnabled()) {
            // include the request to the configSecurity.jsp page 
            ctx.getRequestDispatcher(CONFIG_SEC).include(req, resp);
            return;
        }

        // read the parameters
        String username = req.getParameter(USERNAME);
        String password = req.getParameter(PASSWORD);

        // server-side JAAS login
        javax.security.auth.login.LoginContext lc = null;
        javax.security.auth.callback.CallbackHandler wscbh;

        try {
            // create the login context using WebSphere Application Server
            // callback handler
            wscbh = new WSCallbackHandlerImpl(username, password);
            lc = new javax.security.auth.login.LoginContext("WSLogin", wscbh);
        } catch (LoginException le) {
            System.out.println(
                "JAAS Sample Authentication failed"
                    + " - Cannot create LoginContext."
                    + le.getMessage());
            // store the error message in the session
            req.setAttribute(ERR_MSG, le.getMessage());
            ctx.getRequestDispatcher(LOGINERROR).include(req, resp);
            return;
        }

        if (lc != null) {

            try {
                // perform login
                lc.login();
            } catch (LoginException le) {
                Throwable root_exception = determineCause(le);
                System.out.println(
                    "JAAS Sample Authentication failed "
                        + "to create Subject.");
                req.setAttribute(ERR_MSG, root_exception.getMessage());
                ctx.getRequestDispatcher(LOGINERROR).include(req, resp);
                return;
            }

            // get the authenticated subject
            javax.security.auth.Subject userSubject = lc.getSubject();

            Iterator it = null;
            Vector tempPrincipals = null;     // to hold a list of Principals
            Vector tempCredentials = null;    // to hold a list of Credentials
            String ejbMessage = null;         // return message from EJB call
            String ejbErrMsg = null;          // error from EJB call	

            // loop through all Principals
            it = lc.getSubject().getPrincipals().iterator();
            tempPrincipals = new Vector();
            while (it.hasNext())
                tempPrincipals.add(it.next().toString());
            req.setAttribute(PRINCIPALS, tempPrincipals);

            // loop through all Credentials.
            it = lc.getSubject().getPublicCredentials(WSCredential.class)
                 .iterator();
            tempCredentials = new Vector();
            while (it.hasNext())
                tempCredentials.add(it.next().toString());
            req.setAttribute(CREDENTIALS, tempCredentials);
                
            try {
                // invoke a J2EE resource using the authenticated subject;
                // retrieve the security identification information about the
                // caller of the enterprise bean.
                ejbMessage = (String)com.ibm.websphere.security.auth
                             .WSSubject.doAs(userSubject,
                    new java.security.PrivilegedExceptionAction() {
                        private String message = null;
                        public Object run() throws Exception {
                            try {
                                SampleBean theBean = sampleBeanHome.create(); 
                                return theBean.getCallerInfo();
                            } catch (javax.ejb.CreateException ce) {
       	                        throw ce;
                            } catch (java.rmi.RemoteException re) {
                                throw re;
                            }
                        }
                    }
                );
                req.setAttribute(EJB_MSG, ejbMessage);
            } catch (java.lang.SecurityException e) {
                // thrown if therer is no doAs and other required permissions. 
                ejbErrMsg = "ERROR while executing doAs(), exception: "
                            + e.getMessage();
                req.setAttribute(ERR_MSG, ejbErrMsg);
                // include the request to the dspCred.jsp page 
                ctx.getRequestDispatcher(DSP_CRED).include(req, resp);
                return;
            } catch (PrivilegedActionException pe) {
                Exception e = pe.getException();
                ejbErrMsg = "ERROR while accessing EJB resource, exception: " 
                    + e.getMessage();
                req.setAttribute(ERR_MSG, ejbErrMsg);
                // include the request to the dspCred.jsp page 
                ctx.getRequestDispatcher(DSP_CRED).include(req, resp);
                return;
            } catch (Exception e) {
                ejbErrMsg = "ERROR: " + e.getMessage();
                req.setAttribute(ERR_MSG, ejbErrMsg);
                // include the request to the dspCred.jsp page 
                ctx.getRequestDispatcher(DSP_CRED).include(req, resp);
                return;
            }

            // perform logout 
            try {
                lc.logout();
            } catch (LoginException le) {
                le.printStackTrace(); 
                throw new ServletException("JAAS Sample logout failed: "
                    + le.getMessage());
            }
        } // end of lc != null

        // include the request to the dspCred.jsp page 
        ctx.getRequestDispatcher(DSP_CRED).include(req, resp);
    }
    
    /**
     * Method used to drill down into the WSLoginFailedException to find the 
     * "root cause" exception from a JAAS login.
     * 
     * @param  e an exception of type LoginException.
     * @return   the root cause login exception.
     */ 
    public Throwable determineCause(Throwable e) {
        Throwable root_exception = e, temp_exception = null;

        // keep looping until there are no more embedded WSLoginFailedException
        // or WSSecurityException exceptions
        while (true) {
            if (e instanceof com.ibm.websphere.security.auth.WSLoginFailedException) {
                temp_exception = ((com.ibm.websphere.security.auth.WSLoginFailedException)
                e).getCause();
            } else if (e instanceof com.ibm.websphere.security.WSSecurityException) {
                temp_exception = ((com.ibm.websphere.security.WSSecurityException)
                e).getCause();
            } else if (e instanceof javax.naming.NamingException) {
                // check for LDAP embedded exception
                temp_exception = ((javax.naming.NamingException)e).getRootCause();
            // your custom processing here, if necessary    
            // } else if (e instanceof your_custom_exception_here){
            //
            } else {
                // this exception is not one of the types we are looking for,
                // lets return now, this is the root from the WebSphere
                // Application Server perspective
                return root_exception;
            }

           if (temp_exception != null) {
               // we have an exception, let's go back an see if this has another
               // one embedded within it.
               root_exception = temp_exception;
               e = temp_exception;
               continue;
            } else {
               // we finally have the root exception from this call path, this
               // has to occur at some point
               return root_exception;
            }
        } // end of while
    } 
}
