/* "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
 * instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
 * or for redistribution by customer, as part of such an application, in customer's own products. " 
*/

package com.ibm.websphere.samples.technologysamples.sdo.AlbumCatalog;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.websphere.sdo.mediator.exception.MediatorException;
import com.ibm.websphere.sdo.mediator.jdbc.exception.OCCException;
import commonj.sdo.DataObject;

/**
 * AlbumCatalogServlet is the controller servlet of the sample:
 * <UL
 * 	<LI> It takes http requests from jsps to perform operations on the database via SDOs. </LI> 
 * 	<LI> It uses AlbumCatalogJDBCSDOService and SDO apis to retrieve and manipulate Datagraphs.</LI>
 * 	<LI> It places Datagraphs in the http session. Note that a new Datagraph is placed in the session everytime
 *   	<LI> a "findXX" operation or "add band" operation is done. </LI>
 * 	<LI> It dispatches requests to appropriate jsps         </LI>
 * </UL>
 */

public class AlbumCatalogServlet extends HttpServlet {

	
	// operations handled by the servlet 
	private static String ADDBAND_OP = "Add Band";
	private static String ADDALBUM_OP = "Add Albums";
	private static String CREATEALBUM_OP = "Create Album";	
	private static String CANCELALBUM_OP = "Cancel Albums";
	private static String COMMITALBUMS_OP = "Commit Albums";
	private static String ADDANOTHERALBUM_OP = "Add Another Album";
	private static String MODIFYALBUM_OP = "Modify Album";
	private static String FINDBAND_OP = "Find Bands";
	private static String FINDALBUM_OP = "Find Albums";
	private static String FINDALBUM_BANDANDYEAR_OP = "Find Albums By Band And Year";
	private static String UPDATE_OP = "Update Albums";
	private static String VIEWALBUMS_OP = "View Albums";
	private static String COMMITALBUMSDONE_OP = "CommitAlbumsDone";
	private static String REMOVEBANDS_OP = "Delete Bands";
	private static String REMOVEALBUMS_OP = "Delete Albums";

	// an instance of an AlbumCatalogJDBCSDOService that is used by all operations
	private AlbumCatalogJDBCSDOService albumSDOService;
	
	public void init()
	{
		// note that there is only one instance of the AlbumCatalogJDBCSDOService
		albumSDOService = AlbumCatalogJDBCSDOService.soleInstance();
	}
	
  
	public void doGet(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException {
		
		performTask(req, resp);
	}

	
	public void doPost(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException {
		
		performTask(req, resp);
	}
	
		
	private void performTask(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{		
		
                // to support languages that use doube byte character sets, change encoding to UTF-8
                try
                {
                    req.setCharacterEncoding("UTF-8");
                }
                catch(UnsupportedEncodingException e)
                {
                    throw new ServletException(e);
                }
            
                String operation = req.getParameter("operation");
	
		if(operation.equals(FINDBAND_OP))
		{
			findBand(req, resp);
		}
		else if(operation.equals(VIEWALBUMS_OP))
		{
			viewAlbums(req, resp);	
		}
		else if(operation.equals(FINDALBUM_OP))
		{
			findAlbum(req, resp);
		}
		else if(operation.equals(FINDALBUM_BANDANDYEAR_OP))
		{						
			findAlbumBandAndYear(req, resp);
		}
		else if(operation.equals(UPDATE_OP))
		{
			updateAlbumTracks(req, resp);
		}
		else if(operation.equals(ADDBAND_OP))
		{
			addBand(req, resp);	
		}
		else if(operation.equals(REMOVEBANDS_OP))
		{
			removeBands(req, resp);	
		}
		else if(operation.equals(ADDALBUM_OP))
		{
			addAlbums(req, resp, ADDALBUM_OP);	
		}
		else if(operation.equals(CREATEALBUM_OP))
		{
			createAlbum(req, resp);	
		}
		else if(operation.equals(CANCELALBUM_OP))
		{
			cancelAlbums(req, resp);	
		}
		else if(operation.equals(COMMITALBUMS_OP))
		{			
			commitBandWithAlbumsGraph(req, resp, COMMITALBUMSDONE_OP);	
		}
		else if(operation.equals(ADDANOTHERALBUM_OP))
		{			
			addAlbums(req, resp, ADDANOTHERALBUM_OP);	
		}
		else if(operation.equals(REMOVEALBUMS_OP))
		{
			removeAlbums(req, resp);	
		}

	}


	/**
	 * All findXXX methods call a method on AlbumCatalogJDBCSDOService to retrieve a Datagraph.
	 * They place the retrieved Datagraph in http session.
	 * Finally, they dispatch the request to the appropriate jsp, which displays the DataObjects in the graph.
	 * Note that the term "Datagraph" in this context is referring to the root DataObject returned by the mediator. A graph  
	 * data structure, in which every node is of type DataObject, is being referred to as a Datagraph. 
 	 */


	private void findBand(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		
		// removes session attributes from previous operations  
		removeSessionAttributes(req.getSession());
		
		String bandToFind = req.getParameter("bandName");				  
		
		// since wild cards (*) are allowed in the GUI, the (*) characters need to be replaced with SQL friendly (%) characters. 
		bandToFind = normalizeParameter(bandToFind);				
		
		try
		{						
			// use SDO service to perform query and get the resultant Datagraph     
			DataObject bandRootDO = albumSDOService.findBand(bandToFind);
									
			// place root DataObject of the graph in the session.
			req.getSession().setAttribute("bandRootDO", bandRootDO);								
						 
			req.setAttribute("lastOp", FINDBAND_OP);
				
			//	forward to jsp which displays bands
			dispatch("displayBands.jsp",req, resp);								
			
		}
		catch(MediatorException e)
		{
			e.printStackTrace();
			throw new ServletException(e.getMessage());			
		}
																			
	}

	private void findAlbum(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		// removes session attributes from previous operations  
		removeSessionAttributes(req.getSession());	
		
		String albumToFind = req.getParameter("albumName");
				
		//	since wild cards (*) are allowed in the GUI, the (*) characters need to be replaced with SQL friendly (%) characters.
		albumToFind = normalizeParameter(albumToFind);		
		
		try
		{
			//	use SDO service to perform query and get the resultant Datagraph
			DataObject albumRootDO = albumSDOService.findAlbum(albumToFind);						
					
			//	place root DataObject of the graph in the session.
			req.getSession().setAttribute("albumRootDO", albumRootDO);
			
			req.setAttribute("lastOp", FINDALBUM_OP);								
				
			//	forward to jsp which displays albums
			dispatch("displayAlbums.jsp",req, resp);		
		}														
		catch(MediatorException e)
		{
			e.printStackTrace();
			throw new ServletException(e.getMessage());
		}

	}

	private void findAlbumBandAndYear(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		// removes session attributes from previous operations  
		removeSessionAttributes(req.getSession());
				
		String bandName = req.getParameter("bandName");		
		String albumYear = req.getParameter("albumYear");
		
		//	since wild cards (*) are allowed in the GUI, the (*) characters need to be replaced with SQL friendly (%) characters.
		bandName  = normalizeParameter(bandName);
		albumYear = normalizeParameter(albumYear);
		
		try
		{
			//	use SDO service to perform query and get the resultant Datagraph
			DataObject albumRootDO = albumSDOService.findAlbumsByBandAndYear(bandName, albumYear);						
	
			//	since wild cards (*) are allowed in the GUI, the (*) characters need to be replaced with SQL friendly (%) characters.
			req.getSession().setAttribute("albumRootDO", albumRootDO);			
			
			req.setAttribute("lastOp", FINDALBUM_BANDANDYEAR_OP);								
					 
			// forward to jsp which displays albums
			dispatch("displayAlbums.jsp",req, resp);														
		}
		catch(MediatorException e)
		{
			e.printStackTrace();
			throw new ServletException(e.getMessage());
		}
		
	}



	/**
	 * This method marks the band, "selectedBandIndex", for which the related albums need to be viewed and then forwards to jsp that will
	 * display the albums  
	 *   
	 */	
	private void viewAlbums(HttpServletRequest req, HttpServletResponse resp) 
	{
		req.setAttribute("lastOp", VIEWALBUMS_OP);
		
		//the band for which to show albums 
		req.setAttribute("selectedBandIndex", req.getParameter("selectedBandIndex"));
		
		//	forward to jsp which displays related albums of selected band
		dispatch("displayBandAlbums.jsp",req, resp);		
	}



	/**
	 * This method updates all selected album DataObjects with new values specified in "displayAlbums.jsp".         
	 */	
	private void updateAlbumTracks(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{		
		
		// the "select" checkboxes in displayAlbums.jsp that were checked
		String[] albumsToUpdate = req.getParameterValues("selectedAlbumIndex");
		String[] albumTracksToUpdate = req.getParameterValues("recommendedTracks");
		
		// get the album root DataObject from session
		DataObject albumRootDO = (DataObject) req.getSession().getAttribute("albumRootDO");
		
		// get all the DataObjects for the Album table from the root DO. The albums were displayed in displayALbums.jsp,
		// in the same order that they appeared in this list. 
		List albums =  albumRootDO.getList(AlbumCatalogMetadata.ALBUMTABLE);
			
		for(int i = 0; i < albumsToUpdate.length; i++)
		{
			//	get index of the album DataObject, in the albums list, that a user selected for update
			int albumIndex = Integer.valueOf(albumsToUpdate[i]).intValue();
			
			// every object in the albums list is of type DataObject and represents a row in the Album table.
			DataObject albumToUpdate = (DataObject) albums.get(albumIndex);
			
			// get new tracks field specified in displayAlbums.jsp
			String newTracks = albumTracksToUpdate[albumIndex];

			// set the tracks field of the DataObject, i.e. the tracks field of the selected row in the Album table. 
			albumToUpdate.set(AlbumCatalogMetadata.ALBUM_TRACKS_FIELD, newTracks);					
			
		}
		
		// commit album graph to db
		commitAlbumGraph(req, resp, UPDATE_OP);
		
						
	}

	/**
	 * This method demonstrates creation of an empty Datagraph, populating it with DataObjects and
	 * then committing it to the database. 
	 */
	private void addBand(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		//	removes session attributes from previous operations
		removeSessionAttributes(req.getSession());
		
		String newBandName = req.getParameter("bandName");
		String newBandGenre = req.getParameter("bandGenre");

		try
		{
			//check if band already exists in db; if there is a Band DataObject in the resulting Datagraph, the band already exists in db.
			DataObject bandRootDataObject = albumSDOService.findBand(newBandName);
			
			// list of all DataObjects from the Band table in the graph.
			List bands = bandRootDataObject.getList(AlbumCatalogMetadata.BANDTABLE);
							 
			if(bands.size() != 0) // band exists
			{				
				String message = "<FONT color=RED> Band already exists in database </FONT>";
				req.setAttribute("message", message);
			}
			else
			{
				// add a DataObject for a row in the Band table.
				DataObject newBandDataObject = bandRootDataObject.createDataObject(AlbumCatalogMetadata.BANDTABLE);		
				
				// each column in the row is accessed by the name it was given in the relational metadata.
				newBandDataObject.set(AlbumCatalogMetadata.BAND_NAME_FIELD, newBandName);
				newBandDataObject.set(AlbumCatalogMetadata.BAND_GENRE_FIELD, newBandGenre);		
			
				//commit newly created Graph to db 
				albumSDOService.commitBandGraph(bandRootDataObject);
								
				req.getSession().setAttribute("bandRootDO", bandRootDataObject);			
				
				String successMessage = "<FONT color=blue> Band created successfully </FONT>";
				req.setAttribute("message", successMessage);
			}
			
			//req.setAttribute("lastOp", ADDBAND_OP);
			dispatch("displayBands.jsp",req, resp);
		}
		catch(MediatorException e)
		{
			e.printStackTrace();
			throw new ServletException(e.getMessage());
		}
		
		
	}
	

	/**
	 * This method demonstrates deletion of DataObjects, that have related DataObjects as well, from a Datagraph.
	 */	
	private void removeBands(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		// the "delete" checkboxes in displayBands.jsp that were checked
		String[] bandsToDelete = req.getParameterValues("bandIndexToDelete");		
		
		// get the band Datagraph 
		DataObject bandRootDO = (DataObject)  req.getSession().getAttribute("bandRootDO");		
		
		// List of DataObjects from the band table in the Datagraph
		List bands = bandRootDO.getList(AlbumCatalogMetadata.BANDTABLE);
		
		// its not possible to delete objects from a list that is being iterated over; therefore a copy is created. 
		List copyOfBands = new ArrayList(bands);
		
		for(int i = 0; i < bandsToDelete.length; i++)
		{
			// get index of the band DataObject that is selected for deletion 
			int bandIndex = Integer.valueOf(bandsToDelete[i]).intValue();
						
			DataObject bandToDelete = (DataObject) copyOfBands.get(bandIndex);			
			
			// since the band DataObjects have a relationship with album DataObjects, deleting a band logically implies that its
			// related albums should also be deleted. This has to be done by getting a copy of the list of related album DataObjects and 
			// deleting the DataObjects in that list.  Note that a band's related albums can be browsed with the BANDALBUM_REL relationship field 			
			List relatedAlbumDataObjects = bandToDelete.getList(AlbumCatalogMetadata.BANDALBUM_REL);
			List copyOfRelatedAlbumDataObjects = new ArrayList(relatedAlbumDataObjects);  
		
			for(int j =0; j < copyOfRelatedAlbumDataObjects.size(); j++)
			{
				DataObject album = (DataObject) copyOfRelatedAlbumDataObjects.get(j);				
				
				// delete album DataObject from Datagraph 
				album.delete();
			}
			
			// delete band DataObject from Datagraph
			bandToDelete.delete();	
		}
		
		
		// all DataObjects that were deleted in the Datagraph above will have their corresponding 
		// rows deleted from the database when the graph is commited.
		commitBandWithAlbumsGraph(req, resp, REMOVEBANDS_OP);
		
	}
	

	/**
	 * This method demonstrates disconnected manipulation of relational data by creating a partially populated DataObject, which 
	 * will be fully populated and then committed in a series of steps in the application. 
	 *
	 * When the Add Album button is clicked in displayBands.jsp, this method retrieves the band Datagraph from session and
	 *  creates a new partially populated band DataObject in the graph.
	 *  
	 * Then the request is forwarded to createAlbum.jsp, which displays the partially populated  DataObject and provides a form to 
	 * populate the rest of the fields of the DataObject. 
	 *
	 */	
	private void addAlbums(HttpServletRequest req, HttpServletResponse resp, String lastOp) throws ServletException
	{			
		
		// get band Datagraph from session
		DataObject bandRootDO = (DataObject) req.getSession().getAttribute("bandRootDO");				
		
		// get list of Band dataobjects in the graph
		List bands = bandRootDO.getList(AlbumCatalogMetadata.BANDTABLE);				
		
		// this is the index of band for which we want to add a related album 
		int selectedBandIndex;
		
		// since the user can add multiple albums using the "add another album" operation, we need to keep
		// track of which albums have been added so far to the Datagraph
		List newAlbumIndexes = (List) req.getSession().getAttribute("newAlbumIndexes");

		if(newAlbumIndexes == null)
			newAlbumIndexes = new ArrayList();

		
		if(lastOp.equals(ADDALBUM_OP))
		{			
			selectedBandIndex = Integer.valueOf(req.getParameter("selectedBandIndex")).intValue();
		}		
		else // lastOp was "add another album" operation
		{
			selectedBandIndex = ((Integer) req.getSession().getAttribute("selectedBandIndex")).intValue();  
		}
	
		
		// the band DataObject for which we want to add a new album
		DataObject selectedBand = (DataObject) bands.get(selectedBandIndex);		
					
		// create a new DataObject for the Album table. Note that the root DataObject, bandRootDO, has metadata for the Album table as well.
		DataObject newAlbum = bandRootDO.createDataObject(AlbumCatalogMetadata.ALBUMTABLE);
		
		// just set the BANDNAME field of the Album. 
		newAlbum.set(AlbumCatalogMetadata.ALBUM_BANDNAME_FIELD, selectedBand.get(AlbumCatalogMetadata.BAND_NAME_FIELD));				
		
		// add the new Album DataObject to the Band's relationship field
		selectedBand.getList(AlbumCatalogMetadata.BANDALBUM_REL).add(newAlbum);
		
		// this is the index of the album we added in the Album list of the Datagraph
		int indexOfAlbumToAdd = bandRootDO.getList(AlbumCatalogMetadata.ALBUMTABLE).size() - 1;
		newAlbumIndexes.add(new Integer(indexOfAlbumToAdd));
							
		// this information is used by addAlbum.jsp and commitAlbums.jsp to figure out which albums have already been created in the session
		req.getSession().setAttribute("selectedBandIndex", new Integer(selectedBandIndex));								
		req.getSession().setAttribute("indexOfAlbumToAdd", new Integer(indexOfAlbumToAdd));		
		req.getSession().setAttribute("newAlbumIndexes", newAlbumIndexes);
		
		req.setAttribute("lastOp", lastOp);				
				
		dispatch("addAlbum.jsp",req, resp);
	}

	/**
	 * Sets lastOp to CREATEALBUM_OP and dispatches to commitAlbums.jsp. This is called when the "createAlbum" button is clicked
	 * in addAlbum.jsp
	 * commitAlbums.jsp uses JSTL to fully populate the fields of the partially populated album DataObject we created above.
	 * When the Commit Albums button is clicked, the new album(s) are committed to the db       
	 *      
	 */	
	private void createAlbum(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{			
		req.setAttribute("lastOp", CREATEALBUM_OP);														
		
		dispatch("commitAlbums.jsp",req, resp);
																				
	}
	
	
	/**
	 * Deletes all album DataObjects that were added to the band Datagraph, but were not committed to the database.
	 * This is also a demonstration of the disconnected feature of SDOs.
	 * 
	 */		
	private void cancelAlbums(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		// get band Datagraph
		DataObject bandRootDO = (DataObject) req.getSession().getAttribute("bandRootDO");		
		
		// get band and album dataobjects in the graph
		List bands =  bandRootDO.getList(AlbumCatalogMetadata.BANDTABLE);
		List albums = bandRootDO.getList(AlbumCatalogMetadata.ALBUMTABLE);
		List copyOfAlbums = new ArrayList(albums);
		
		// the band for which to cancel the Albums
		Integer selectedBandIndex = (Integer)req.getSession().getAttribute("selectedBandIndex");
		DataObject selectedBand = (DataObject) bands.get(selectedBandIndex.intValue());
				
		// the indexes of albums that were added to the graph but were not committed to the database 
		List albumsToDelete = (List) req.getSession().getAttribute("newAlbumIndexes");
		
		for(int i=0; i < albumsToDelete.size(); i++)
		{
			Integer indexOfAlbumToDel = (Integer) albumsToDelete.get(i);
			
			DataObject albumToDelete = (DataObject) copyOfAlbums.get(indexOfAlbumToDel.intValue());
			albumToDelete.delete();	
		}
			
		// remove the list of albums that were previously added
		removeSessionAttributes(req.getSession());
		
		req.setAttribute("lastOp", CANCELALBUM_OP);														
		dispatch("displayBands.jsp",req, resp);				
	}


	/**
	 * Commits Datagraph that has band DataObjects and related Album DataObjects to the db
	 * 
	 */		
	private void commitBandWithAlbumsGraph(HttpServletRequest req, HttpServletResponse resp, String lastOp) throws ServletException
	{

		String message;

		try
		{
			// the band Datagraph
			DataObject bandRootDO = (DataObject) req.getSession().getAttribute("bandRootDO");
			albumSDOService.commitBandWithAlbumsGraph(bandRootDO);

			// remove sessoin attributes from previous operations
			removeSessionAttributes(req.getSession());

			if(lastOp.equals(REMOVEBANDS_OP))
				message = "<FONT color=blue> Band(s) deleted successfully </FONT>";
			else // commit albums op
				message = "<FONT color=blue> Album(s) committed successfully </FONT>";
		}
		catch(MediatorException e)
		{
			e.printStackTrace();
			message = "<FONT color=red> Problems encountered.  Album(s) not committed </FONT>";
		}

		req.setAttribute("lastOp", lastOp);		
		req.setAttribute("message", message);
		dispatch("displayBands.jsp",req, resp);

	}


	/**
	 * Commits Datagraph that has Album DataObjects to the db.
	 * Since Album table is the only table in the sample that can be updated via the Update Albums operation,
	 * the metadata of Album table was created with an OCC (optimistic concurrency control) column.
	 * Therefore, if during committing an album Datagraph, the update collides, an OCC Exception is caught,
	 * and related data for the first row on which the collision happened is displayed.
	 * 
	 */		
	private void commitAlbumGraph(HttpServletRequest req, HttpServletResponse resp, String lastOp) throws ServletException
	{
	
		try
		{
			// get Datagraph to commit
			DataObject albumGraph = (DataObject) req.getSession().getAttribute("albumRootDO");
			
			albumSDOService.commitAlbumGraph(albumGraph);

			String successMessage;
			
			if(lastOp.equals(UPDATE_OP))
				successMessage = "<FONT color=blue> Album(s) updated successfully </FONT>";
			else // delete albums op
				successMessage = "<FONT color=blue> Album(s) deleted successfully </FONT>";
								
			req.setAttribute("lastOp", lastOp);		
			req.setAttribute("message", successMessage);

			dispatch("displayAlbums.jsp", req, resp);
	
		}
		catch(MediatorException e)
		{
			
			if(e instanceof OCCException) 
			{
				
				// The update collided
				OCCException occE = (OCCException) e;

				// attempted data 
				DataObject changedDataObject =  occE.getChangedDO();
				
				// current data in db
				DataObject dbDataObject = occE.getDatabaseDO();		
				
				// original data
				DataObject originalDataObject = occE.getOriginalDO();				
				
				req.setAttribute("lastOp", lastOp);
				req.setAttribute("message", "updatecollided");
				req.setAttribute("changedDO", changedDataObject);
				req.setAttribute("dbDO", dbDataObject);
				req.setAttribute("originalDO", originalDataObject);
				
				dispatch("displayAlbumCollision.jsp", req, resp);
				
			}
			else
			{
				e.printStackTrace();			
				throw new ServletException(e.getMessage());
			}
		}

	}

	
	/**
	 * This method also demonstrates deletion of DataObjects. Look at removeBands for details on how to delete
	 * DataObjects.
	 * 
	 */			
	private void removeAlbums(HttpServletRequest req, HttpServletResponse resp) throws ServletException
	{
		// the "select" checkboxes in displayAlbums.jsp that were checked
		String[] albumsToDelete = req.getParameterValues("selectedAlbumIndex");
		
		// get the album DataObjects in session
		DataObject albumRootDO = (DataObject) req.getSession().getAttribute("albumRootDO");
		List albums =  albumRootDO.getList(AlbumCatalogMetadata.ALBUMTABLE);
		List copyOfAlbums = new ArrayList(albums);
				
		for(int i = 0; i < albumsToDelete.length; i++)
		{
			//	get index of the DataObject in albums list, that a user selected for deletion
			int albumIndex = Integer.valueOf(albumsToDelete[i]).intValue();
						
			DataObject albumToDelete = (DataObject) copyOfAlbums.get(albumIndex);
			
			albumToDelete.delete();	
		}
		
		commitAlbumGraph(req, resp, REMOVEALBUMS_OP);

	}

	/**
	 * replaces (*) characters in string passed in with (%) characters
	 * 
	 */
	private String normalizeParameter(String parameter)
	{
		
		return parameter.replace('*', '%');
	}

	/**
	 * removes attributes that were used in the Add Album/Add Another Album operations to keep track of which
	 * album DataObject were added.
	 * 
	 */	
	private void removeSessionAttributes(HttpSession session)
	{
		session.removeAttribute("selectedBandIndex");
		session.removeAttribute("indexOfAlbumToAdd");		
		session.removeAttribute("newAlbumIndexes");
		
	}
	
	
	/**
	 * dispatches a request to the specified page. 
	 */
	private void dispatch(String page, HttpServletRequest req, HttpServletResponse resp) 
	{
		try
		{
			getServletContext().getRequestDispatcher(page).forward(req, resp);
		}
		catch(Exception e)
		{
			e.printStackTrace();			
		}
		
	}

	
}
